/*
This software module was originally developed by AT&T
and editted by 
Yasuhiro Toguri (Sony)
in the course of development of MPEG-4. 
This software module is an implementation of a part of one or 
more MPEG-4 tools as specified by MPEG-4. 
ISO/IEC gives users of MPEG-4 free license to this
software module or modifications thereof for use in hardware 
or software products claiming conformance to MPEG-4.
Those intending to use this software module in hardware or software
products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company,
the subsequent editors and their companies, and ISO/IEC have no
liability for use of this software module or modifications thereof
in an implementation.
Copyright is not released for non MPEG-4 conforming
products. AT&T. retains full right to use the code for its own
purpose, assign or donate the code to a third party and to
inhibit third parties from using the code for non 
MPEG-4 conforming products.
This copyright notice must be included in all copies or
derivative works. Copyright (c) 1999.
*/

/* April-2005 modified by Chan Ti Eu (Institute of Infocomm Research, A*STAR) for */
/* multiple SLS elementary stream support										  */

/* AAC ADIF file routines */

#define DEBUG				fprintf
#define DEBUG_BUFFER1			nofprintf
#define DEBUG_BUFFER2			nofprintf

#include <stdio.h>
#include <string.h>
#include "MP4Movies.h"
#include "adif.h"
#include "all.h"
#include "port.h"
#include "lit_ll_dec.h"

/* for AudioSpecificInfo */
#define	lenAudioObjectType		5
#define lenSamplingFrequencyIndex	4
#define lenSamplingFrequency		24
#define lenChannelConfiguration		4
#define lenFrameLengthFlag	    	1
#define	lenDependsOnCoreCoder		1
#define lenExtensionFlag		1
#define	lenASIHeader			(5+4+0+4+1+1+1)

/* AAC parameters */
#define	bufferSizePerAudioChn		6144
#define aacFrameLength			1024	    /* AAC Main, LC, SSR only! */

#ifdef	BIGDECODER
#define MAX_FRAME_LEN			(64*1024)   /* bytes */
#else
#define MAX_FRAME_LEN			8192	    /* bytes */
#endif


#define LL_MAX_FRAME_LEN			(64*1024)   /* bytes */
extern	int 		ll_adts_header_present;
extern	ADTS_Header	ll_adts_header;


FILE	*fp;
FILE    *ll_fp;

u8	block_buffer[MAX_FRAME_LEN];
u8	*bp = block_buffer;
u32	bword, bbit; /* for buffer i/o */

u32	ll_bword, ll_bbit; /* for ll_buffer i/o */
u8	ll_block_buffer[LL_MAX_FRAME_LEN];
u8	*ll_bp = ll_block_buffer;
u8  ll_ext_block_buffer[MAX_LLE_TRACKS][LL_MAX_FRAME_LEN];
//u8  *ll_ext_bp = ll_ext_block_buffer[0];
int	ll_ext_bp[MAX_LLE_TRACKS]; 
extern int lle_layer_maxbytes[32];
extern LIT_LL_Info * ll_info[Chans];

long	buffer_fullness;

int char_count = 0;
int ll_char_count = 0;

void	init_dec(void);
void	dec(void);

int get_stream_len(int counter, int layer_num, LIT_LL_Info * ll_info);

u32 sampling_rate[] = 
{
	96000, 
	88200, 
	64000, 
	48000, 
	44100, 
	32000, 
	24000, 
	22050, 
	16000, 
	12000, 
	11025, 
	8000,  
	0,
	0,
	0,
	0
};

void nofprintf(FILE* fp, ...) {;}

void myexit(char *s)
{
    if (s)
        fprintf(stderr,"\n%s\n", s);
    exit(s ? 1 : 0);
}

void reset_input()
{
  bp = block_buffer;
  fseek(fp, 0, SEEK_SET);
  bword=bbit=nbits=cword=0;
}

void ll_reset_input()
{
  ll_bp = ll_block_buffer;
  fseek(ll_fp, 0, SEEK_SET);  
  ll_bword=ll_bbit=ll_nbits=ll_cword=0;
}

void ll_skip_lle_header()
{
  ll_getbits(2+3,0); 
  ll_byte_align(0); //skip 1st byte of LLE header
}

/* get byte and also copy it into block buffer */
int getbyte_aac(void)
{
    u8 c;

    if ((fread(&c, sizeof(char), 1, fp)) != 1) {
	DEBUG(stderr, "end of file\n");
	c = -1;
    }
    *bp++ = c;
	char_count++;
    return c; 

}

int getbyte_aaz(void)
{
    u8 c;

    if ((fread(&c, sizeof(char), 1, ll_fp)) != 1) {
	DEBUG(stderr, "end of aaz file\n");
	c = -1;
    }
    *ll_bp++ = c;
	ll_char_count++;
    return c; 

}

int	(*getbyte)(void)=&getbyte_aac;
int	(*ll_getbyte)(void)=&getbyte_aaz;

long getshort(void)
{
    int c1, c2;

    c1 = (*getbyte)();
    c2 = (*getbyte)();
    if(c2 < 0)
        c2 = 0;
    if(c1 < 0)
        c1 = 0;
    return (c1<<8) | c2;
}

long
getbits(int n)
{
  if(nbits <= 16){
    cword = (cword<<16) | getshort();
    nbits += 16;
  }

  nbits -= n;
  return (cword>>nbits) & ((1<<n)-1);
}

long ll_getshort(void)
{
    int c1, c2;

    c1 = (*ll_getbyte)();
    c2 = (*ll_getbyte)();
    if(c2 < 0)
        c2 = 0;
    if(c1 < 0)
        c1 = 0;
    return (c1<<8) | c2;
}

long
ll_getbits(int n)
{
  if(ll_nbits <= 16){
    ll_cword = (ll_cword<<16) | ll_getshort(n);
    ll_nbits += 16;
  }

  ll_nbits -= n;
  return (ll_cword>>ll_nbits) & ((1<<n)-1);
}

/*long getlongbits(int n)
{
    long v1, v2;
    if (n > 16) {
	v1 = getbits(8);
	v2 = getbits(n-8);
	v2 = (v1<<(n-8)) | v2;
    }
    else {
	v2 = getbits(n);
    }
    return v2;
    }*/

/* writes n bits from data (msb first) to **pd if pd is not NULL
 * otherwise does nothing
 */
static void putbits( char **pd, u32 data, u32 n)
{
        u32 dbit;
        u32 word, bit;
	char *d;

	if (pd == NULL) {
		d = NULL;
	}
	else {
		d = *pd;
	}
        dbit = 1<<(n-1);
        word = bword;
        bit = bbit;
        while(dbit != 0) {
                if(data & dbit) {
                        word |= bit;
                }
                dbit >>= 1;
                bit >>= 1;
                if(bit == 0) {
			if (d)
				*d++ = word;
                        word = 0;
                        bit = 0x80;
                }
        }
        bword = word;
        bbit = bit;
	if (d)
	    *pd = d;
}

static u32 num_audio_chn( EleList *p, int cce )
{
	u32 n;
	int i, j;
	
	n = 0;
	for (i=0; i<p->num_ele; i++) {
		j = p->ele_is_cpe[i];
		if (!cce)
			n += (j) ? 2 : 1;
		else 
			n += (j) ? 1 : 0;	/* ind sw cce */
	}
	return n;
}

static void put_ele_list( char **pb, EleList *p, int flag )
{
	int i;
	for (i=0; i<p->num_ele; i++) {
		if (flag)
			putbits(pb, p->ele_is_cpe[i], LEN_ELE_IS_CPE);
		/* tag must agree with that in bitstream! */
		putbits(pb, p->ele_tag[i], LEN_TAG);
	}
}

static void put_prog_config( char **pb, int tag, ProgConfig *p )
{
	int i, j;
	
	putbits(pb, tag, LEN_TAG);
	putbits(pb, p->profile, LEN_PROFILE);
	putbits(pb, p->sampling_rate_idx, LEN_SAMP_IDX);
	putbits(pb, p->front.num_ele, LEN_NUM_ELE);
	putbits(pb, p->side.num_ele, LEN_NUM_ELE);
	putbits(pb, p->back.num_ele, LEN_NUM_ELE);
	putbits(pb, p->lfe.num_ele, LEN_NUM_LFE);
	putbits(pb, p->data.num_ele, LEN_NUM_DAT);
	putbits(pb, p->coupling.num_ele, LEN_NUM_ELE);
	putbits(pb, 0, LEN_MIX_PRES);   /* no mono mixdown */
	putbits(pb, 0, LEN_MIX_PRES);   /* no stereo mixdown */
	putbits(pb, 0, LEN_MIX_PRES);   /* no matrix mixdown */
	
	/* front, side, back sce or cpe */
	put_ele_list(pb, &p->front, 1);
	put_ele_list(pb, &p->side, 1);
	put_ele_list(pb, &p->back, 1);
	put_ele_list(pb, &p->lfe, 0);
	put_ele_list(pb, &p->data, 0);
	put_ele_list(pb, &p->coupling, 1);

	/* byte align */
	while (bbit != 0x80)
		putbits( pb, 0, 1);
	
	/* comment */
	j = strlen(p->comments);
	putbits(pb, j, LEN_COMMENT_BYTES);
	for (i=0; i<j; i++)
		putbits(pb, p->comments[i], 8);
}

/* AAC raw data block is always an integral number of bytes
 * and is byte aligned
 * returns byte count
 */
static u32 get_raw_data_block( char **p )
{
	int i, n;
	char *tp;

	if ((nbits & 3) != 0) {
		fprintf(stderr, "byte alignment error\n");
		return 0;
	}
	/* shift buffer, retaining bytes that are still in cword */
	n = (nbits>>3); /* unused bytes */
	tp = (char*)(bp - n);
	bp = block_buffer;
	for (i=0; i<n; i++)
		*bp++ = *tp++;
	
	/* parse this block */
	dec();
        bno++;
	byte_align();

	/* set pointer */
	*p = (char*)block_buffer;

	/* return byte count */	
	n = (nbits>>3); /* unused bytes */
	return (bp - block_buffer) - n;
}

static u32 ll_get_raw_data_block( char **p, int layer_num )
{
	int i, n;
	char *tp;
	int stream_len = 0;

	if ((ll_nbits & 3) != 0) {
		fprintf(stderr, "byte alignment error\n");
		return 0;
	}

	/* shift buffer, retaining bytes that are still in cword */
	n = (ll_nbits>>3); /* unused bytes */

	tp = (char*)(ll_bp - n);
	ll_bp = ll_block_buffer;
	for (i=0; i<n; i++)
		*ll_bp++ = *tp++;
	
	/* parse this block */
		ll_dec();
        bno++;
	ll_byte_align(0);

	/* set pointer */
	*p = (char*)ll_block_buffer;

	/* return byte count */	
	n = (ll_nbits>>3); /* unused bytes */
	return (ll_bp - ll_block_buffer) - n;
}

/* open file */
MP4Err openAdifFile( char *filename )
{
	if ((fp = fopen( filename, "rb" )) == NULL) {
		DEBUG(stderr, "open\n");
		return MP4FileNotFoundErr;
	}
	return MP4NoErr;
}

MP4Err openAazFile( char *filename )
{
	if ((ll_fp = fopen( filename, "rb" )) == NULL) {
		DEBUG(stderr, "Failed to open AAZ file.\n");
		return MP4FileNotFoundErr;
	}
	return MP4NoErr;
}

#ifdef ADTS

MP4Err getADTSinfo( MP4Handle decoderSpecificInfoH, u32 *timeScale, u8 *profile )
{
  char *dsi = (char*) *decoderSpecificInfoH, *plen;
  u32 n;
  MP4Err err;

  init_dec();
  dec();
  *timeScale = sampling_rate[adts_header.sampling_freq_idx];

  bbit = 0x80; /* byte align bit stream */
  bword = 0;

  /* tag is DecSpecificInfoTag, 14496-1, Sec 8.2.2.2, Table 1 */
  putbits( &dsi, 0x05, 8 );
  /* dummy sizeOfInstance, 14496-1, Sec 12.3.3 */
  plen = dsi;
  putbits( &dsi, 0, 8 );
  /* Audio Specific Configuration */
  *profile = adts_header.profile+1;
  putbits( &dsi, adts_header.profile+1, lenAudioObjectType ); /* For Main LC SSR LTP */
  putbits( &dsi, adts_header.sampling_freq_idx, lenSamplingFrequencyIndex );
  putbits( &dsi, adts_header.channel_config, lenChannelConfiguration );
  /* GA Specific Configurtaion */
  putbits( &dsi, adts_header.frame_length, lenFrameLengthFlag );
  putbits( &dsi, 0, lenDependsOnCoreCoder );
  putbits( &dsi, 0, lenExtensionFlag );

  if (!adts_header.channel_config) {  /* pce */
    put_prog_config( &dsi, 0, &prog_config);

    if ((bbit != 0x80) != 0) {
      DEBUG(stderr, "PCE write byte alignment error\n");
      return MP4BadParamErr;
    }
  }

  /* set exact handle size */
  n = dsi - *decoderSpecificInfoH;
  err = MP4SetHandleSize( decoderSpecificInfoH, n ); if (err) return err;

  /* set length of decoderSpecificInfoH part in header,
   * encoded as sizeOfInstance
   */
  if (n < 128) {
    *plen = n-2;
  } else {
    /* encoding of size >= 128 bytes currently not supported */
    DEBUG(stderr, "decoderSpecificInfo too large\n");
    return MP4BadParamErr;
  }

  reset_input();
  adts_header_present = 1;

  return MP4NoErr;
}

MP4Err ll_getADTSinfo( MP4Handle decoderSpecificInfoH, u32 *timeScale, u8 *profile, u8 i ,u32 lle_main_stream, u32 aac_core_present)
{
  char *dsi = (char*) *decoderSpecificInfoH, *plen;
  u32 n;
  MP4Err err;
  int type_pcm;
  int frameLength;

  type_pcm = ll_getbits(2,0); 
  frameLength = ll_getbits(3,0);
  ll_byte_align(0);

  ll_dec();
  *timeScale = sampling_rate[ll_adts_header.sampling_freq_idx];

  bbit = 0x80; /* byte align bit stream */
  bword = 0;

  /* tag is DecSpecificInfoTag, 14496-1, Sec 8.2.2.2, Table 1 */
  putbits( &dsi, 0x05, 8 );
  /* dummy sizeOfInstance, 14496-1, Sec 12.3.3 */
  plen = dsi;
  putbits( &dsi, 0, 8 );
  /* Audio Specific Configuration */
  *profile = ll_adts_header.profile+1;
  putbits( &dsi, /*AAZ Object Type*/29, lenAudioObjectType ); /* For Main LC SSR LTP */
  putbits( &dsi, ll_adts_header.sampling_freq_idx, lenSamplingFrequencyIndex );
  putbits( &dsi, ll_adts_header.channel_config, lenChannelConfiguration );

  /* SLS Specific Configuration */
  putbits( &dsi, type_pcm/*pcmWordLengthlength*/, 3 );
  putbits(&dsi,aac_core_present,1);
  putbits(&dsi,lle_main_stream,1);
  putbits(&dsi,0,1); /* reserved*/
  putbits( &dsi, frameLength, 3 );

  if (!ll_adts_header.channel_config) {  /* pce */
    put_prog_config( &dsi, 0, &prog_config);

    if ((bbit != 0x80) != 0) {
      DEBUG(stderr, "PCE write byte alignment error\n");
      return MP4BadParamErr;
    }
  }
  else
  putbits( &dsi, 0, 5);	//to finish off the last byte to correct 'n'

  /* set exact handle size */
  n = dsi - *decoderSpecificInfoH;
  err = MP4SetHandleSize( decoderSpecificInfoH, n ); if (err) return err;

  /* set length of decoderSpecificInfoH part in header,
   * encoded as sizeOfInstance
   */
  if (n < 128) {
    *plen = n-2;
  } else {
    /* encoding of size >= 128 bytes currently not supported */
    DEBUG(stderr, "decoderSpecificInfo too large\n");
    return MP4BadParamErr;
  }

  ll_reset_input();
  ll_adts_header_present = 1;

  return MP4NoErr;
}

#endif

int adif_id_present()
{
  char adif_id[4];
  fread(adif_id, sizeof(char), 4, fp);
  fseek(fp, 0, SEEK_SET);
  if(!strncmp(adif_id, "ADIF", 4)) return 1;
  return 0;
}

MP4Err getDecoderSpecificInfo( u32 prog, MP4Handle decoderSpecificInfoH )
{
  u32 i, n;
  char *dsi, *plen;
  ProgConfig *pc;
  MP4Err err;


  pc = &prog_config;
  current_program = prog; /* set desired program */
	
  /* initialize decoder */
  init_dec();

  if (!get_adif_header(0)) {
    DEBUG(stderr, "adif_header\n");
    return MP4BadParamErr;
  }

  init();        /* initialize data structures */

  if ((nbits & 7) != 0) {
    DEBUG(stderr, "PCE read byte alignment error\n");
    return MP4BadParamErr;
  }

  /* write DecoderSpecificInfo */	
  bbit = 0x80; /* byte align bit stream */
  bword = 0;
  dsi = (char *) *decoderSpecificInfoH;

  /* tag is DecSpecificInfoTag, 14496-1, Sec 8.2.2.2, Table 1*/
  putbits( &dsi, 0x05, 8 );
  /* dummy sizeOfInstance, 14496-1, Sec 12.3.3 */
  plen = dsi;
  putbits( &dsi, 0, 8 );
  /* Audio Specific Configuration */
  putbits( &dsi, pc->profile+1, lenAudioObjectType );/* For Main LC SSR LTP */
  putbits( &dsi, pc->sampling_rate_idx, lenSamplingFrequencyIndex );
  putbits( &dsi, 0, lenChannelConfiguration );
  /* GA Specific Configurtaion */
  putbits( &dsi, 0, lenFrameLengthFlag );
  putbits( &dsi, 0, lenDependsOnCoreCoder );
  putbits( &dsi, 0, lenExtensionFlag );
  /* pce */
  put_prog_config( &dsi, 0, pc);
  if ((bbit != 0x80) != 0) {
    DEBUG(stderr, "PCE write byte alignment error\n");
    return MP4BadParamErr;
  }

  /* set exact handle size */
  n = dsi - *decoderSpecificInfoH;
  err = MP4SetHandleSize( decoderSpecificInfoH, n ); if (err) return err;

  /* set length of decoderSpecificInfoH part in header,
   * encoded as sizeOfInstance
   */
  if (n < 128) {
    *plen = n-2;
  } else {
    /* encoding of size >= 128 bytes currently not supported */
    DEBUG(stderr, "decoderSpecificInfo too large\n");
    return MP4BadParamErr;
  }

  DEBUG_BUFFER1(stderr, "decoderSpecificInfo length %d\n", n);
  dsi = *decoderSpecificInfoH;
  for (i=0; i<n; i++)
    DEBUG_BUFFER2(stderr, "%3i 0x%02x\n", i, *dsi++); 

  return MP4NoErr;
}

MP4Err	getAudio_profileAndLevel( u8 *audio_profileAndLevel )
{
	ProgConfig *p;
	
	p = &prog_config;
	*audio_profileAndLevel = (u8) (p->profile);
	if (*audio_profileAndLevel <= SSR_Profile) {
		return MP4NoErr;
		   
	}
	else {
		DEBUG(stderr, "profile\n");
		return MP4BadParamErr;
	}
}

MP4Err	getSamplingRate( u32 *samplingRate )
{
	ProgConfig *p;
	
	p = &prog_config;
	*samplingRate = sampling_rate[p->sampling_rate_idx];
	if (*samplingRate != 0) {
		return MP4NoErr;
	}
	else {
		DEBUG(stderr, "samplingRate\n");
		return MP4BadParamErr;
	}
}

MP4Err	getDecoderBufferSize( u32 *decoderBufferSize )
{
	u32 nch;
	ProgConfig *p;
	
	p = &prog_config;
	nch  = num_audio_chn(&p->front, 0);
	nch += num_audio_chn(&p->side, 0);
	nch += num_audio_chn(&p->back, 0);
	nch += num_audio_chn(&p->coupling, 1);

#ifdef ADTS
        if (nch==0) { /* use implicit speaker-mapping */
          nch = adts_header.channel_config;
          if (nch == 7) nch++;
        }
#endif

	*decoderBufferSize = nch * bufferSizePerAudioChn;
	if ( (0 < nch) && (nch <= 48)) {
		return MP4NoErr;
	}
	else {
		DEBUG(stderr, "decoderBufferSize: %d channels\n", nch);
		return MP4BadParamErr;
	}
}

MP4Err	getBitRates( u32 *maxBitrate, u32 *avgBitrate )
{
	ADIF_Header *p;
	
	p = &adif_header;
	*maxBitrate = (u32) p->bitrate;
	*avgBitrate = (u32) p->bitrate;
	if ( *avgBitrate > 0 ) {
		return MP4NoErr;
	}
	else {
		DEBUG(stderr, "avgBitrate\n");
		return MP4BadParamErr;
	}
}

MP4Err	getSampleDuration( u32 *sampleDuration )
{
	*sampleDuration = aacFrameLength; 
	return MP4NoErr;
}

MP4Err getRawDataBlock( MP4Handle sampleDataH )
{
        unsigned int i; 
        int c;
	char *p1, *p2, *rdb;
	u32 frameLength;
	MP4Err err;

	err = MP4NoErr;
	
	/* check for EOF */
	if ((c = getc(fp)) == EOF) {
	    DEBUG(stderr, "End of bitstream\n");
	    return MP4EOF;
	}
	ungetc(c, fp);
	    
	frameLength = get_raw_data_block(&rdb);
	if ( (frameLength == 0) || (frameLength > MAX_FRAME_LEN) ) {
		DEBUG(stderr, "frameLength error: %d\n", frameLength);
		return MP4BadParamErr;
	}
		
	err = MP4SetHandleSize( sampleDataH, frameLength ); if (err) goto bail;
	p1 = *sampleDataH;
	p2 = rdb;
	DEBUG_BUFFER1(stderr, "Block length %d\n", frameLength);
	for (i=0; i<frameLength; i++) {
		DEBUG_BUFFER2(stderr, "%3i 0x%02x\n", i, *p2); 
		*p1++ = *p2++;
	}
bail:
	return MP4NoErr;
}

MP4Err ll_getRawDataBlock( MP4Handle *ll_sampleDataH, int layer_num )
{
        unsigned int i, j; 
        int c;
	char *p1, *p2, *rdb;
	u32 frameLength;
	u32 ll_frameLength[MAX_LLE_TRACKS] = {0};
	MP4Err ll_err;
	short stream_len;
	int lle_header_present[2] = {1, 1};

	ll_err = MP4NoErr;
	
	/* check for EOF */
	if ((c = getc(ll_fp)) == EOF) {
	    DEBUG(stderr, "End of AAZ bitstream\n");
	    return MP4EOF;
	}

	ungetc(c, ll_fp);
	    
	frameLength = ll_get_raw_data_block(&rdb, layer_num);
	if ( (frameLength == 0) || (frameLength > LL_MAX_FRAME_LEN) ) {
		DEBUG(stderr, "frameLength error: %d\n", frameLength);
		return MP4BadParamErr;
	}

	p2 = rdb;

	for (i = 0; i < layer_num; i++)
	ll_ext_bp[i] = /**ll_sampleDataH[i];*/ll_ext_block_buffer[i];	

	for (j = 0; j < 2 /*no of channels*/;j++)
	{
		for (i = 0; i < layer_num; i++)
		{
			int k;
			stream_len = get_stream_len(i+1, layer_num, ll_info[j+1]);
			
			*((char*)ll_ext_bp[i]) = (stream_len&0xff);
			ll_ext_bp[i]++; 
			*((char*)ll_ext_bp[i]) = (stream_len>>8);
			ll_ext_bp[i]++;

			ll_frameLength[i] = ll_frameLength[i] + stream_len + 2;

			if (lle_header_present[j])
			{
				p2++;
				p2++;
			}

			lle_header_present[j] = 0;

			for (k = 0; k < stream_len; k++)
			{
				*((char*)ll_ext_bp[i]) = *p2++;
				ll_ext_bp[i]++;
			}
		}
	}

	for (i = 0; i < layer_num; i++)
	ll_err = MP4SetHandleSize( ll_sampleDataH[i], ll_frameLength[i] ); if (ll_err) goto bail;

	for (j = 0; j < layer_num; j++)
	{
		p1 = *ll_sampleDataH[j];
		p2 = ll_ext_block_buffer[j];
	for (i=0; i<ll_frameLength[j]; i++) {
		DEBUG_BUFFER2(stderr, "%3i 0x%02x\n", i, *p2); 
		*p1++ = *p2++;
	}
	}

bail:
	return MP4NoErr;
}

int get_stream_len(int counter, int layer_num, LIT_LL_Info * ll_info)
{

	short stream_len;
	short stream_start;
	short stream_end;

	if (lle_layer_maxbytes[counter] <= (ll_info->stream_len))
	{
		stream_len = lle_layer_maxbytes[counter] - lle_layer_maxbytes[counter - 1];
		stream_start = lle_layer_maxbytes[counter - 1];
		if (counter == layer_num) {
			stream_end = ll_info->stream_len;
			stream_len = stream_end - stream_start;
		}
		else
			stream_end = lle_layer_maxbytes[counter];
		if (stream_len > ll_info->stream_len)
			stream_len = ll_info->stream_len;
		if (stream_len <= 0)
			stream_start = stream_end = stream_len = 0;
	}
	else if (lle_layer_maxbytes[counter - 1] < (ll_info->stream_len))
	{
		stream_start = lle_layer_maxbytes[counter - 1];
		stream_end = ll_info->stream_len;
		stream_len = stream_end - stream_start;
	}
	else
	{
		stream_start = stream_end = stream_len = 0;
	}

	return stream_len;
}

