/***********

This software module was originally developed by Dolby
Laboratories in the course of development of the MPEG-2 AAC/MPEG-4 
Audio standard ISO/IEC13818-7, 14496-1, 2 and 3. This software module is an implementation of a part
of one or more MPEG-2 AAC/MPEG-4 Audio tools as specified by the
MPEG-2 aac/MPEG-4 Audio standard. ISO/IEC  gives users of the
MPEG-2aac/MPEG-4 Audio standards free license to this software module
or modifications thereof for use in hardware or software products
claiming conformance to the MPEG-2 aac/MPEG-4 Audio  standards. Those
intending to use this software module in hardware or software products
are advised that this use may infringe existing patents. The original
developer of this software module, the subsequent
editors and their companies, and ISO/IEC have no liability for use of
this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-2 aac/MPEG-4
Audio conforming products. The original developer retains full right to
use the code for the developer's own purpose, assign or donate the code to a
third party and to inhibit third party from using the code for non
MPEG-2 aac/MPEG-4 Audio conforming products. This copyright notice
must be included in all copies or derivative works. Copyright 1996.  

***********/

#ifndef AAC_QC_H
#define AAC_QC_H

/******************************************************************** 
/ 
/ filename : aac_qc.h
/ project : MPEG-2 aac
/ author : SNL, Dolby Laboratories, Inc
/          CL, TI <lueck@ti.com>
           ADD, NDS <alberto@ndsuk.com>
	   JB, NDS <jbennett@ndsuk.com>
	   
/ date : November 11, 1996
/ contents/description : header file for aac_qc.c
/          
/ Changes:
/ 07-jun-97   CL  Added AACQuantInfo structure.
/ 03-dec-97   ADD & JB  Added reset group number to AACQuantInfo sruct.
/
*******************************************************************/

#include "interface.h"
#include "tf_main.h"
#include "bitstream.h"
#include "block.h"
#include "tns.h"            
#include "all.h"
#include "nok_ltp_common.h"
  
#ifdef __cplusplus
extern "C" {
#endif

/* assumptions for the first run of this quantizer */
#define DEBUG 1
#define CHANNEL  1
#define NUM_COEFF  BLOCK_LEN_LONG       /* now using BLOCK_LEN_LONG of block.h */
#define MAGIC_NUMBER  0.4054
#define MAX_QUANT 8192
#define SF_OFFSET 100

#define ABS(A) ((A) < 0 ? (-A) : (A))
#define sgn(A) ((A) > 0 ? (1) : (-1))
#define SFB_NUM_MAX MAX_SCFAC_BANDS     /* now using MAX_SCFAC_BANDS of tf_main.h */
#define MAX_SHORT_IN_LONG_BLOCK 8 /*taken from tf_main.h*/

/*********************************************************/
/* AACQuantInfo, info for AAC quantization and coding.   */
/*********************************************************/
typedef struct {
  int max_sfb;                          /* max_sfb, should = nr_of_sfb/num_window_groups */
  int nr_of_sfb;                        /* Number of scalefactor bands, interleaved */
  int spectralCount;                    /* Number of spectral data coefficients */
  byte block_type;	        /* Block type */      
  int scale_factor[SFB_NUM_MAX];        /* Scalefactor data array , interleaved */			
  int sfb_offset[250];                  /* Scalefactor spectral offset, interleaved */
  int sfb_offset_lossless_only_short[250];/* Scalefactor spectral offset for lossless-only */
  int book_vector[SFB_NUM_MAX];         /* Huffman codebook selected for each sf band */
  int data[5*NUM_COEFF];                /* Data of spectral bitstream elements, for each spectral pair, 
                                           5 elements are required: 1*(esc)+2*(sign)+2*(esc value)=5 */
  int len[5*NUM_COEFF];                 /* Lengths of spectral bitstream elements */
  int num_window_groups;                /* Number of window groups */
  int window_group_length
    [MAX_SHORT_IN_LONG_BLOCK];          /* Length (in windows) of each window group */
  int common_scalefac;                  /* Global gain */
  Window_shape window_shape;            /* Window shape parameter */
  BsBitBuffer *gcBitBuf;                /* bit buffer for packed gain_control_data() */
  int max_pred_sfb;                     /* max prediction band */
  short pred_global_flag;               /* Global prediction enable flag */
  int pred_sfb_flag[SFB_NUM_MAX];       /* Prediction enable flag for each scalefactor band */
  int reset_group_number;               /* Prediction reset group number */
/*  TNS_INFO* tnsInfo;  */                  /* Ptr to tns data */
  NOK_LT_PRED_STATUS *ltpInfo;          /* Prt to LTP data */
  float pns_sfb_nrg[SFB_NUM_MAX];
  int pns_sfb_flag[SFB_NUM_MAX];
#ifdef PULSE 
  int number_pulse;
  int pulse_start_sfb;
  int pulse_offset[4];
  int pulse_amp[4];
#endif

  int total_sfb;		/*  total sfb number */

} AACQuantInfo;


#ifdef __cplusplus
}
#endif

#endif
