/*
This software module was originally developed by Apple Computer, Inc.
in the course of development of MPEG-4. 
This software module is an implementation of a part of one or 
more MPEG-4 tools as specified by MPEG-4. 
ISO/IEC gives users of MPEG-4 free license to this
software module or modifications thereof for use in hardware 
or software products claiming conformance to MPEG-4.
Those intending to use this software module in hardware or software
products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company,
the subsequent editors and their companies, and ISO/IEC have no
liability for use of this software module or modifications thereof
in an implementation.
Copyright is not released for non MPEG-4 conforming
products. Apple Computer, Inc. retains full right to use the code for its own
purpose, assign or donate the code to a third party and to
inhibit third parties from using the code for non 
MPEG-4 conforming products.
This copyright notice must be included in all copies or
derivative works. Copyright (c) 1999.
*/

/* April-2005 modified by Chan Ti Eu (Institute of Infocomm Research, A*STAR) for */
/* multiple SLS elementary stream support										  */

#include <stdio.h>
#include <stdlib.h>
#include "MP4Movies.h"
#include "adif.h"
#include "all.h"

#define MAX_DSI_LEN			1024	    /* bytes */

#define DEBUG				fprintf

#ifdef I2R_LOSSLESS
int lle_layer_bitrate[32] = {0};
int lle_layer_maxbytes[32] = {0};
u32 sampling_freq = 48000;
u32 sampling_freq_index = 3;
u32 ll_sampling_freq_index = 3;
u32  frameLength = 0;
u32 core_present;
u8  osf = 1;
#endif

MP4Err createMyMovie( u32 prog, char *inFile, char *outFile
#ifdef I2R_LOSSLESS
,char *ll_inFile
,int num_lle_layers
#endif
					 );
MP4Err addMySamples( MP4Track trak, MP4Media media, MP4Handle sampleH );

int main( int argc, char **argv )
{
	u32 i, prog;
	char *inFile, *outFile;
#ifdef I2R_LOSSLESS
	char ll_inFile[128];
	char* para_filename;
	int para_file_present = 0;
	int num_lle_layers = 1;
#endif
	MP4Err err;
	/* set defaults */
        /*        debug['v']=1;
                  debug['n']=1;
                  debug['N']=1;
                  debug['a']=1;
                  debug['x']=1;*/
	prog = 0;

	if (argc < 3) {
		fprintf( stderr, "usage: %s [-p<n>] adif_file mp4_file\n", argv[0]);
		return MP4BadParamErr;
	}
	i = 1;
	if (argv[i][0] == '-') {
		if (argv[i][1] == 'p') {
			prog = atoi(&argv[i][2]);
			fprintf(stderr, "Program %d selected \n", prog);
		}
#ifdef I2R_LOSSLESS
		else if (argv[i][1] == 'c') {
			i++;
			para_filename = argv[i];
			para_file_present = 1;
		}
#endif
		i++;
	}
	inFile = argv[i++];

#ifdef I2R_LOSSLESS
	{
		strcpy(ll_inFile,argv[i-1]);
		strcpy(strrchr(ll_inFile,'.'),"\0");
		strcat(ll_inFile,".aaz");
	}

	outFile = argv[i];

if (para_file_present)
{
	char string[10];
	int k = 0, numread;
	FILE *para_file = NULL;

	if ( (para_file = fopen(para_filename, "r")) != NULL)
	{
		/*
		do	{
			numread = fread(&string[k], sizeof(char), 1, para_file);
			k++;
			if ((feof(para_file)) != 0) break;
		} while (string[k-1] != ' '); 
			string[k-1] = '\0';
			frameLength = atoi(string);
			k = 0;

		do	{
			numread = fread(&string[k], sizeof(char), 1, para_file);
			k++;
			if ((feof(para_file)) != 0) break;
		} while (string[k-1] != ' '); 
			string[k-1] = '\0';
			sampling_freq = atoi(string);
		*/
		fscanf(para_file,"%d %d %d",&core_present,&frameLength,&sampling_freq);
			switch (frameLength)
			{
				case 0:
					osf = 1;
					break;
				case 1:
					osf = 2;
					break;
				case 2:
					osf = 4;
					break;
			}

		while ((feof(para_file)) == 0) {
			/*
			k = 0;
			do	{
				numread = fread(&string[k], sizeof(char), 1, para_file);
				k++;
				if (numread == 0) break;
			} while (string[k-1] != ' ');
				string[k-1] = '\0';
				if (string[0] == '\0') break;
				lle_layer_bitrate[num_lle_layers] = atoi(string);
				*/
			fscanf(para_file,"%d ",&lle_layer_maxbytes[num_lle_layers]);
				lle_layer_maxbytes[num_lle_layers] = lle_layer_bitrate[num_lle_layers] * 1000 * 1024 / 48000 / 8 / 2;
				lle_layer_maxbytes[num_lle_layers] += lle_layer_maxbytes[num_lle_layers - 1]; 
				num_lle_layers++;
		} 
	}
	else 
	{
		printf("Warning: Unable to open config file, proceeding with default values.\n");
	}
}

	set_parameters(sampling_freq, osf, &sampling_freq_index, &ll_sampling_freq_index,core_present);

#endif

	err = createMyMovie( prog, inFile, outFile
#ifdef I2R_LOSSLESS
		, ll_inFile 
		, num_lle_layers
#endif
		);
	fprintf( stderr, "returns %d\n", err );
	return err;
}

MP4Err createMyMovie( u32 prog, char *inFile, char *outFile
#ifdef I2R_LOSSLESS
,char *ll_inFile
,int num_lle_layers
#endif
					 )
{
	MP4Err err;
	MP4Movie moov;
	MP4Track trak;
	MP4Media media;
	MP4Handle decoderSpecificInfoH;

	u8 OD_profileAndLevel;
	u8 scene_profileAndLevel;
	u8 audio_profileAndLevel;
	u8 visual_profileAndLevel;
	u8 graphics_profileAndLevel;
	u32 initialObjectDescriptorID;
	u32 timeScale;
	u64 mediaDuration;

#ifdef I2R_LOSSLESS
	int i;
	MP4Err ll_err[MAX_LLE_TRACKS];
	MP4Track ll_trak[MAX_LLE_TRACKS];
	MP4Media ll_media[MAX_LLE_TRACKS];
	MP4Handle ll_decoderSpecificInfoH[MAX_LLE_TRACKS];

	u8 ll_OD_profileAndLevel[MAX_LLE_TRACKS];
	u8 ll_scene_profileAndLevel[MAX_LLE_TRACKS];
	u8 ll_audio_profileAndLevel[MAX_LLE_TRACKS];
	u8 ll_visual_profileAndLevel[MAX_LLE_TRACKS];
	u8 ll_graphics_profileAndLevel[MAX_LLE_TRACKS];
	u32 ll_initialObjectDescriptorID[MAX_LLE_TRACKS];
	u32 ll_timeScale[MAX_LLE_TRACKS];
	u64 ll_mediaDuration[MAX_LLE_TRACKS];
#endif

	err = MP4NoErr;
	err = MP4NewHandle( 0, &decoderSpecificInfoH ); if (err) goto bail;
        err = MP4SetHandleSize( decoderSpecificInfoH, MAX_DSI_LEN ); if (err) goto bail;

	err = openAdifFile( inFile ); if (err) goto bail;
	
#ifdef I2R_LOSSLESS
 	for (i = 0; i < num_lle_layers; i++) {
	ll_err[i] = MP4NoErr;
	ll_err[i] = MP4NewHandle( 0, &ll_decoderSpecificInfoH[i] ); if (ll_err[i]) goto bail;
        ll_err[i] = MP4SetHandleSize( ll_decoderSpecificInfoH[i], MAX_DSI_LEN ); if (ll_err[i]) goto bail;
	}
	ll_err[0] = openAazFile( ll_inFile ); if (ll_err[0]) goto bail;
#endif
        if ( adif_id_present() == 1 ) {
          adif_header_present = 1;
#ifdef ADTS
          adts_header_present = 0;
#endif
          err = getDecoderSpecificInfo( prog, decoderSpecificInfoH ); if (err) goto bail;
          err = getAudio_profileAndLevel( &audio_profileAndLevel ); if (err) return err;
          err = getSamplingRate( &timeScale ); if (err) return err;

#ifdef I2R_LOSSLESS
		  for (i = 0; i < num_lle_layers; i++) {
          ll_err[i] = getDecoderSpecificInfo( prog, ll_decoderSpecificInfoH[i] ); if (ll_err[i]) goto bail;
          ll_err[i] = getAudio_profileAndLevel( &ll_audio_profileAndLevel[i] ); if (ll_err[i]) return ll_err[i];
          ll_err[i] = getSamplingRate( &ll_timeScale[i] ); if (ll_err[i]) return ll_err[i];
		  }
#endif
    
        } else {
#ifndef ADTS
          DEBUG(stderr, "adif_header\n");
          return MP4BadParamErr;
#else
          adif_header_present = 0;
          adts_header_present = 1;
          err = getADTSinfo( decoderSpecificInfoH, &timeScale, &audio_profileAndLevel ); if (err) goto bail;

#ifdef I2R_LOSSLESS
		  for (i = 0; i < num_lle_layers; i++) {
			  ll_err[i] = ll_getADTSinfo( ll_decoderSpecificInfoH[i], &ll_timeScale[i], &ll_audio_profileAndLevel[i], i ,(i==0)?1:0,core_present); if (ll_err[i]) goto bail;
		  }

		ll_skip_lle_header();
#endif

#endif
        }

	initialObjectDescriptorID = 1;
	OD_profileAndLevel = 0xff; /* none required */
	scene_profileAndLevel = 0xff; /* none required */
	visual_profileAndLevel = 0xff; /* none required */
	graphics_profileAndLevel = 0xff; /* none required */
	err = MP4NewMovie( &moov,
		initialObjectDescriptorID, 
		OD_profileAndLevel,
		scene_profileAndLevel,
		audio_profileAndLevel,
		visual_profileAndLevel,
		graphics_profileAndLevel ); if (err) goto bail;
	err = MP4NewMovieTrack( moov, MP4NewTrackIsAudio, &trak ); if (err) goto bail;
	err = MP4NewTrackMedia( trak, &media, MP4AudioHandlerType, timeScale, NULL ); if (err) goto bail;
	err = MP4BeginMediaEdits( media ); if (err) goto bail;
	err = addMySamples( trak, media, decoderSpecificInfoH ); if (err) goto bail;
	err = MP4EndMediaEdits( media ); if (err) goto bail;
	err = MP4GetMediaDuration( media, &mediaDuration ); if (err) goto bail;
	err = MP4InsertMediaIntoTrack( trak, 0, 0, mediaDuration, 1 ); if (err) goto bail;
        err = MP4SetTrackEnabled( trak, 1); if (err) goto bail;


#ifdef I2R_LOSSLESS
	
	for (i = 0; i < num_lle_layers; i++) {
	ll_initialObjectDescriptorID[i] = 1;
	ll_OD_profileAndLevel[i] = 0xff; /* none required */
	ll_scene_profileAndLevel[i] = 0xff; /* none required */
	ll_visual_profileAndLevel[i] = 0xff; /* none required */
	ll_graphics_profileAndLevel[i] = 0xff; /* none required */
	ll_err[i] = MP4NewMovieTrack( moov, MP4NewTrackIsAudio, &ll_trak[i] ); if (ll_err[i]) goto bail;
	ll_err[i] = MP4NewTrackMedia( ll_trak[i], &ll_media[i], MP4AudioHandlerType, ll_timeScale[i], NULL ); if (ll_err[i]) goto bail;
	ll_err[i] = MP4BeginMediaEdits( ll_media[i] ); if (ll_err[i]) goto bail;
	}


	ll_err[0] = ll_addMySamples( ll_trak, ll_media, ll_decoderSpecificInfoH, num_lle_layers); if (ll_err[0]) goto bail;


	for (i = 0; i < num_lle_layers; i++) {	
	ll_err[i] = MP4EndMediaEdits( ll_media[i] ); if (ll_err[i]) goto bail;
	ll_err[i] = MP4GetMediaDuration( ll_media[i], &ll_mediaDuration[i] ); if (ll_err[i]) goto bail;
	ll_err[i] = MP4InsertMediaIntoTrack( ll_trak[i], 0, 0, ll_mediaDuration[i], 1 ); if (ll_err[i]) goto bail;
        ll_err[i] = MP4SetTrackEnabled( ll_trak[i], 1); if (ll_err[i]) goto bail;
	}

#endif

	err = MP4WriteMovieToFile( moov, outFile ); if (err) goto bail;
	err = MP4DisposeMovie( moov ); if (err) goto bail;

bail:
	return err;
}

MP4Err addMySamples( MP4Track trak, MP4Media media, MP4Handle decoderSpecificInfoH )
{
	MP4Err err;
	MP4Handle sampleEntryH;
	MP4Handle sampleDurationH;
	MP4Handle sampleSizeH;
	MP4Handle sampleDataH;
 	u32 objectTypeIndication;
	u32 streamType;
	u32 decoderBufferSize=0;
	u32 maxBitrate=0;
	u32 avgBitrate=0;

	err = MP4SetMediaLanguage( media, "und" ); if (err) goto bail;
	objectTypeIndication = 0x40; /* mpeg-4 audio */
	streamType = 0x05; /* audio stream */
        if (adif_header_present) {
          err = getBitRates( &maxBitrate, &avgBitrate ); if (err) goto bail;
        } else {
          maxBitrate=avgBitrate=0;
        }
        err = getDecoderBufferSize( &decoderBufferSize ); if (err) goto bail;
	err = MP4NewHandle( 0, &sampleEntryH ); if (err) goto bail;
	err = MP4NewSampleDescription( trak, 
		sampleEntryH,
		1,
		objectTypeIndication,
		streamType,
		decoderBufferSize,
		maxBitrate,
		avgBitrate,
		decoderSpecificInfoH ); if (err) goto bail;

	err = MP4NewHandle( sizeof(u32), &sampleDurationH ); if (err) goto bail;
	err = getSampleDuration( (u32*) *sampleDurationH ); if (err) goto bail;
	err = MP4NewHandle( sizeof(u32), &sampleSizeH ); if (err) goto bail;
	err = MP4NewHandle( sizeof(u32), &sampleDataH ); if (err) goto bail;
	for ( ;; ) {
		err = getRawDataBlock( sampleDataH );
		if ( err ) {
			if ( err == MP4EOF )
				err = 0;
			break;
		}
		err = MP4GetHandleSize( sampleDataH, (u32*) *sampleSizeH ); if (err) goto bail;
		err = MP4AddMediaSamples( media, 
			sampleDataH,
			1,
			sampleDurationH,
			sampleSizeH, 
			sampleEntryH, 
			NULL, 
			NULL ); if (err) goto bail;
		if ( sampleEntryH ) {
			err = MP4DisposeHandle( sampleEntryH ); if (err) goto bail;
			sampleEntryH = NULL;
		}	
	}	
bail:
	return err;
}

#ifdef I2R_LOSSLESS
MP4Err ll_addMySamples( MP4Track *ll_trak, MP4Media *ll_media, MP4Handle *ll_decoderSpecificInfoH, int num_lle_layers )
{
	MP4Err ll_err[MAX_LLE_TRACKS];
	MP4Handle ll_sampleEntryH[MAX_LLE_TRACKS];
	MP4Handle ll_sampleDurationH[MAX_LLE_TRACKS];
	MP4Handle ll_sampleSizeH[MAX_LLE_TRACKS];
	MP4Handle ll_sampleDataH[MAX_LLE_TRACKS];
 	u32 ll_objectTypeIndication[MAX_LLE_TRACKS];
	u32 ll_streamType[MAX_LLE_TRACKS];
	u32 ll_decoderBufferSize[MAX_LLE_TRACKS]={0};
	u32 ll_maxBitrate[MAX_LLE_TRACKS]={0};
	u32 ll_avgBitrate[MAX_LLE_TRACKS]={0};
	int i=0;

	for (i = 0; i < num_lle_layers; i++) {
	ll_err[i] = MP4SetMediaLanguage( ll_media[i], "und" ); if (ll_err[i]) goto bail;
	ll_objectTypeIndication[i] = 0x40; /* mpeg-4 audio */
	ll_streamType[i] = 0x05; /* audio stream */
        if (adif_header_present) {
          ll_err[i] = getBitRates( &ll_maxBitrate[i], &ll_avgBitrate[i] ); if (ll_err[i]) goto bail;
        } else {
          ll_maxBitrate[i]=ll_avgBitrate[i]=lle_layer_bitrate[i+1] * 1000;//0;
        }
        ll_err[i] = getDecoderBufferSize( &ll_decoderBufferSize[i] ); if (ll_err[i]) goto bail;
	ll_err[i] = MP4NewHandle( 0, &ll_sampleEntryH[i] ); if (ll_err[i]) goto bail;
	ll_err[i] = MP4NewSampleDescription( ll_trak[i], 
		ll_sampleEntryH[i],
		1,
		ll_objectTypeIndication[i],
		ll_streamType[i],
		ll_decoderBufferSize[i],
		ll_maxBitrate[i],
		ll_avgBitrate[i],
		ll_decoderSpecificInfoH[i] ); if (ll_err[i]) goto bail;

	ll_err[i] = MP4NewHandle( sizeof(u32), &ll_sampleDurationH[i] ); if (ll_err[i]) goto bail;
	ll_err[i] = getSampleDuration( (u32*) *ll_sampleDurationH[i] ); if (ll_err[i]) goto bail;
	ll_err[i] = MP4NewHandle( sizeof(u32), &ll_sampleSizeH[i] ); if (ll_err[i]) goto bail;
	ll_err[i] = MP4NewHandle( sizeof(u32), &ll_sampleDataH[i] ); if (ll_err[i]) goto bail;
	ll_err[i] = MP4SetSampleDescriptionPriority(ll_sampleEntryH[i], i+1);
	}

	for ( ;; ) {

			ll_err[0] = ll_getRawDataBlock( ll_sampleDataH, num_lle_layers );
			if ( ll_err[0] ) {
				if ( ll_err[0] == MP4EOF )
					ll_err[0] = 0;
				break;
			}
	
		for (i = 0; i < num_lle_layers; i++) {
			ll_err[i] = MP4GetHandleSize( ll_sampleDataH[i], (u32*) *ll_sampleSizeH[i] ); if (ll_err[i]) goto bail;
			ll_err[i] = MP4AddMediaSamples( ll_media[i], 
				ll_sampleDataH[i],
				1,
				ll_sampleDurationH[i],
				ll_sampleSizeH[i], 
				ll_sampleEntryH[i], 
				NULL, 
				NULL ); if (ll_err[i]) goto bail;
			if ( ll_sampleEntryH[i] ) {
				ll_err[i] = MP4DisposeHandle( ll_sampleEntryH[i] ); if (ll_err[i]) goto bail;
				ll_sampleEntryH[i] = NULL;
			}
		}
	}	
bail:
	return ll_err[0];
}
#endif