/************************* MPEG-2 NBC Audio Decoder **************************
 *                                                                           *
"This software module was originally developed by 
AT&T, Dolby Laboratories, Fraunhofer Gesellschaft IIS 
and edited by
Yoshiaki Oikawa (Sony Corporation),
Mitsuyuki Hatanaka (Sony Corporation)
Yasuhiro Toguri (Sony Corporation)
in the course of development of the MPEG-2 NBC/MPEG-4 Audio standard ISO/IEC 13818-7, 
14496-1,2 and 3. This software module is an implementation of a part of one or more 
MPEG-2 NBC/MPEG-4 Audio tools as specified by the MPEG-2 NBC/MPEG-4 
Audio standard. ISO/IEC  gives users of the MPEG-2 NBC/MPEG-4 Audio 
standards free license to this software module or modifications thereof for use in 
hardware or software products claiming conformance to the MPEG-2 NBC/MPEG-4
Audio  standards. Those intending to use this software module in hardware or 
software products are advised that this use may infringe existing patents. 
The original developer of this software module and his/her company, the subsequent 
editors and their companies, and ISO/IEC have no liability for use of this software 
module or modifications thereof in an implementation. Copyright is not released for 
non MPEG-2 NBC/MPEG-4 Audio conforming products.The original developer
retains full right to use the code for his/her  own purpose, assign or donate the 
code to a third party and to inhibit third party from using the code for non 
MPEG-2 NBC/MPEG-4 Audio conforming products. This copyright notice must
be included in all copies or derivative works." 
Copyright(c)1996.
 *                                                                           *
 ****************************************************************************/

/* Oct-2003  modified by RSY,I2R for MPEG-4 audio scalable lossless (SLS) coding */

#include "lit_ll_dec.h"
#include "inv_int_mdct.h"
#include "all.h"
#include "monopred.h"
#include "port.h"
#include "nok_lt_prediction.h"
#include "lit_ms.h"

#ifdef BSAC
#include ".\BSAC\allHandles.h"
extern int codec_type;
extern int bsac_ch;
#endif
/* prediction */
static	PRED_STATUS	*sp_status[Chans];
/* long term prediction */
static  NOK_LT_PRED_STATUS *nok_lt_status[Chans];

/* all storage is global */
Float *coef[Chans], *data[Chans], *state[Chans];

byte hasmask[Winds], *mask[Winds], *group[Chans],
  wnd[Chans], max_sfb[Chans],
  *cb_map[Chans], d_bytes[MAX_DBYTES];
Wnd_Shape wnd_shape[Chans];
short *factors[Chans];
int *lpflag[Chans], *prstflag[Chans];
TNS_frame_info *tns[Chans];

double pcm_scaling[4] = {1.0, 1.0, 1.0/16, 1.0/256}; /* 8,16,20,24 bit PCM */

LIT_LL_quantInfo* ll_quantInfo[Chans];
LIT_LL_Info * ll_info[Chans];

int *ll_coef[Chans];
int *mdct_buf[Chans] ;
int *ll_timesig[Chans] ;
int type_pcm, frameLength;

static int     block_size_samples = 1024;  
int numSample,totNumSample;
int frameNumSample = 1024;
float **sampleBuf;
int nChan;
long ll_bits_to_use;
float fSample, ll_bitRate;
short msenable = 1;

int msStereoIntMDCTMode, start_offset;
  

#ifdef PLOTMTV
int frame; /* global for debugging */
#else
static  int frame;
#endif 
  

#ifdef BSAC
struct tagBsBitBufferStruct	/* bit buffer */
{
  unsigned char* data;          /* data bits */
  long           numBit;        /* number of bits in buffer */
  long           size;          /* buffer size in bits */
};

struct tagBsBitStreamStruct	/* bit stream */
{
  FILE *	file;		/* file or NULL for buffer i/o */
  int		write;		/* 0=read  1=write */
  long		streamId;	/* stream id (for debug) */
  char*		info;		/* info string (for read file) */
  char*         fileName;       
  HANDLE_BSBITBUFFER 	buffer[2];	/* bit buffers */
				/* (buffer[1] is only used for read file) */
  long		currentBit;	/* current bit position in bit stream */
				/* (i.e. number of bits read/written) */
  long		numByte;	/* number of bytes read/written (only file) */
};
#endif


void
init()
{
  huffbookinit();


#ifndef ADIF2MP4
#ifdef USELIB_ISOMP4
    type_pcm = mc_info.pcmWordLength;
#endif
#endif
  winmap[0] = win_seq_info[ONLY_LONG_WINDOW];
  winmap[1] = win_seq_info[ONLY_LONG_WINDOW];
  winmap[2] = win_seq_info[EIGHT_SHORT_WINDOW];
  winmap[3] = win_seq_info[ONLY_LONG_WINDOW];      
}

int
getdata(int *tag, int *dt_cnt, byte *data_bytes)
{
  int i, align_flag, cnt;

  *tag = getbits(LEN_TAG);
  align_flag = getbits(LEN_D_ALIGN);
  if ((cnt = getbits(LEN_D_CNT)) == (1<<LEN_D_CNT)-1)
    cnt +=  getbits(LEN_D_ESC);
  *dt_cnt = cnt;
  if (debug['x'])
    PRINT(SE, "data element %d has %d bytes\n", *tag, cnt);
  if (align_flag)
    byte_align();

  for (i=0; i<cnt; i++) {
    data_bytes[i] = getbits(LEN_BYTE);
    if (debug['X'])
      PRINT(SE, "%6d %2x\n", i, data_bytes[i]);
  }
    
  return 0;
}

int
extension_payload(int cnt, byte *data)
{
  int type, i;
    
  /* fill bytes should not emulate any EX types below! */
  type = getbits(LEN_EX_TYPE);
  if (debug['x'])
    PRINT(SE, "fill element extension %d\n", type);
    
  switch(type) {
  case EX_FILL_DATA:
    getbits(LEN_NIBBLE);
    for (i=0; i<cnt-1; i++)
      data[i] = getbits(LEN_BYTE);
    return cnt;
  default: 
    getbits(LEN_NIBBLE);
    for (i=0; i<cnt-1; i++)
      getbits(LEN_BYTE);
    return cnt;
  }
}

void
getfill(byte *data)
{
  int cnt;
  if ((cnt = getbits(LEN_F_CNT)) == (1<<LEN_F_CNT)-1)
    cnt +=  getbits(LEN_F_ESC) - 1;
  if (debug['x'])
    PRINT(SE, "fill element has %d bytes\n", cnt);
  while (cnt > 0)
    cnt -= extension_payload(cnt, data);
}

void
init_dec(void)
{
  int i, ch;

  /*****
        debug['N']=1;
        debug['v']=1;
        debug['f']=1;
        debug['s']=1;
        debug['r']=1;
  *****/

  for(i=0; i<Chans; i++){
    coef[i] = (Float *)mal1(MAX_OSF*LN2*sizeof(*coef[0]));
    ll_coef[i] = (int *)mal1(MAX_OSF*LN2*sizeof(*ll_coef[0]));
    mdct_buf[i] = (int *)mal1(MAX_OSF*2*LN2*sizeof(*mdct_buf[0]));
    ll_timesig[i] = (int *)mal1(MAX_OSF*LN2*sizeof(*ll_timesig[0]));
    ll_quantInfo[i] = (LIT_LL_quantInfo *) mal1(sizeof(*ll_quantInfo[0]));
    ll_quantInfo[i]->info = (Info *) mal1(sizeof(Info));
    ll_quantInfo[i]->quantCoef = (int *)mal1(MAX_OSF*LN2*sizeof(int));  
    ll_info[i] = (LIT_LL_Info *)mal1(sizeof(LIT_LL_Info));
    ll_info[i]->stream = (char *) malloc(LN2*4*MAX_OSF);
    sampleBuf=(float**)malloc(/*numChannel*/2*sizeof(float*));
    for (ch=0; ch<2/*numChannel*/; ch++)
      (sampleBuf[ch]=(float*)malloc(MAX_OSF*frameNumSample*sizeof(float))) ;
   
    ll_bits_to_use = (int)(ll_bitRate*frameNumSample/fSample+0.5);

    data[i] = (Float *)mal1(MAX_OSF*LN2*sizeof(*data[0]));	// I2R: included MAX_OSF 21/2/07
    state[i] = (Float *)mal1(LN*sizeof(*state[0]));	/* changed LN4 to LN 1/97 mfd */
    fltclr(state[i], LN);
    factors[i] = (short *)mal1(MAXBANDS*sizeof(*factors[0]));
    cb_map[i] = (byte *)mal1(MAXBANDS*sizeof(*cb_map[0]));
    group[i] = (byte *)mal1(NSHORT*sizeof(group[0]));
    lpflag[i] = (int *)mal1(MAXBANDS*sizeof(*lpflag[0]));
    prstflag[i] = (int *)mal1((LEN_PRED_RSTGRP+1)*sizeof(*prstflag[0]));
    tns[i] = (TNS_frame_info *)mal1(sizeof(*tns[0]));
    sp_status[i]  = (PRED_STATUS *)mal1(LN*sizeof(*sp_status[0]));
    wnd_shape[i].prev_bk = 0;
    nok_lt_status[i]  = (NOK_LT_PRED_STATUS *)mal1(sizeof(*nok_lt_status[0]));
  }
  for(i=0; i<Winds; i++) {
    mask[i] = (byte *)mal1(MAXBANDS*sizeof(mask[0]));
  }

  for(i = 0; i < Chans; i++){
    wnd_shape[i].prev_bk = 0;
    wnd_shape[i].this_bk = 0;
  }

}


void dec(int frame)
{    

  int msStereoIntMDCT = 0; 

  int ele_id, d_tag, d_cnt;
  MC_Info *mip = &mc_info;
  
  // I2R: added 28 May 07
  static int numChannels;

#ifdef   ADTS
  static int first_adts = 0;
#endif

  int i, ch, wn;
  int left, right;
  Ch_Info *cip;
#ifdef BSAC
  int ChannelOutNum = 0;
#endif
  if(debug['n'])
    PRINT(SE, "\rblock %ld  ", bno);
  if(debug['N'])
    PRINT(SE, "block %ld\n", bno);

#ifdef BSAC
  if (codec_type != 1)
  {
  if (mip->aac_core_present)
    byte_align();
  }
#endif

#ifdef   ADTS
  if(adts_header_present == 1){
#ifdef   ADIF2MP4
    extern char *bp,block_buffer[];
#endif

    get_adts_header();

#ifdef   ADIF2MP4
    /* cut away adts-header from bitstream, but beware of cword/nbits */
    bp = block_buffer;
    if (nbits > 24) *bp++=(char)(cword >> 24);
    if (nbits > 16) *bp++=(char)(cword >> 16);
    if (nbits >  8) *bp++=(char)(cword >>  8);
    /*if (nbits > 0)*/ *bp++=(char)cword;
#endif

    if(first_adts == 0){
      init();
      first_adts = 1;
    }
    byte_align();
  }
#endif /*ADTS*/

  // I2R: added 28 May 07
  if ((!mip->aac_core_present)&&(frame==0))
	  numChannels=mip->nch;
 
  reset_mc_info(mip);

  /* losless only: switch off decoding of aac */
  if (mip->aac_core_present){
#ifdef BSAC
	  if (codec_type == 1)
	  {
	  int len;
	  HANDLE_BSBITSTREAM bsac_stream;
	  HANDLE_BSBITBUFFER bsac_bitBuf;
	  int ch;

	  //read bitstream
	  bsac_bitBuf = BsAllocBuffer(256000);
	  if( !getBSACFrameBst(bsac_bitBuf->data,&len) )
		  myexit("bsacdec");
	  bsac_bitBuf->numBit = len*8;
	  
	  bsac_stream = BsOpenBufferRead(bsac_bitBuf); 
	  BsManipulateSetNumByte((BsBufferNumBit(bsac_bitBuf)+7)/8, bsac_stream);

	  //decode BSAC
	  ChannelOutNum = sam_decode_frame( 
						bsac_stream, 
						0, 
						coef,
                        wnd,
                        wnd_shape,
                        bsac_ch,
//SLS_BSAC
						ll_quantInfo,
						tns,
						hasmask,
						mask
						);

	  for(ch=0;ch<ChannelOutNum;ch++)
		memcpy(ll_quantInfo[ch]->info, winmap[*wnd], sizeof(Info));


//	  hasmask[1] = 2;
//	  for(i=0;i<ll_quantInfo[0]->max_sfb;i++)
//		  mask[1][i] = 1;

	  //TNS
	  BsFreeBuffer(bsac_bitBuf);
	  }
	  else
	  {
#endif


    while ((ele_id=getbits(LEN_SE_ID)) != ID_END) {
      /*    while (ele_id != ID_END) { */   
      /* get audio syntactic element */
      
      if(debug['v'])
        PRINT(SE, "\nele_id %d\n", ele_id);

      switch (ele_id) {
      case ID_SCE:		/* single channel */
      case ID_CPE:		/* channel pair */
      case ID_LFE:		/* low freq effects channel */
        if (huffdecode(ele_id, 
                       mip, 
                       wnd, 
                       wnd_shape,
                       cb_map, 
                       factors,
                       group, 
                       hasmask, 
                       mask, 
                       max_sfb,
                       lpflag, 
                       prstflag, 
                       nok_lt_status, 
                       tns, 
                       coef,
                       ll_quantInfo) 
            < 0)
          myexit("huffdecode"); 
        break;
      case ID_DSE:		/* data element */
        if (getdata(&d_tag, &d_cnt, d_bytes) < 0)
          myexit("data channel");
        break;
      case ID_PCE:		/* program config element */
        get_prog_config(&prog_config);
        break;
      case ID_FIL:		/* fill element */
        getfill(d_bytes);
        break;
      default:
        PRINT(SE, "Element not supported: %d\n", ele_id);
        break;
      }
    }
#ifdef BSAC
	  }
#endif
  }
#ifdef BSAC
  if (codec_type == 1)
  {
  mip->nch = ChannelOutNum;
  mip->profile = 1;
  /* mono */
  if (mip->nch == 1) {
    mip->ch_info[0].present = 1;
  }
  /* stereo */
  if (mip->nch == 2) {
    for (ch=0;ch<2;ch++){
      mip->ch_info[ch].present = 1;
      mip->ch_info[ch].cpe = 1;
//	  mip->ch_info[ch].common_window = 1;
      mip->ch_info[ch].common_window = (hasmask[1] == 0 ? 0 : 1);
      mip->ch_info[ch].widx = 1;
      if (ch == 0){
        mip->ch_info[ch].ch_is_left = 1;
        mip->ch_info[ch].paired_ch = 1;
      }
      else {
        mip->ch_info[ch].ch_is_left = 0;
        mip->ch_info[ch].paired_ch = 0;
      }
    }
  }
  if (mip->nch == 6) {
	  // SCE
	  mip->ch_info[0].present = 1;
	  mip->ch_info[0].cpe = 0;
	  mip->ch_info[0].common_window = 1;
	  mip->ch_info[0].widx = 0;
	  // CPE1
	  for (ch=1;ch<=2;ch++){
          mip->ch_info[ch].present = 1;
          mip->ch_info[ch].cpe = 1;
          mip->ch_info[ch].common_window = 1;
          mip->ch_info[ch].widx = 4;
          if (ch == 1){
			  mip->ch_info[ch].ch_is_left = 1;
			  mip->ch_info[ch].paired_ch = 2;
          }
          else {
			  mip->ch_info[ch].ch_is_left = 0;
			  mip->ch_info[ch].paired_ch = 1;
          }
	  }
	  // CPE2
	  for (ch=3;ch<=4;ch++){
          mip->ch_info[ch].present = 1;
          mip->ch_info[ch].cpe = 1;
          mip->ch_info[ch].common_window = 1;
          mip->ch_info[ch].widx = 4;
          if (ch == 3){
			  mip->ch_info[ch].ch_is_left = 1;
			  mip->ch_info[ch].paired_ch = 4;
          }
          else {
			  mip->ch_info[ch].ch_is_left = 0;
			  mip->ch_info[ch].paired_ch = 3;
          }
	  }
	  // LFE
	  mip->ch_info[5].present = 1;
	  mip->ch_info[5].cpe = 0;
	  mip->ch_info[5].common_window = 1;
	  mip->ch_info[5].widx = 0;
  }
  }
  else
  {
#endif

  if(debug['v']) {
    PRINT(SE, "\nele_id %d\n", ele_id);
    print_mc_info(mip);
  }

  check_mc_info(mip, (frame/*bno*/==0 && default_config));

#ifdef BSAC
  }
#endif



    /* lossless only: fill channel structure with dataset for stereo, ch 1 resp 2 */
    if (!(mip->aac_core_present)){ 
	// I2R: modified 28 May 07
	  mip->nch = numChannels; 
      mip->profile = 1;
      /* mono */
      if (mip->nch == 1) {
        mip->ch_info[0].present = 1;
      }
      /* stereo */
      if (mip->nch == 2) {
        for (ch=1;ch<=2;ch++){
          mip->ch_info[ch].present = 1;
          mip->ch_info[ch].cpe = 1;
          mip->ch_info[ch].common_window = 1;
          mip->ch_info[ch].widx = 1;
          if (ch == 1){
            mip->ch_info[ch].ch_is_left = 1;
            mip->ch_info[ch].paired_ch = 2;
          }
          else {
            mip->ch_info[ch].ch_is_left = 0;
            mip->ch_info[ch].paired_ch = 1;
          }
        }
      }

	 // I2R: added for m/c support (22 Feb)			
	  /* multichannel 5.1 - SCE, CPE1, CPE2, LFE */
      if (mip->nch == 6) {
		// SCE
        mip->ch_info[0].present = 1;
		mip->ch_info[0].cpe = 0;
		mip->ch_info[0].common_window = 1;
		mip->ch_info[0].widx = 0;
		// CPE1
        for (ch=1;ch<=2;ch++){
          mip->ch_info[ch].present = 1;
          mip->ch_info[ch].cpe = 1;
          mip->ch_info[ch].common_window = 1;
          mip->ch_info[ch].widx = 1;
          if (ch == 1){
            mip->ch_info[ch].ch_is_left = 1;
            mip->ch_info[ch].paired_ch = 2;
          }
          else {
            mip->ch_info[ch].ch_is_left = 0;
            mip->ch_info[ch].paired_ch = 1;
          }
        }
		// CPE2
		for (ch=3;ch<=4;ch++){
          mip->ch_info[ch].present = 1;
          mip->ch_info[ch].cpe = 1;
          mip->ch_info[ch].common_window = 1;
          mip->ch_info[ch].widx = 1;
          if (ch == 3){
            mip->ch_info[ch].ch_is_left = 1;
            mip->ch_info[ch].paired_ch = 4;
          }
          else {
            mip->ch_info[ch].ch_is_left = 0;
            mip->ch_info[ch].paired_ch = 3;
          }
        }
		// LFE
		mip->ch_info[5].present = 1;
		mip->ch_info[5].cpe = 0;
		mip->ch_info[5].common_window = 1;
		mip->ch_info[5].widx = 0;
      
      }

    }


    for (ch=0; ch<Chans; ch++) {
      cip = &mip->ch_info[ch];
      if (!(cip->present)) continue;

      LIT_ll_read(ll_info[ch], 0);
      ll_info[ch]->type_PCM = type_pcm;
    }

    for (ch=0; ch<Chans; ch++) {
      cip = &mip->ch_info[ch];
      if ((cip->present) && (cip->cpe) && (cip->ch_is_left)) {
        wn = cip->widx;
        if(hasmask[wn]) {
          left = ch;
          right = cip->paired_ch;
          if (hasmask[wn] == 1)
            map_mask(ll_quantInfo[ch]->info, group[wn], mask[wn], cb_map[right]);
        }
      }
    }

    for (ch=0; ch<Chans; ch++) {
      cip = &mip->ch_info[ch];
      if (!(cip->present)) continue;
      wn = cip->widx;

#ifdef BSAC
	  if (codec_type == 1)
      ll_quantInfo[ch]->max_sfb = ll_quantInfo[ch]->num_sfb;
	  else
#endif
      ll_quantInfo[ch]->max_sfb = max_sfb[wn];

      if (mip->aac_core_present) {
        calc_sfb_offset_table(ll_quantInfo[ch],
                              mip->osf);
      }
      /* decode block type for lossless-only before calc_sfb_offset_table(), otherwise the sfb_offset table is not correct for lossless-only */

      LIT_ll_Dec(ll_info[ch],
                 ll_quantInfo[ch],
                 mask[wn],
                 hasmask[wn],
                 cb_map,
                 ll_coef[ch],
                 cip,/*->ch_is_left,*/
                 samp_rate_info[mip->sampling_rate_idx].samp_rate*mip->osf,	// I2R: modified to osf*core_sample_rate (28 Jun 07)
                 mip->osf,
                 &wnd[wn],
                 &msStereoIntMDCT,
                 mip->aac_core_present,
                 ch,
                 group[wn]);

    }

    for (ch=0; ch<Chans; ch++) {
      cip = &mip->ch_info[ch];
      if (!(cip->present)) continue;
      wn = cip->widx;
      
      if ((cip->cpe)&&(cip->ch_is_left)) {
        left = ch;
        right = cip->paired_ch;

        /*         printf("common_window: %d, hasmask: %d\n", cip->common_window, hasmask[wn]); */

        if ((cip->common_window) && (mip->aac_core_present)) {
          LL_MSDecode(ll_coef[left],
                      ll_coef[right],
                      ll_quantInfo[ch],
                      mask[wn],
                      hasmask[wn],
                      msStereoIntMDCT
                      );
        }
      }


      if (!(ll_quantInfo[ch]->info->islong)) {
        int temp_coef[MAX_OSF*LN2];
        lit_ll_deinterleave(ll_coef[ch], 
                            temp_coef, 
                            group[wn], 
                            ll_quantInfo[ch]);
        memcpy(ll_coef[ch], temp_coef, sizeof(temp_coef));

      }
    }


    for (ch=0; ch<Chans; ch++) {
		cip = &mip->ch_info[ch];
		if (!(cip->present)) continue;
		wn = cip->widx;  
		if ((cip->cpe)&&(cip->ch_is_left)) {
			left = ch;
			right = cip->paired_ch;
		}
		
		if ( (cip->cpe) && (cip->common_window) && msStereoIntMDCT ) {
			if ((cip->ch_is_left) && (mip->aac_core_present)) {
				
				/* Int-TNS */      
				{
					{
						int i;
						/* Int M/S */
						/* do inverse M/S to provide L/R spectrum for TNS */
						for (i=0; i<mip->osf*1024; i++) {
							InverseIntMidSideINT32(&ll_coef[left][i],
								&ll_coef[right][i]);
						}
					}
					{
						int i,j;
						
						for (i=j=0; i<tns[left]->n_subblocks; i++) {
							
							int_tns_decode_subblock(&ll_coef[left][j],
#ifdef BSAC
								ll_quantInfo[left]->max_sfb,//SLS_BSAC max_sfb[wn],
#else					
								max_sfb[wn],
#endif
								ll_quantInfo[left]->info->sbk_sfb_top[i],
								ll_quantInfo[left]->info->islong,
								&(tns[left]->info[i]) );  
							j += mip->osf*ll_quantInfo[left]->info->bins_per_sbk[i];
						}
						for (i=j=0; i<tns[right]->n_subblocks; i++) {
							
							int_tns_decode_subblock(&ll_coef[right][j],
#ifdef BSAC
								ll_quantInfo[right]->max_sfb,//SLS_BSAC max_sfb[wn],
#else
								max_sfb[wn],
#endif
								ll_quantInfo[right]->info->sbk_sfb_top[i],
								ll_quantInfo[right]->info->islong,
								&(tns[right]->info[i]) );  
							j += mip->osf*ll_quantInfo[right]->info->bins_per_sbk[i];
						}    
					} 
					{
						int i;
						/* Int M/S */
						/* do forward M/S to revert to M/S spectrum after TNS */
						for (i=0; i<mip->osf*1024; i++) {
							IntMidSideINT32(&ll_coef[left][i],
                                &ll_coef[right][i]);
						}
					}
				} 
			}
			
			/* StereoIntMDCT */
			if (cip->ch_is_left) {
				InvStereoIntMDCT(ll_coef[left],
					ll_coef[right],
					mdct_buf[left],
					mdct_buf[right],
					ll_timesig[left],
					ll_timesig[right],
					wnd[wn],
					wnd_shape[ch].prev_bk,
					mip->osf);
			}
		} else {
			if (mip->aac_core_present) {
				int i,j;
				
				for (i=j=0; i<tns[ch]->n_subblocks; i++) {
					if (debug['T']) {
						PRINT(SE, "%ld %d %d\n", bno, ch, i);
						print_tns( &(tns[ch]->info[i]));
					}
					
					int_tns_decode_subblock(&ll_coef[ch][j],
#ifdef BSAC
						ll_quantInfo[left]->max_sfb,//SLS_BSAC max_sfb[wn],
						ll_quantInfo[left]->info->sbk_sfb_top[i],
						ll_quantInfo[left]->info->islong,
#else
						max_sfb[wn],
						ll_quantInfo[ch]->info->sbk_sfb_top[i],
						ll_quantInfo[ch]->info->islong,
#endif
						&(tns[ch]->info[i]) );  
					j += mip->osf*ll_quantInfo[ch]->info->bins_per_sbk[i];
				}
			}
			/* Mono IntMDCT */
			InvIntMDCT(ll_coef[ch],
				mdct_buf[ch],
				ll_timesig[ch],
				wnd[wn],
				wnd_shape[ch].prev_bk,
				mip->osf);
		}
		wnd_shape[ch].prev_bk = wnd_shape[wn].this_bk;      
    }

    
    for (ch=0; ch<Chans; ch++) {
      int ii;

      if (!(mip->ch_info[ch].present)) continue;

      for (ii = 0;ii<mip->osf*1024-start_offset;ii++)
        data[ch][ii] = ll_timesig[ch][ii+start_offset] * pcm_scaling[type_pcm];
    }

    if (bno > 1){
      writeout(data, mip,mip->osf*1024-start_offset/* len */);

      start_offset = 0;
    }   
    /* end of Decoder mode */


#ifdef	ADTS
  byte_align();  /* moved from above */
#endif


}





