/************************* MPEG-2 NBC Audio Decoder **************************
 *                                                                           *
"This software module was originally developed by 
AT&T, Dolby Laboratories, Fraunhofer Gesellschaft IIS and edited by
Yoshiaki Oikawa (Sony Corporation),
Mitsuyuki Hatanaka (Sony Corporation),
in the course of development of the MPEG-2 NBC/MPEG-4 Audio standard ISO/IEC 13818-7, 
14496-1,2 and 3. This software module is an implementation of a part of one or more 
MPEG-2 NBC/MPEG-4 Audio tools as specified by the MPEG-2 NBC/MPEG-4 
Audio standard. ISO/IEC  gives users of the MPEG-2 NBC/MPEG-4 Audio 
standards free license to this software module or modifications thereof for use in 
hardware or software products claiming conformance to the MPEG-2 NBC/MPEG-4
Audio  standards. Those intending to use this software module in hardware or 
software products are advised that this use may infringe existing patents. 
The original developer of this software module and his/her company, the subsequent 
editors and their companies, and ISO/IEC have no liability for use of this software 
module or modifications thereof in an implementation. Copyright is not released for 
non MPEG-2 NBC/MPEG-4 Audio conforming products.The original developer
retains full right to use the code for his/her  own purpose, assign or donate the 
code to a third party and to inhibit third party from using the code for non 
MPEG-2 NBC/MPEG-4 Audio conforming products. This copyright notice must
be included in all copies or derivative works." 
Copyright(c)1996.
 *                                                                           *
 ****************************************************************************/

#ifndef	_all_h_
#define _all_h_


#define	nil			((void*)0)
#define	Tnleaf			0x8000

/*includes decoder*/
#include <sys/types.h>

typedef	unsigned char	byte;

/*includes shared*/
#include "stdinc.h"
#include "interface.h"
#include "tns.h"
#include "chandefs.h"

typedef	float	Float; 

/*defines decoder*/

#define BLOCK_LEN_LONG      1024

#define MAX_LLE_TRACKS 32

enum
{
    /* block switch windows for single channels or channel pairs */
    Winds	= Chans,
   
    /* max length of DSE, bytes */
    MAX_DBYTES	= ((1<<LEN_D_CNT) + (1<<LEN_D_ESC)),

    TEXP	= 128,		/* size of exp cache table */
    MAX_IQ_TBL	= 128,		/* size of inv quant table */
    MAXFFT	= LN4,

    XXXXX
};

/* typedefs shared*/

  
typedef struct
{
    int	    islong;			/* true if long block */
    int	    nsbk;			/* sub-blocks (SB) per block */
    int	    bins_per_bk;		/* coef's per block */
    int	    sfb_per_bk;			/* sfb per block */
    int	    bins_per_sbk[MAX_SBK];	/* coef's per SB */
    int	    sfb_per_sbk[MAX_SBK];	/* sfb per SB */
    int	    sectbits[MAX_SBK];
    short   *sbk_sfb_top[MAX_SBK];	/* top coef per sfb per SB */
    short   *sfb_width_128;		/* sfb width for short blocks */
    short   bk_sfb_top[MAXBANDS];		/* cum version of above */
    int	    num_groups;
    short   group_len[8];
  short   group_offs[8]; /* currently unused! */
} Info;

typedef struct {
    int	    samp_rate;
    int	    nsfb1024;
    short*  SFbands1024;
    int	    nsfb128;
    short*  SFbands128;
} SR_Info;

typedef struct
{
    byte    this_bk;
    byte    prev_bk;
} Wnd_Shape;

typedef struct
{
    int		index;
    int		len;
    unsigned long	cw;
} Huffman;

typedef	struct
{
    int		n;
    int		dim;
    int		lav;
    int		mod;
    int		off;
    int		signed_cb;
    Huffman	*hcw;
} Hcb;

typedef struct
{
    Float	sin;
    Float	cos;
}Stab;

typedef struct
{
    int is_present;
    int ms_used[MAXBANDS];
} MS_Info;
  
typedef struct
{
    int is_present;	/* right channel uses intensiy stereo */
    int is_used[MAXBANDS];
    int sign[2*(MAXBANDS+1)];
    int fac[2*(MAXBANDS+1)];
    int bot[2*(MAXBANDS+1)];
    int top[2*(MAXBANDS+1)];
} IS_Info;

typedef struct
{
    int present;	/* channel present */
    int tag;		/* element tag */
    int cpe;		/* 0 if single channel, 1 if channel pair */
    int	common_window;	/* 1 if common window for cpe */
    int	ch_is_left;	/* 1 if left channel of cpe */
    int	paired_ch;	/* index of paired channel in cpe */
    int widx;		/* window element index for this channel */
    int is_present;	/* intensity stereo is used */
    int ncch;		/* number of coupling channels for this ch */
    char *fext;		/* filename extension */
    MS_Info ms_info;    /* MS information */
    IS_Info is_info;	/* IS information */
    int lfe;
} Ch_Info;

typedef struct {
    int nch;		/* total number of audio channels */
    int nfsce;		/* number of front SCE's pror to first front CPE */
    int nfch;		/* number of front channels */
    int nsch;		/* number of side channels */
    int nbch;		/* number of back channels */
    int nlch;		/* number of lfe channels */
    int ncch;		/* number of valid coupling channels */
    int cc_tag[(1<<LEN_TAG)];	/* tags of valid CCE's */
    int cc_ind[(1<<LEN_TAG)];	/* independently switched CCE's */
    int profile;
    int sampling_rate_idx;
    Ch_Info ch_info[Chans];
  int pcmWordLength;
  int osf;
  int aac_core_present;
} MC_Info;

typedef struct {
    int num_ele;
    int ele_is_cpe[(1<<LEN_TAG)];
    int ele_tag[(1<<LEN_TAG)];
} EleList;

typedef struct {
    int present;
    int ele_tag;
    int pseudo_enab;
} MIXdown;

typedef struct {
    int profile;
    int sampling_rate_idx;
    EleList front;
    EleList side;
    EleList back;
    EleList lfe;
    EleList data;
    EleList coupling;
    MIXdown mono_mix;
    MIXdown stereo_mix;
    MIXdown matrix_mix;
    char comments[(1<<LEN_PC_COMM)+1];
    long    buffer_fullness;	/* put this transport level info here */
} ProgConfig;

typedef struct {
    char    adif_id[LEN_ADIF_ID+1];
    int	    copy_id_present;
    char    copy_id[LEN_COPYRT_ID+1];
    int	    original_copy;
    int	    home;
    int	    bitstream_type;
    long    bitrate;
    int	    num_pce;
    int	    prog_tags[(1<<LEN_TAG)];
} ADIF_Header;

#ifdef	ADTS
typedef struct {
	int	syncword;
	int	id;
	int	layer;
	int	protection_abs;
	int 	profile;
	int	sampling_freq_idx;
	int	private_bit;
	int	channel_config;
	int	original_copy;
	int	home;
	int	copyright_id_bit;
	int	copyright_id_start;
	int	frame_length;
	int	adts_buffer_fullness;
	int	num_of_rdb;
	int	crc_check;
} ADTS_Header;
#endif

extern	long		bno;
extern	Huffman		book1[];
extern	Huffman		book2[];
extern	Huffman		book3[];
extern	Huffman		book4[];
extern	Huffman		book5[];
extern	Huffman		book6[];
extern	Huffman		book7[];
extern	Huffman		book8[];
extern	Huffman		book9[];
extern	Huffman		book10[];
extern	Huffman		book11[];
extern	Huffman		bookscl[];
extern	Hcb		book[NSPECBOOKS+2];
extern	int		bufpool;
extern	int		chn_present;
extern	long		cword;
extern	int		debug[256];
extern	Info		eight_short_info;
extern	Float		exptable[TEXP];
extern	Float*		hvals[NSPECBOOKS];
extern	Float		iq_exp_tbl[MAX_IQ_TBL];
extern	int		inerror;
/*extern	int		istrans;*/
extern	int		*lpflag[Chans];
extern	int		maxfac;
extern	int		nbits;

#ifdef ADIF2MP4
extern	int		ll_nbits;
extern  long	ll_cword;
extern  int     ll_inerror;
#else
extern	int		ll_nbits[MAX_LLE_TRACKS];
extern  long	ll_cword[MAX_LLE_TRACKS];
extern  int     ll_inerror[MAX_LLE_TRACKS];
#endif

extern	Info		only_long_info;
extern	int		pred_max_bands_tbl[(1<<LEN_SAMP_IDX)];
extern	short		sfbwidth128[];
extern	SR_Info		samp_rate_info[];
extern	Stab		table2[MAXFFT];
extern	Stab		table[MAXFFT/2];
extern	int		tns_max_bands_tbl[(1<<LEN_SAMP_IDX)][4];
extern	long		total1;
extern	long		total2;
extern	Info*		winmap[NUM_WIN_SEQ];
extern	Info		*win_seq_info[NUM_WIN_SEQ];

extern	int		default_config;
extern	int		adif_header_present;
extern	int		current_program;
extern	ADIF_Header	adif_header;
extern	ProgConfig	prog_config;
extern	MC_Info		mc_info;

#ifdef ADTS
extern	int 		adts_header_present;
extern	ADTS_Header	adts_header;
#endif


#define _WINDOW_TYPE_EXT
typedef byte WINDOW_TYPE_EXT;
#include "nok_lt_prediction.h"
#ifdef USELIB_ISOMP4
extern  int             mpeg4_header_present;
#endif

#endif	/* _all.h_ */
