/************************* MPEG-2 NBC Audio Decoder **************************
 *                                                                           *
"This software module was originally developed by 
AT&T, Dolby Laboratories, Fraunhofer Gesellschaft IIS in the course of 
development of the MPEG-2 NBC/MPEG-4 Audio standard ISO/IEC 13818-7, 
14496-1,2 and 3. This software module is an implementation of a part of one or more 
MPEG-2 NBC/MPEG-4 Audio tools as specified by the MPEG-2 NBC/MPEG-4 
Audio standard. ISO/IEC  gives users of the MPEG-2 NBC/MPEG-4 Audio 
standards free license to this software module or modifications thereof for use in 
hardware or software products claiming conformance to the MPEG-2 NBC/MPEG-4
Audio  standards. Those intending to use this software module in hardware or 
software products are advised that this use may infringe existing patents. 
The original developer of this software module and his/her company, the subsequent 
editors and their companies, and ISO/IEC have no liability for use of this software 
module or modifications thereof in an implementation. Copyright is not released for 
non MPEG-2 NBC/MPEG-4 Audio conforming products.The original developer
retains full right to use the code for his/her  own purpose, assign or donate the 
code to a third party and to inhibit third party from using the code for non 
MPEG-2 NBC/MPEG-4 Audio conforming products. This copyright notice must
be included in all copies or derivative works." 
Copyright(c)1996.
 *                                                                           *
 ****************************************************************************/

#include "all.h"
#include "port.h"

/*
 * profile dependent parameters
 */
int
tns_max_bands(int islong)
{
    int i;

    /* construct second index */
    i = islong ? 0 : 1;
    i += (mc_info.profile ==  SSR_Profile) ? 2 : 0;
    
    return tns_max_bands_tbl[mc_info.sampling_rate_idx][i];
}

int
tns_max_order(int islong)
{
    if (islong) {
	switch (mc_info.profile) {
	case Main_Profile:
	case LTP_Profile:
	    return 20;
	case LC_Profile:
	case SSR_Profile:
	    return 12;
	}
    }
    else {
	/* short window */
	return 7;
    }
    return 0;
}

int
pred_max_bands(void)
{
    return pred_max_bands_tbl[mc_info.sampling_rate_idx];
}

#define LF	"%-25s %d\n"
/*
 * adif_header
 */
int
get_adif_header(int extended_show)
{
    int i, n, tag, select_status;
    ProgConfig tmp_config;
    ADIF_Header *p = &adif_header;
    
    select_status = 0;
     
    /* adif header */
    for (i=0; i<LEN_ADIF_ID; i++)
	p->adif_id[i] = getbits(LEN_BYTE); 
    p->adif_id[i] = 0;	    /* null terminated string */
    /* test for id */
    if (strncmp(p->adif_id, "ADIF", 4) != 0)
	return -1;	    /* bad id */
    /* copyright string */	
    if ((p->copy_id_present = getbits(LEN_COPYRT_PRES)) == 1) {
	for (i=0; i<LEN_COPYRT_ID; i++)
	    p->copy_id[i] = getbits(LEN_BYTE); 
	p->copy_id[i] = 0;  /* null terminated string */
    }
    p->original_copy = getbits(LEN_ORIG);
    p->home = getbits(LEN_HOME);
    p->bitstream_type = getbits(LEN_BS_TYPE);
    p->bitrate = getbits(LEN_BIT_RATE);

    /* program config elements */ 
    n = getbits(LEN_NUM_PCE) + 1;

    if (debug['v']||extended_show) {
	PRINT(SE, "\nADIF header:\n");
	PRINT(SE, LF, "p->copy_id_present", p->copy_id_present);
	PRINT(SE, LF, "p->original_copy", p->original_copy);
	PRINT(SE, LF, "p->home", p->home);
	PRINT(SE, LF, "p->bitstream_type", p->bitstream_type);
	PRINT(SE, LF, "p->bitrate", (int)p->bitrate);
	PRINT(SE, LF, "number of PCE", n);
    }
    
    for (i=0; i<n; i++) {
	tmp_config.buffer_fullness =
	    (p->bitstream_type == 0) ? getbits(LEN_ADIF_BF) : 0;
	tag = get_prog_config(&tmp_config);
/***** this is redundant
	if (current_program < 0)
	    current_program = tag;
*****/
	if (tag == current_program) {
	    memcpy(&prog_config, &tmp_config, sizeof(prog_config));
	    select_status = 1;
	}
	if (debug['v'])
	    PRINT(SE, "tag %d %s\n", tag, (tag == current_program) ? "match" : "");
        if (extended_show) {
          PRINT(SE, "\n%-25s %d\n", "PCE number", i); /* shouldn't this be tag? */
          print_PCE(&tmp_config);
        }
    }

    if ((debug['v'] && select_status)&&(!extended_show)) {
      PRINT(SE, "\n" );
      print_PCE(&prog_config);
    }

    return select_status;
}

#ifdef	ADTS

void get_adts_header(void){

	ADTS_Header *p = &adts_header;

   p->syncword = getbits(LEN_SYNCWORD);
   p->id = getbits(LEN_ID);
   p->layer = getbits(LEN_LAYER);
   p->protection_abs = getbits(LEN_PROTECT_ABS);
   p->profile = getbits(LEN_PROFILE);
   p->sampling_freq_idx = getbits(LEN_SAMP_IDX);
   p->private_bit = getbits(LEN_PRIVTE_BIT);
   p->channel_config = getbits(LEN_CHANNEL_CONF);
   p->original_copy = getbits(LEN_ORIG);
   p->home = getbits(LEN_HOME);
   p->copyright_id_bit = getbits(LEN_COPYRT_ID_ADTS);
   p->copyright_id_start = getbits(LEN_COPYRT_START);
   p->frame_length = getbits(LEN_FRAME_LEN);
   p->adts_buffer_fullness = getbits(LEN_ADTS_BUF_FULLNESS);
   p->num_of_rdb = getbits(LEN_NUM_OF_RDB);

	if(p->protection_abs == 0){
		p->crc_check = getbits(LEN_CRC);
	}

	if(p->layer != 0){
		printf("ADTS layer data error!   layer: %d \n",p->layer);
		exit(0);
	}

	mc_info.profile = p->profile;
   mc_info.sampling_rate_idx = p->sampling_freq_idx;

   infoinit(&samp_rate_info[mc_info.sampling_rate_idx]);
}

#endif

/*
 * program configuration element
 */
static void
get_ele_list(EleList *p, int enable_cpe)
{  
    int i, j;
    for (i=0, j=p->num_ele; i<j; i++) {
	if (enable_cpe)
	    p->ele_is_cpe[i] = getbits(LEN_ELE_IS_CPE);
        else
            p->ele_is_cpe[i] = 0; /* sdb */
	p->ele_tag[i] = getbits(LEN_TAG);
    }
}

int
get_prog_config(ProgConfig *p)
{
    int i, j, tag;

    tag = getbits(LEN_TAG);

    p->profile = getbits(LEN_PROFILE);
    p->sampling_rate_idx = getbits(LEN_SAMP_IDX);
    p->front.num_ele = getbits(LEN_NUM_ELE);
    p->side.num_ele = getbits(LEN_NUM_ELE);
    p->back.num_ele = getbits(LEN_NUM_ELE);
    p->lfe.num_ele = getbits(LEN_NUM_LFE);
    p->data.num_ele = getbits(LEN_NUM_DAT);
    p->coupling.num_ele = getbits(LEN_NUM_CCE);
    if ((p->mono_mix.present = getbits(LEN_MIX_PRES)) == 1)
	p->mono_mix.ele_tag = getbits(LEN_TAG);
    if ((p->stereo_mix.present = getbits(LEN_MIX_PRES)) == 1)
	p->stereo_mix.ele_tag = getbits(LEN_TAG);
    if ((p->matrix_mix.present = getbits(LEN_MIX_PRES)) == 1) {
	p->matrix_mix.ele_tag = getbits(LEN_MMIX_IDX);
	p->matrix_mix.pseudo_enab = getbits(LEN_PSUR_ENAB);
    }
    get_ele_list(&p->front, 1);
    get_ele_list(&p->side, 1);
    get_ele_list(&p->back, 1);
    get_ele_list(&p->lfe, 0);
    get_ele_list(&p->data, 0);
    get_ele_list(&p->coupling, 1);
    
    byte_align();
    j = getbits(LEN_COMMENT_BYTES);
    for (i=0; i<j; i++)
	p->comments[i] = getbits(LEN_BYTE);
    p->comments[i] = 0;	/* null terminator for string */

    /* activate new program configuration if appropriate */
    if (current_program < 0)
	current_program = tag;	    /* select first program found */
    if (tag == current_program) {
	/* enter configuration into MC_Info structure */
	if (enter_mc_info(&mc_info, p) < 0)
	    return -1;
	/* inhibit default configuration */
	default_config = 0;
    }
   
    return tag;
}

void
print_ele(char *s, EleList *p) /* not located in library */
{
    int i;
    PRINT(SE, "%s\n", s);
    if (p->num_ele) {
      if (!strstr(s,"coupling"))
	PRINT(SE, "%6s %6s %6s\n", "idx", "cpe", "tag");
      else
        PRINT(SE, "%6s %6s %6s\n", "idx", "ind", "tag");
    }
    for (i=0; i<p->num_ele; i++)
	PRINT(SE, "%6d %6d %6d\n", i, p->ele_is_cpe[i], p->ele_tag[i]);    
}

void
print_adif_header(ADIF_Header *p)
{
  PRINT(SE, "\nADIF header:\n");
  PRINT(SE, "%-25s %d\n", "p->copy_id_present", p->copy_id_present);
  PRINT(SE, "%-25s %d\n", "p->original_copy", p->original_copy);
  PRINT(SE, "%-25s %d\n", "p->home", p->home);
  PRINT(SE, "%-25s %d\n", "p->bitstream_type", p->bitstream_type);
  PRINT(SE, "%-25s %d\n", "p->bitrate", (int) p->bitrate);
}

void
print_PCE(ProgConfig *p)
{
  PRINT(SE, "%-25s %d\n", "p->profile", p->profile);
  PRINT(SE, "%-25s %d\n", "p->sampling_rate_idx", p->sampling_rate_idx);
	
  print_ele("p->front", &p->front);
  print_ele("p->side", &p->side);
  print_ele("p->back", &p->back);
  print_ele("p->lfe", &p->lfe);
  print_ele("p->coupling", &p->coupling);
	
  PRINT(SE, "%-25s %d\n", "p->data.num_ele", p->data.num_ele);
  PRINT(SE, "%-25s %d\n", "p->mono_mix.present", p->mono_mix.present);
  PRINT(SE, "%-25s %d\n", "p->stereo_mix.present", p->stereo_mix.present);
  PRINT(SE, "%-25s %d\n", "p->matrix_mix.present", p->matrix_mix.present);
  PRINT(SE, "Comment: %s\n", p->comments);
}

/* enter program configuration into MC_Info structure
 *  only configures for channels specified in all.h
 */
int
enter_mc_info(MC_Info *mip, ProgConfig *pcp)
{  
    int i, j, cpe, tag, cw;
    EleList *elp;
    MIXdown *mxp;

    /* reset channel counts */
    mip->nch = 0;
    mip->nfch = 0;
    mip->nfsce = 0;
    mip->nsch = 0;
    mip->nbch = 0;
    mip->nlch = 0;
    mip->ncch = 0;

    /* profile and sampling rate
     *	re-configure if new sampling rate
     */    
    mip->profile = pcp->profile;
    if (mip->sampling_rate_idx != pcp->sampling_rate_idx) {
	mip->sampling_rate_idx = pcp->sampling_rate_idx;
	infoinit(&samp_rate_info[mip->sampling_rate_idx]);
    }
    
    cw = 0;			    /* changed later */
    
    /* front elements, center out */
    elp = &pcp->front;
    /* count number of leading SCE's */
    for (i=0, j=elp->num_ele; i<j; i++) {
	if (elp->ele_is_cpe[i])
	    break;
	mip->nfsce++;
    }
    for (i=0, j=elp->num_ele; i<j; i++) {
	cpe = elp->ele_is_cpe[i];
	tag = elp->ele_tag[i];
	if (enter_chn(cpe, tag, 'f', cw, mip) < 0)
	    return(-1);
    }
    
    /* side elements, left to right then front to back */
    elp = &pcp->side;
    for (i=0, j=elp->num_ele; i<j; i++) {
	cpe = elp->ele_is_cpe[i];
	tag = elp->ele_tag[i];
	if (enter_chn(cpe, tag, 's', cw, mip) < 0)
	    return(-1);
    }
	  
    /* back elements, outside to center */
    elp = &pcp->back;
    for (i=0, j=elp->num_ele; i<j; i++) {
	cpe = elp->ele_is_cpe[i];
	tag = elp->ele_tag[i];
	if (enter_chn(cpe, tag, 'b', cw, mip) < 0)
	     return(-1);
    }
    
    /* lfe elements */
    elp = &pcp->lfe;
    for (i=0, j=elp->num_ele; i<j; i++) {
	cpe = elp->ele_is_cpe[i];
	tag = elp->ele_tag[i];
	if (enter_chn(cpe, tag, 'l', cw, mip) < 0)
	    return(-1);
    }
        
    /* coupling channel elements */
    elp = &pcp->coupling;
    for (i=0, j=elp->num_ele; i<j; i++)
	mip->cc_tag[i] = elp->ele_tag[i];
    mip->ncch = j;
    
    /* mono mixdown elements */
    mxp = &pcp->mono_mix;
    if (mxp->present) {
	PRINT(SE,"Unanticipated mono mixdown channel\n");
	return(-1);
    }
    
    /* stereo mixdown elements */
    mxp = &pcp->stereo_mix;
    if (mxp->present) {
	PRINT(SE,"Unanticipated stereo mixdown channel\n");
	return(-1);
    }
    
    /* matrix mixdown elements */
    mxp = &pcp->matrix_mix;
    if (mxp->present) {
	PRINT(SE,"Unanticipated matrix mixdown channel\n");
	return(-1);
    }
    
    /* save to check future consistency */
    check_mc_info(&mc_info, 1);

    return 1;
}

/* translate prog config or default config 
 * into to multi-channel info structure
 *  returns index of channel in MC_Info
 */
int
enter_chn(int cpe, int tag, char position, int common_window, MC_Info *mip)
{
    int nch, cidx=0;
    Ch_Info *cip;

    nch = (cpe == 1) ? 2 : 1;
    
    switch (position) {
    /* use configuration already in MC_Info, but now common_window is valid */
    case 0:
	cidx = ch_index(mip, cpe, tag);

	if (nch == 1) {
	  mip->ch_info[cidx].widx = cidx; /* only one channel for window info*/
	}
	else {
	  if (common_window) {
	    mip->ch_info[cidx].widx = cidx; /* window info is left */
	    mip->ch_info[cidx+1].widx = cidx;
	  }
	  else {
	    mip->ch_info[cidx].widx = cidx; /* each has window info */
	    mip->ch_info[cidx+1].widx = cidx+1;
	  }
	}
	return cidx;
	
    /* build configuration */
    case 'f':
	if (mip->nfch + nch > FChans) {
	    PRINT(SE,"Unanticipated front channel\n");
	    return -1;
	}
	if (mip->nfch == 0) {
	    /* consider case of center channel */
	    if (FCenter) {
		if (cpe) {
		    /* has center speaker but center channel missing */
		    cidx = 0 + 1;
		    mip->nfch = 1 + nch;
		}
		else {
		    if (mip->nfsce & 1) {
			/* has center speaker and this is center channel */
			/* odd number of leading SCE's */
			cidx = 0;
			mip->nfch = nch;
		    }
		    else {
			/* has center speaker but center channel missing */
			/* even number of leading SCE's */
			/* (Note that in implicit congiguration
			 * channel to speaker mapping may be wrong 
			 * for first block while count of SCE's prior
			 * to first CPE is being make. However first block
			 * is not written so it doesn't matter.
			 * Second block will be correct.
			 */
			cidx = 0 + 1;
			mip->nfch = 1 + nch;
		    }
		}
	    }
	    else {
		if (cpe) {
		    /* no center speaker and center channel missing */
		    cidx = 0;
		    mip->nfch = nch;
		}
		else {
		    /* no center speaker so this is left channel */
		    cidx = 0;
		    mip->nfch = nch;
		}
	    }
	}
	else {
	    cidx = mip->nfch;
	    mip->nfch += nch;
	}
	break;
    case 's':
	if (mip->nsch + nch > SChans)  {
	    PRINT(SE,"Unanticipated side channel\n");
	    return -1;
	}
	cidx = FChans + mip->nsch;
	mip->nsch += nch;
	break;
    case 'b':
	if (mip->nbch + nch > BChans) {
	    PRINT(SE,"Unanticipated back channel\n");
	    return -1;
	}
	cidx = FChans + SChans + mip->nbch;
	mip->nbch += nch;
	break;
    case 'l':
	if (mip->nlch + nch > LChans) {
	    PRINT(SE,"Unanticipated LFE channel\n");
	    return -1;
	}
	cidx = FChans + SChans + BChans + mip->nlch;
	mip->nlch += nch;
	break;
    default:
	myexit("enter_chn");
    }
    mip->nch += nch;

    if (cpe == 0) {
	/* SCE */
	cip = &mip->ch_info[cidx];
	cip->present = 1;
	cip->tag = tag;
	cip->cpe = 0;
	cip->common_window = common_window;
	cip->widx = cidx;
	/* mip->nch = cidx + 1; */
    }
    else {
        /* CPE */
	/* left */
	cip = &mip->ch_info[cidx];
	cip->present = 1;
	cip->tag = tag;
	cip->cpe = 1;
	cip->common_window = common_window;
	cip->ch_is_left = 1;
	cip->paired_ch = cidx+1;
	/* right */
	cip = &mip->ch_info[cidx+1];
	cip->present = 1;
	cip->tag = tag;
	cip->cpe = 1;
	cip->common_window = common_window;
	cip->ch_is_left = 0;
	cip->paired_ch = cidx;
	if (common_window) {
	    mip->ch_info[cidx].widx = cidx;	/* window info is left */
	    mip->ch_info[cidx+1].widx = cidx;
	}
	else {
	    mip->ch_info[cidx].widx = cidx;	/* each has window info */
	    mip->ch_info[cidx+1].widx = cidx+1;
	}
	/* mip->nch = cidx + 2; */
    }
    
    return cidx;
}

static char
default_position(MC_Info *mip, int id)
{
    static int first_cpe = 0;

    if (mip->nch < FChans) {
	if (id == ID_CPE)
	    /* here is the first CPE */
	    first_cpe = 1;
	else if ( (bno==0) && !first_cpe )
	    /* count number of SCE prior to first CPE in first block */
	    mip->nfsce++;
	return('f');
    }
    else if  (mip->nch < FChans+SChans) {
	return('s');
    }
    else if  (mip->nch < FChans+SChans+BChans) {
	return('b');
    }
    
    return 0;
}

/* retrieve appropriate channel index for the program
 * and decoder configuration
 */
int
chn_config(int id, int tag, int common_window, MC_Info *mip)
{
    int cpe, cidx=0;
    char position;
    
    /* channel index to position mapping for 5.1 configuration is
     *  0	center
     *  1	left  front
     *  2	right front
     *  3	left surround
     *  4	right surround
     *  5	lfe
     */

    cpe = (id == ID_CPE) ? 1 : 0;
    
    if (default_config) {
	switch (id) {
	case ID_SCE:
	case ID_CPE:
	    if ((position = default_position(mip, id)) == 0) {
		PRINT(SE,"Unanticipated channel\n");
		myexit("default_position");
	    }
	    cidx = enter_chn(cpe, tag, position, common_window, mip);
	    break;
	case ID_LFE:
	    cidx = enter_chn(cpe, tag, 'l', common_window, mip);
	    break;
	}
    }
    else {	
	cidx = enter_chn(cpe, tag, 0, common_window, mip);
    }	

    return cidx;	    /* index of chn in mc_info */
}

/* 
 * check continuity of configuration from one
 * block to next
 */
void
reset_mc_info(MC_Info *mip)
{
    int i;
    Ch_Info *p;

    /* always reset number of coupling channels per audio channel */
    for (i=0; i<Chans; i++)
	mip->ch_info[i].ncch = 0;
	
    /* only reset for implicit configuration */
    if (default_config) {
	/* reset channel counts */
	mip->nch = 0;
	mip->nfch = 0;
	mip->nsch = 0;
	mip->nbch = 0;
	mip->nlch = 0;
	mip->ncch = 0;
	/* reset prior to first block scan only! */
	if (bno==0)
	    mip->nfsce = 0;
	for (i=0; i<Chans; i++) {
	    p = &mip->ch_info[i];
	    p->present = 0;
	    p->cpe = 0;
	    p->ch_is_left = 0;
	    p->paired_ch = 0;
	    p->is_present = 0;
	    p->widx = 0;
	    p->ncch = 0;
	}
    }
}


void
check_mc_info(MC_Info *mip, int new_config)
{
    int i, nch, err;
    Ch_Info *s, *p;
    static MC_Info save_mc_info;

    nch = mip->nch;
    if (new_config) {
	/* enter valid configuration */
	for (i=0; i<nch; i++) {
	    s = &save_mc_info.ch_info[i];
	    p = &mip->ch_info[i];
	    s->present = p->present;
	    s->cpe = p->cpe;
	    s->ch_is_left = p->ch_is_left;
	    s->paired_ch = p->paired_ch;
	}
    }
    else {
	/* check this block's configuration */
	err = 0;
	for (i=0; i<nch; i++) {
	    s = &save_mc_info.ch_info[i];
	    p = &mip->ch_info[i];
	    if (s->present != p->present) err=1;
            if (!s->present) continue; /* sdb */
	    if (s->cpe != p->cpe) err=1;
	    if (s->ch_is_left != p->ch_is_left) err=1;
	    if (s->paired_ch != p->paired_ch) err=1;
	}
	if (err) {
	    PRINT(SE, "Channel configuration inconsistency\n");
	    for (i=0; i<nch; i++) {
		s = &save_mc_info.ch_info[i];
		p = &mip->ch_info[i];
		PRINT(SE, "Channel %d\n", i);
		PRINT(SE, "present    %d %d\n", s->present,  p->present);
		PRINT(SE, "cpe        %d %d\n", s->cpe,  p->cpe);
		PRINT(SE, "ch_is_left %d %d\n", s->ch_is_left,  p->ch_is_left);
		PRINT(SE, "paired_ch  %d %d\n", s->paired_ch,  p->paired_ch);
		PRINT(SE, "\n");
	    }
	    myexit("check_mc_info");
	}
    }
}

void
print_mc_info(MC_Info *mip){
    int i, j;
    Ch_Info *cip;
    PRINT(SE, "Ch Info\n");
    PRINT(SE, "idx tag cpe wdx  cch dom ind\n");
    for (i=0; i<Chans; i++) {
	cip = &mip->ch_info[i];
	if (!cip->present)
	    continue;
	PRINT(SE, "%3d %3d %3d %3d ", i, cip->tag, cip->cpe, cip->widx);
	for (j=0; j<cip->ncch; j++) {
	    if (j>0)
		PRINT(SE, "\n%16s", "");
	}
	PRINT(SE, "\n");
    }
}

/* given cpe and tag,
 *  returns channel index of SCE or left chn in CPE
 */
int
ch_index(MC_Info *mip, int cpe, int tag)
{
    int ch;
    
    for (ch=0; ch<Chans; ch++) {

	if (!(mip->ch_info[ch].present))
	    continue;

	if ( (mip->ch_info[ch].cpe == cpe) && 
	    (mip->ch_info[ch].tag == tag) ){
		return ch;
	}
    }
    
    /* no match, so channel is not in this program
     *	dummy up the ch_info structure so rest of chn will parse
     */
    if (XChans > 0) {
	ch = Chans - XChans;	/* left scratch channel */
	mip->ch_info[ch].cpe = cpe;
	mip->ch_info[ch].ch_is_left = 1;
	mip->ch_info[ch].widx = ch;
	if (cpe) {
	    mip->ch_info[ch].paired_ch = ch+1;
	    mip->ch_info[ch+1].ch_is_left = 0;
	    mip->ch_info[ch+1].paired_ch = ch;
	}
    }
    else {
	ch = -1;		/* error, no scratch space */
    }
    
    return ch;
}
