/*******************************************************************************
This software module was originally developed by

Institute for Infocomm Research and Fraunhofer IIS

and edited by

-

in the course of development of ISO/IEC 14496 for reference purposes and its
performance may not have been optimized. This software module is an
implementation of one or more tools as specified by ISO/IEC 14496. ISO/IEC gives
You a royalty-free, worldwide, non-exclusive, copyright license to copy,
distribute, and make derivative works of this software module or modifications
thereof for use in implementations of ISO/IEC 14496 in products that satisfy
conformance criteria (if any). Those intending to use this software module in
products are advised that its use may infringe existing patents. ISO/IEC have no
liability for use of this software module or modifications thereof. Copyright is
not released for products that do not conform to audiovisual and image-coding
related ITU Recommendations and/or ISO/IEC International Standards.

#ifdef NOT_PUBLISHED

Assurance that the originally developed software module can be used (1) in
ISO/IEC 14496 once ISO/IEC 14496 has been adopted; and (2) to develop ISO/IEC
14496:
Institute for Infocomm Research and Fraunhofer IIS grant ISO/IEC all
rights necessary to include the originally developed software module or
modifications thereof in ISO/IEC 14496 and to permit ISO/IEC to offer You a
royalty-free, worldwide, non-exclusive, copyright license to copy, distribute,
and make derivative works for use in implementations of ISO/IEC 14496 in
products that satisfy conformance criteria (if any), and to the extent that such
originally developed software module or portions of it are included in ISO/IEC
14496. To the extent that Institute for Infocomm Research and Fraunhofer IIS 
own patent rights that would be required to make, use, or sell the
originally developed software module or portions thereof included in ISO/IEC
14496 in a conforming product, Institute for Infocomm Research and Fraunhofer IIS 
will assure the ISO/IEC that they are willing to negotiate
licenses under reasonable and non-discriminatory terms and conditions with
applicants throughout the world. ISO/IEC gives You a free license to this
software module or modifications thereof for the sole purpose of developing
ISO/IEC 14496.

#endif

Institute for Infocomm Research and Fraunhofer IIS retain full right to
modify and use the code for their own purpose, assign or donate the code
to a third party and to inhibit third parties from using the code for products
that do not conform to MPEG-related ITU Recommendations and/or ISO/IEC
International Standards. This copyright notice must be included in all copies or
derivative works.

Copyright (c) ISO/IEC 2005.
*******************************************************************************/



#include <stdio.h>

#include "int_mdct_defs.h"
#include "inv_int_mdct.h"
#include "int_win_dctiv.h"

#ifdef COUNT_CYCLES
/* 10/19/2004 6:11PM, Yuriy A. Reznik <yreznik@real.com>
 *  read Pentium's internal cycle counter:
 **/
#ifdef _M_IX86
static __int64 rdtsc ()
{
    __asm _emit 0x0f
    __asm _emit 0x31
}
#endif

/* intMDCT cycle counters: */
__int64 cntCycles = 0, totalCycles = 0, totalSamples = 0;
float avgCyclesPerSample, maxCyclesPerSample;
#endif

#ifdef COUNT_32BIT_INSTRUCTIONS_NEW
/* per frame */
float cntMults = 0;
float cntAdds = 0;
float cntShifts = 0;
float cntOrs = 0;
float cntNegs = 0;
float cntMovs = 0;

float cntSamples = 0;

/* total */
float totalMults = 0;
float totalAdds = 0;
float totalShifts = 0;
float totalOrs = 0;
float totalNegs = 0;
float totalMovs = 0;

/* average */
float avgMultsPerSample = 0;
float avgAddsPerSample = 0;
float avgShiftsPerSample = 0;
float avgOrsPerSample = 0;
float avgNegsPerSample = 0;
float avgMovsPerSample = 0;

/* max */
float maxMultsPerSample = 0;
float maxAddsPerSample = 0;
float maxShiftsPerSample = 0;
float maxOrsPerSample = 0;
float maxNegsPerSample = 0;
float maxMovsPerSample = 0;
#endif


static void swapInt(int* x, int* y)
{
  int dummy;

  dummy = *x;
  *x = *y;
  *y = dummy;

#ifdef COUNT_32BIT_INSTRUCTIONS
  cntMovs += 2;
#endif
}


/* Inverse IntMDCT:
   Processed buffer has length N+N/2.
   The first N/2 values contain TDA+windowed signal from last frame.
   The next N signal contain N MDCT lines. These N lines are inverse DCT-IV
   transformed, minused and mirrored.
   Inverse TDA+windowing is done on the left N samples.
   The right N/2 should be stored for next frame.


   < N/2 TDA+w > <     N MDCT lines        >

                              |
                          inv DCT-IV
                   minus, mirror
                        |

   < N/2 TDA+w > < N/2 TDA+w > < N/2 TDA+w >

                |
      inv TDA+w
          |

   <      N time samples     > < N/2 TDA+w >

*/
void InvIntMDCT(int *p_in,
		int *p_overlap,
		int *p_out,
		byte blockType,
		byte windowShape,
		int osf)
{
  int i;
  int j;
  int N;
  int transFac = 8;
  int NShort;
  int startOnesLength;
  int signal[MAX_OSF*INTMDCT_BUFFER_SIZE];

#ifdef COUNT_CYCLES
  __int64 tm = rdtsc();
#endif

#ifdef COUNT_32BIT_INSTRUCTIONS_NEW
  cntMults = 0;
  cntAdds = 0;
  cntShifts = 0;
  cntOrs = 0;
  cntNegs = 0;
  cntMovs = 0;
#endif

  N = 1024*osf;
  NShort = N/transFac;
  startOnesLength = N/2 - NShort/2;

  /* copy buffers */
  for (i=0; i<N/2; i++) {
    signal[i]=p_overlap[i];
  }
  for (i=0; i<N; i++) {
    signal[N/2+i]=p_in[i];
  }
#ifdef COUNT_32BIT_INSTRUCTIONS
  cntMovs += N/2 + N;
#endif

  /* Inverse DCT-IV */
  switch(blockType){
  case ONLY_LONG_WINDOW:
  case LONG_START_WINDOW:
  case LONG_STOP_WINDOW:
    /* perform inverse Integer DCT-IV on right N samples */
    invIntDCTIV(&signal[N/2],
		&signal[N],
		N,
		1);

    /* revert order of right N samples */
    for (i=0; i<N/2; i++) {
      swapInt(&signal[N/2+i],&signal[N/2+N-1-i]);
    }
    for (i=0; i<N; i++) {
      signal[N/2+i] *= -1;
    }
#ifdef COUNT_32BIT_INSTRUCTIONS
    cntNegs += N;
#endif
    break;
  case EIGHT_SHORT_WINDOW:
    for (j=0; j<transFac; j++) {
      /* perform inverse Integer DCT-IV on right N samples */
      invIntDCTIV(&signal[N/2+j*NShort],
		  &signal[N/2+j*NShort+NShort/2],
		  NShort,
		  1);
      /* revert order of right N samples */
      for (i=0; i<NShort/2; i++) {
        swapInt(&signal[N/2+j*NShort+i],&signal[N/2+j*NShort+NShort-1-i]);
      }
      for (i=0; i<NShort; i++) {
        signal[N/2+j*NShort+i] *= -1;
      }
#ifdef COUNT_32BIT_INSTRUCTIONS
      cntNegs += NShort;
#endif
    }
    break;
  default:
    printf("blockType not implemented!\n");
    break;
  }

  /* perform inverse TDA and windowing (sine window) of left N samples
     based on Integer Rotations */
  switch(blockType){
  case ONLY_LONG_WINDOW:
  case LONG_START_WINDOW:
    windowingTDA(signal,
		 N,
		 windowShape,
		 -1);
    break;
  case LONG_STOP_WINDOW:
    windowingTDA(&signal[startOnesLength],
		 NShort,
		 windowShape,
		 -1);
    break;
  case EIGHT_SHORT_WINDOW:
    for (j=0; j<transFac; j++) {
      windowingTDA(&signal[startOnesLength+j*NShort],
		   NShort,
		   windowShape,
		   -1);
    }
    break;
  default:
    printf("blockType not implemented!\n");
    break;
  }

  /* copy buffers */
  for (i=0; i<N; i++) {
    p_out[i]=signal[i];
  }
  for (i=0; i<N/2; i++) {
    p_overlap[i]=signal[N+i];
  }
#ifdef COUNT_32BIT_INSTRUCTIONS
  cntMovs += N/2 + N;
#endif

#ifdef COUNT_CYCLES
  /* process cycle stats: */
  cntCycles = rdtsc() - tm;
  totalCycles += cntCycles;
  totalSamples += N; /* mono */
  avgCyclesPerSample = (double) totalCycles / (double) totalSamples;
  if (maxCyclesPerSample < (double) cntCycles / (double) N)
    maxCyclesPerSample = (double) cntCycles / (double) N;
  /* dump stats: */
  printf("Cycles: %f (average), %f (max)\n", avgCyclesPerSample, maxCyclesPerSample);
#endif

#ifdef COUNT_32BIT_INSTRUCTIONS_NEW
  cntSamples += N; /* mono */
  totalMults += cntMults;
  totalAdds += cntAdds;
  totalShifts += cntShifts;
  totalOrs += cntOrs;
  totalNegs += cntNegs;
  totalMovs += cntMovs;

  avgMultsPerSample = totalMults / cntSamples;
  avgAddsPerSample = totalAdds / cntSamples;
  avgShiftsPerSample = totalShifts / cntSamples;
  avgOrsPerSample = totalOrs / cntSamples;
  avgNegsPerSample = totalNegs / cntSamples;
  avgMovsPerSample = totalMovs / cntSamples;

  if (maxMultsPerSample < cntMults/(N))
    maxMultsPerSample = cntMults/(N);
  if (maxAddsPerSample < cntAdds/(N))
    maxAddsPerSample = cntAdds/(N);
  if (maxShiftsPerSample < cntShifts/(N))
    maxShiftsPerSample = cntShifts/(N);
  if (maxOrsPerSample < cntOrs/(N))
    maxOrsPerSample = cntOrs/(N);
  if (maxNegsPerSample < cntNegs/(N))
    maxNegsPerSample = cntNegs/(N);
  if (maxMovsPerSample < cntMovs/(N))
    maxMovsPerSample = cntMovs/(N);
#endif
}



void InvStereoIntMDCT(int* p_in_0,
		      int* p_in_1,
		      int* p_overlap_0,
		      int* p_overlap_1,
		      int* p_out_0,
		      int* p_out_1,
		      byte blockType,
		      byte windowShape,
		      int osf)
{
  int i;
  int j;
  int N;
  int transFac = 8;
  int NShort;
  int startOnesLength;
  int signal[2][MAX_OSF*INTMDCT_BUFFER_SIZE];
  int ch;

#ifdef COUNT_CYCLES
  __int64 tm = rdtsc();
#endif

#ifdef COUNT_32BIT_INSTRUCTIONS_NEW
  cntMults = 0;
  cntAdds = 0;
  cntShifts = 0;
  cntOrs = 0;
  cntNegs = 0;
  cntMovs = 0;
#endif

  N = 1024*osf;
  NShort = N/transFac;
  startOnesLength = N/2 - NShort/2;

  /* copy buffers */
  for (i=0; i<N/2; i++) {
    signal[0][i]=p_overlap_0[i];
    signal[1][i]=p_overlap_1[i];
  }
  for (i=0; i<N; i++) {
    signal[0][N/2+i]=p_in_0[i];
    signal[1][N/2+i]=p_in_1[i];
  }
#ifdef COUNT_32BIT_INSTRUCTIONS
  cntMovs += 2 * (N/2 + N);
#endif

  /* Inverse DCT-IV */
  switch(blockType){
  case ONLY_LONG_WINDOW:
  case LONG_START_WINDOW:
  case LONG_STOP_WINDOW:
    /* perform inverse Integer DCT-IV on right N samples */
    invIntDCTIV(&signal[0][N/2],
		&signal[1][N/2],
		2*N,
		0);
    break;
  case EIGHT_SHORT_WINDOW:
    for (j=0; j<transFac; j++) {
      /* perform inverse Integer DCT-IV on right N samples */
      invIntDCTIV(&signal[0][N/2+j*NShort],
		  &signal[1][N/2+j*NShort],
		  2*NShort,
		  0);
    }
    break;
  default:
    printf("blockType not implemented!\n");
    break;
  }

  for (ch=0; ch<2; ch++) {
    /* Inverse DCT-IV */
    switch(blockType){
    case ONLY_LONG_WINDOW:
    case LONG_START_WINDOW:
    case LONG_STOP_WINDOW:

      /* revert order of right N samples */
      for (i=0; i<N/2; i++) {
        swapInt(&signal[ch][N/2+i],&signal[ch][N/2+N-1-i]);
      }
      for (i=0; i<N; i++) {
        signal[ch][N/2+i] *= -1;
      }
#ifdef COUNT_32BIT_INSTRUCTIONS
      cntNegs += N;
#endif
      break;
    case EIGHT_SHORT_WINDOW:
      for (j=0; j<transFac; j++) {
        /* revert order of right N samples */
        for (i=0; i<NShort/2; i++) {
          swapInt(&signal[ch][N/2+j*NShort+i],&signal[ch][N/2+j*NShort+NShort-1-i]);
        }
        for (i=0; i<NShort; i++) {
          signal[ch][N/2+j*NShort+i] *= -1;
        }
#ifdef COUNT_32BIT_INSTRUCTIONS
        cntNegs += NShort;
#endif
      }
      break;
    default:
      printf("blockType not implemented!\n");
      break;
    }
  }

  for (ch=0; ch<2; ch++) {
    /* perform inverse TDA and windowing (sine window) of left N samples
       based on Integer Rotations */
    switch(blockType){
    case ONLY_LONG_WINDOW:
    case LONG_START_WINDOW:
      windowingTDA(signal[ch],
		   N,
		   windowShape,
		   -1);
      break;
    case LONG_STOP_WINDOW:
      windowingTDA(&signal[ch][startOnesLength],
		   NShort,
		   windowShape,
		   -1);
      break;
    case EIGHT_SHORT_WINDOW:
      for (j=0; j<transFac; j++) {
        windowingTDA(&signal[ch][startOnesLength+j*NShort],
		     NShort,
		     windowShape,
		     -1);
      }
      break;
    default:
      printf("blockType not implemented!\n");
      break;
    }
  }

  /* copy buffers */
  for (i=0; i<N; i++) {
    p_out_0[i]=signal[0][i];
    p_out_1[i]=signal[1][i];
  }
  for (i=0; i<N/2; i++) {
    p_overlap_0[i]=signal[0][N+i];
    p_overlap_1[i]=signal[1][N+i];
  }
#ifdef COUNT_32BIT_INSTRUCTIONS
  cntMovs += 2 * (N/2 + N);
#endif

#ifdef COUNT_CYCLES
  /* process cycle stats: */
  cntCycles = rdtsc() - tm;
  totalCycles += cntCycles;
  totalSamples += 2*N; /* stereo */
  avgCyclesPerSample = (double) totalCycles / (double) totalSamples;
  if (maxCyclesPerSample < (double) cntCycles / (double) (2*N))
    maxCyclesPerSample = (double) cntCycles / (double) (2*N);
  /* dump stats: */
  printf("Cycles: %f (average), %f (max)\n", avgCyclesPerSample, maxCyclesPerSample);
#endif

#ifdef COUNT_32BIT_INSTRUCTIONS_NEW
  cntSamples += 2*N; /* stereo */

  totalMults += cntMults;
  totalAdds += cntAdds;
  totalShifts += cntShifts;
  totalOrs += cntOrs;
  totalNegs += cntNegs;
  totalMovs += cntMovs;

  avgMultsPerSample = totalMults / cntSamples;
  avgAddsPerSample = totalAdds / cntSamples;
  avgShiftsPerSample = totalShifts / cntSamples;
  avgOrsPerSample = totalOrs / cntSamples;
  avgNegsPerSample = totalNegs / cntSamples;
  avgMovsPerSample = totalMovs / cntSamples;

  if (maxMultsPerSample < cntMults/(2*N))
    maxMultsPerSample = cntMults/(2*N);
  if (maxAddsPerSample < cntAdds/(2*N))
    maxAddsPerSample = cntAdds/(2*N);
  if (maxShiftsPerSample < cntShifts/(2*N))
    maxShiftsPerSample = cntShifts/(2*N);
  if (maxOrsPerSample < cntOrs/(2*N))
    maxOrsPerSample = cntOrs/(2*N);
  if (maxNegsPerSample < cntNegs/(2*N))
    maxNegsPerSample = cntNegs/(2*N);
  if (maxMovsPerSample < cntMovs/(2*N))
    maxMovsPerSample = cntMovs/(2*N);
#endif
}

void countInstructionsPrint()
{
#ifdef COUNT_32BIT_INSTRUCTIONS_NEW
  printf("AVERAGE: mults: %.2f, adds: %.2f, shifts %.2f, ors: %.2f, negs: %.2f, movs: %.2f\n",
         avgMultsPerSample, avgAddsPerSample, avgShiftsPerSample, avgOrsPerSample, avgNegsPerSample, avgMovsPerSample);
  printf("MAX: mults: %.2f, adds: %.2f, shifts: %.2f, ors: %.2f, negs: %.2f, movs: %.2f\n",
         maxMultsPerSample, maxAddsPerSample, maxShiftsPerSample, maxOrsPerSample, maxNegsPerSample, maxMovsPerSample);
#endif
}
