/*******************************************************************************
This software module was originally developed by

Institute for Infocomm Research and Fraunhofer IIS

and edited by

-

in the course of development of ISO/IEC 14496 for reference purposes and its
performance may not have been optimized. This software module is an
implementation of one or more tools as specified by ISO/IEC 14496. ISO/IEC gives
You a royalty-free, worldwide, non-exclusive, copyright license to copy,
distribute, and make derivative works of this software module or modifications
thereof for use in implementations of ISO/IEC 14496 in products that satisfy
conformance criteria (if any). Those intending to use this software module in
products are advised that its use may infringe existing patents. ISO/IEC have no
liability for use of this software module or modifications thereof. Copyright is
not released for products that do not conform to audiovisual and image-coding
related ITU Recommendations and/or ISO/IEC International Standards.

#ifdef NOT_PUBLISHED

Assurance that the originally developed software module can be used (1) in
ISO/IEC 14496 once ISO/IEC 14496 has been adopted; and (2) to develop ISO/IEC
14496:
Institute for Infocomm Research and Fraunhofer IIS grant ISO/IEC all
rights necessary to include the originally developed software module or
modifications thereof in ISO/IEC 14496 and to permit ISO/IEC to offer You a
royalty-free, worldwide, non-exclusive, copyright license to copy, distribute,
and make derivative works for use in implementations of ISO/IEC 14496 in
products that satisfy conformance criteria (if any), and to the extent that such
originally developed software module or portions of it are included in ISO/IEC
14496. To the extent that Institute for Infocomm Research and Fraunhofer IIS 
own patent rights that would be required to make, use, or sell the
originally developed software module or portions thereof included in ISO/IEC
14496 in a conforming product, Institute for Infocomm Research and Fraunhofer IIS 
will assure the ISO/IEC that they are willing to negotiate
licenses under reasonable and non-discriminatory terms and conditions with
applicants throughout the world. ISO/IEC gives You a free license to this
software module or modifications thereof for the sole purpose of developing
ISO/IEC 14496.

#endif

Institute for Infocomm Research and Fraunhofer IIS retain full right to
modify and use the code for their own purpose, assign or donate the code
to a third party and to inhibit third parties from using the code for products
that do not conform to MPEG-related ITU Recommendations and/or ISO/IEC
International Standards. This copyright notice must be included in all copies or
derivative works.

Copyright (c) ISO/IEC 2005.
*******************************************************************************/


/******************************************************************** 
/
/ filename : lit_ll_dec.c
/ project : MPEG-4 audio scalable lossless coding
/ author : RSY, Institute for Infocomm Research <rsyu@i2r.a-star.edu.sg>
/ date : Oct., 2003
/ contents/description : AC/BPGC decoding, inverse error mapping
/    

*/

#include <stdlib.h>//SLS_BSAC

#include "lit_ll_dec.h"
#include "all.h"
#include "port.h"
#include "acod.h"
#include "cbac.h"
#include "int_compact_tables.h"


#define MDCT_SCALE 118
#define MDCT_SCALE_MS 116 
#define MDCT_SCALE_SHORT 112
#define MDCT_SCALE_SHORT_MS 110



#define BPGC_LAZY 4
#define CBAC_LAZY 6

enum {
	ExplicitBand = 0,
	ImplicitBand ,
	ForcedExplicitBand
};

#define GARBAGE_BIT 14

static short scale_pcm[4] = {0, 0, 4, 8}; /* 8,16,20,24 bit PCM input */

#define MAX_QUANT 8192


static unsigned int bpc_mask[32]=
{ 0x00000001,0x00000002,0x00000004,0x00000008,0x00000010,0x00000020,0x00000040,0x00000080,
  0x00000100,0x00000200,0x00000400,0x00000800,0x00001000,0x00002000,0x00004000,0x00008000,
  0x00010000,0x00020000,0x00040000,0x00080000,0x00100000,0x00200000,0x00400000,0x00800000,
  0x01000000,0x02000000,0x04000000,0x08000000,0x10000000,0x20000000,0x40000000,0x80000000
};	


static int bpgc_reconst[5] = {0x007FFFFF,0x006AAAAB,0x4EEEEF,0x2EFEFF,0x17FF00}; /*24bit fixed-point bpgc reconstruction value*/

static int silence_freq[4][3] = {{12603,7447,6302},{9638,3344,745},{6554,1820,552},{3810,16384,16384}};
static int half_freq = 8192;



void calc_sfb_offset_table(LIT_LL_quantInfo *quantInfo,
                           int osf)
{
  int grp, sfb;

  quantInfo->num_window_groups = quantInfo->info->num_groups;
  quantInfo->num_sfb = quantInfo->info->sfb_per_sbk[0];
  if (quantInfo->info->islong) {
    quantInfo->num_osf_sfb = (osf-1)*16;
    quantInfo->osf_sfb_width = 64;
  } else {
    quantInfo->num_osf_sfb = (osf-1)*4;
    quantInfo->osf_sfb_width = 32;
  }

  quantInfo->aac_sfb_offset[0] = 0;
  quantInfo->sls_sfb_offset[0] = 0;


  for (grp=0; grp<quantInfo->num_window_groups; grp++) {
    for (sfb=0; sfb<quantInfo->num_sfb; sfb++) {
      int sfb_aac = grp * quantInfo->num_sfb + sfb;
      quantInfo->aac_sfb_offset[sfb_aac+1] = quantInfo->info->bk_sfb_top[sfb_aac];
    }
  }

  for (grp=0; grp<quantInfo->num_window_groups; grp++) {
    for (sfb=0; sfb<quantInfo->num_sfb; sfb++) {
      int sfb_aac = grp * quantInfo->num_sfb + sfb;
      int sfb_sls = grp * quantInfo->num_sfb + grp * quantInfo->num_osf_sfb + sfb;

      quantInfo->sls_sfb_offset[sfb_sls+1] = 
        quantInfo->sls_sfb_offset[sfb_sls]
        + quantInfo->aac_sfb_offset[sfb_aac+1] 
        - quantInfo->aac_sfb_offset[sfb_aac];     
    }
    for (sfb=0; sfb<quantInfo->num_osf_sfb; sfb++) {
      int sfb_sls = (grp+1) * quantInfo->num_sfb + grp * quantInfo->num_osf_sfb + sfb;
      quantInfo->sls_sfb_offset[sfb_sls+1] =
        quantInfo->sls_sfb_offset[sfb_sls]
        + quantInfo->info->group_len[grp] * quantInfo->osf_sfb_width;
    }
  }

  for (grp=0; grp<quantInfo->num_window_groups; grp++) {
    for (sfb=0; sfb<quantInfo->num_sfb; sfb++) {
      int sfb_aac = grp * quantInfo->num_sfb + sfb;
      quantInfo->aac_sfb_width[sfb_aac] = 
        quantInfo->aac_sfb_offset[sfb_aac+1] 
        - quantInfo->aac_sfb_offset[sfb_aac];
    }
  }

  for (grp=0; grp<quantInfo->num_window_groups; grp++) {
    for (sfb=0; sfb<quantInfo->num_sfb+quantInfo->num_osf_sfb; sfb++) {
      int sfb_sls = grp * quantInfo->num_sfb + grp * quantInfo->num_osf_sfb + sfb;
      quantInfo->sls_sfb_width[sfb_sls] = 
        quantInfo->sls_sfb_offset[sfb_sls+1] 
        - quantInfo->sls_sfb_offset[sfb_sls];
    }
  }
}


void lit_ll_deinterleave(int inptr[], int outptr[], byte * group, LIT_LL_quantInfo *quantInfo)
{
  int i, j, k, l;
  int *start_inptr, *start_subgroup_ptr, *subgroup_ptr;
  short cell_inc, subgroup_inc;
  short ngroups,nsubgroups[8];
  int ncells;
  short cellsize[MAXBANDS];

  ncells = quantInfo->info->sfb_per_sbk[0];
  for (j=0; j<ncells; j++) {
    cellsize[j] = quantInfo->info->sfb_width_128[j];
  }
  for (j=ncells; j<ncells+quantInfo->num_osf_sfb; j++) {
    cellsize[j] = quantInfo->osf_sfb_width;
  }
  ncells += quantInfo->num_osf_sfb;
  
  j = 0;
  i = 0;
  do  {
    nsubgroups[i]=group[i]-j;
    j=group[i];
    i++;
  } while (j<8);
  ngroups=i;
	

  start_subgroup_ptr = outptr;

  for (i = 0; i < ngroups; i++)
    {
      cell_inc = 0;
      start_inptr = inptr;

      /* Compute the increment size for the subgroup pointer */

      subgroup_inc = 0;
      for (j = 0; j < ncells; j++) {
        subgroup_inc += cellsize[j];
      }

      /* Perform the deinterleaving across all subgroups in a group */

      for (j = 0; j < ncells; j++) {
        subgroup_ptr = start_subgroup_ptr;

        for (k = 0; k < nsubgroups[i]; k++) {
          outptr = subgroup_ptr + cell_inc;
          for (l = 0; l < cellsize[j]; l++) {
            *outptr++ = *inptr++;
          }
          subgroup_ptr += subgroup_inc;
        }
        cell_inc += cellsize[j];
      }
      start_subgroup_ptr += (inptr - start_inptr);
    }
}



void LIT_ll_Dec(LIT_LL_Info *ll_Info,
                LIT_LL_quantInfo *quantInfo,
                byte * mask,
                byte hasmask,
                byte ** cb_mask,
                int *rec_spectrum,
                Ch_Info *channelInfo,
                int sampling_rate,
                int osf,
                short * block_type_lossless_only,
                int *msStereoIntMDCT,
                int aac_core_present,
                int ch,
                byte * group
                )
{
  int i,j,ii,grp,sfb;  
  int band_type[MAXBANDS];
  int sig_rec[MAX_OSF*1024];
  int ref[MAX_OSF*1024];
  int quant_aac[MAX_OSF*1024];
  int init,m0,m;
  int frame_type;
  ac_coder coder;
  int p_significant_error[MAX_OSF*1024],p_significant_error_bak[MAX_OSF*1024];
  int	*p_bpc_L = ll_Info->bpc_L; /* lazy layer for BPC */
  int	*p_bpc_maxbitplane = ll_Info->bpc_maxbitplane; /* maximum bit plane for BPC */
  int bpc_interval[MAX_OSF*1024];
  int significant_band[MAXBANDS];
  int band_type_signaling;
  int *ptr_book_vector=quantInfo->table; 
  int* scale_factor = quantInfo->scale_factor; 
  int noise_floor_reached;
  long total_bits = ll_Info->stream_len*8;
  int isTrunc;
  int p_significant_core[MAX_OSF*1024];
  int amp_rec[MAX_OSF*1024];
  int is_lazy;
  int is_first_channel;
  int sfb_aac, sfb_sls, i_aac, i_sls, s, cur_bp[MAX_OSF*1024], distance_to_lazy[MAX_OSF*1024],lazy_bp, sym, bin, ch_tmp;
  int overall_scale[MAXBANDS]; 

  is_first_channel = ((channelInfo->cpe && channelInfo->ch_is_left) || (!channelInfo->cpe))?1:0;

  memset(ref,0,1024*osf*sizeof(int));
  memset(quant_aac,0,1024*osf*sizeof(int));
  
  memset(p_significant_error,0,1024*osf*sizeof(int));

  ac_decoder_init(&coder,ll_Info->stream,total_bits);
  
  if (is_first_channel)
  {
     read_bits(&coder, 4); /*element instance tag*/
  }
  
  read_bits(&coder,1); /* lle_reserved_bit */
  
  if (channelInfo->cpe && channelInfo->common_window && channelInfo->ch_is_left) {  
     *msStereoIntMDCT = read_bits(&coder,1);
  }

  if (aac_core_present) {
 
    for (grp=0; grp<quantInfo->num_window_groups; grp++) {
      for (sfb=0; sfb<quantInfo->num_sfb+quantInfo->num_osf_sfb; sfb++) {
        sfb_sls = grp * quantInfo->num_sfb + grp * quantInfo->num_osf_sfb + sfb;
        
        if (sfb < quantInfo->max_sfb) {
          sfb_aac = grp * quantInfo->num_sfb + sfb;
          ch_tmp = ch;
          band_type[sfb_sls] = (ptr_book_vector[sfb_aac])? ImplicitBand:ExplicitBand;
          if (cb_mask != NULL) {
            if ((cb_mask[ch][sfb_aac] == NOISE_HCB) 
                || (cb_mask[ch][sfb_aac] == NOISE_HCB+100) ) 
              band_type[sfb_sls] = ExplicitBand;
            if (channelInfo->ch_is_left)
              ch_tmp = channelInfo->paired_ch;
            if ((cb_mask[ch_tmp][sfb_aac] == INTENSITY_HCB) 
                || (cb_mask[ch_tmp][sfb_aac] == INTENSITY_HCB2) )
              band_type[sfb_sls] = ExplicitBand;
          }
        } else {
          band_type[sfb_sls] = ExplicitBand;
        }
      }
    }
    
    band_type_signaling = read_bits(&coder,2);
    
    if (band_type_signaling == 1) {
      
      for (grp=0; grp<quantInfo->num_window_groups; grp++) {
        for (sfb=0; sfb<quantInfo->max_sfb; sfb++) {
          sfb_sls = grp * quantInfo->num_sfb + grp * quantInfo->num_osf_sfb + sfb;
      
          if (!read_bits(&coder,1)) {
            if (band_type[sfb_sls] == ImplicitBand) {
              band_type[sfb_sls] = ForcedExplicitBand;
            }
          }
        }	
      }

    } else if (band_type_signaling == 2) {
	
      for (grp=0; grp<quantInfo->num_window_groups; grp++) {
        for (sfb=0; sfb<quantInfo->max_sfb; sfb++) {
          sfb_sls = grp * quantInfo->num_sfb + grp * quantInfo->num_osf_sfb + sfb;
      
          if (band_type[sfb_sls] == ImplicitBand) {
            band_type[sfb_sls] = ForcedExplicitBand;
          }
        }	
      }  
    }

  } else /* no core */ { 
#ifdef BSAC
     for (grp=0; grp<quantInfo->num_window_groups; grp++) {
        for (sfb=0; sfb<quantInfo->max_sfb; sfb++) {
          quantInfo->table[grp*quantInfo->max_sfb + sfb] = 1;//SLS_BSAC book_vector
        }	
      }  
#endif

    if (is_first_channel) {
      *block_type_lossless_only = read_bits(&coder,2);
    }

    /* retrieve default data for info structure
       in core mode this is done by huffdecode() */
    memcpy(quantInfo->info, winmap[*block_type_lossless_only], sizeof(Info));
    
    if (*block_type_lossless_only == EIGHT_SHORT_WINDOW) { 
      quantInfo->info->num_groups = 4;
      for (i=0; i<4; i++) {
        quantInfo->info->group_len[i] = 2;
        group[i] = 2*(i+1);
      }
      for (i=0; i<4*quantInfo->info->sfb_per_sbk[0]; i++) {
        quantInfo->info->bk_sfb_top[i] *= 2;
        quantInfo->info->bk_sfb_top[4*quantInfo->info->sfb_per_sbk[0]+i] = 0;
      }
    }

    calc_sfb_offset_table(quantInfo,
                          osf);

    for (grp=0; grp<quantInfo->num_window_groups; grp++) {
      for (sfb=0; sfb<quantInfo->num_sfb+quantInfo->num_osf_sfb; sfb++) {
        sfb_sls = grp * quantInfo->num_sfb + grp * quantInfo->num_osf_sfb + sfb;
        band_type[sfb_sls] = ExplicitBand;
      }
    }
  }

  for (i=0; i<1024; i++) {
    quant_aac[i] = quantInfo->quantCoef[i];
  }
  
  i=0;



  for (grp=0; grp<quantInfo->num_window_groups; grp++) {
    for (sfb=0; sfb<quantInfo->num_sfb; sfb++) {
      int sfb_aac = grp * quantInfo->num_sfb + sfb;
      int sfb_sls = grp * quantInfo->num_sfb + grp * quantInfo->num_osf_sfb + sfb;
      int dummy=0;
      int thr,thr1;
      int interval;
      int mdct_scale;
      int scale_osf;

      if ((quantInfo->info->islong)) {
        if (hasmask && (mask != NULL)) /* M/S L/R compensation */
          mdct_scale = (mask[sfb_aac])? MDCT_SCALE_MS:MDCT_SCALE;
        else
          mdct_scale = MDCT_SCALE;
      }	else {
        if (hasmask && (mask != NULL)) /* M/S L/R compensation */
          mdct_scale = (mask[sfb_aac])? MDCT_SCALE_SHORT_MS:MDCT_SCALE_SHORT;
        else
          mdct_scale = MDCT_SCALE_SHORT;
      }

      scale_osf = 0;
      if (osf>1) scale_osf = osf;

      overall_scale[sfb_aac] = 
        scale_factor[sfb_aac] 
        - mdct_scale 
        + scale_pcm[ll_Info->type_PCM]*4
        + scale_osf;
      
      for (i=0; i<quantInfo->sls_sfb_width[sfb_sls]; i++) {		
        i_aac = quantInfo->aac_sfb_offset[sfb_aac]+i;
        i_sls = quantInfo->sls_sfb_offset[sfb_sls]+i;
      
        if (quant_aac[i_aac] == 0) {
	
          p_significant_error[i_sls] = 0;
          if (band_type[sfb_sls] == ImplicitBand) /* SIG band */ {
            interval = thrFunction(0, overall_scale[sfb_aac]);
            interval++;
            bpc_interval[i_sls] = interval;
            for (j=0;(1<<(j+1))<interval;j++);
          } else  /* INSIG band*/ { 
            bpc_interval[i_sls] = 0x7fffffff;
          }
          ref[i_sls] = 0;
	
        } else  /* quant_aac[i] != 0 */ {

          thr = thrFunction(abs(quant_aac[i_aac])-1, overall_scale[sfb_aac]);
          thr1 = thrFunction(abs(quant_aac[i_aac]), overall_scale[sfb_aac]);

          if (band_type[sfb_sls] == ImplicitBand) {
            interval = thr1 - thr+1;
            bpc_interval[i_sls] = interval;
            ref[i_sls] = thr;
            for (j=0;(1<<(j+1))<interval;j++);
            p_significant_error[i_sls] = 1;
          } else if (band_type[sfb_sls] == ForcedExplicitBand)  {
            bpc_interval[i_sls] = 0x7fffffff;
            ref[i_sls] = invQuantFunction(abs(quant_aac[i_aac]), overall_scale[sfb_aac]);
            p_significant_error[i_sls] = 0;
          } else /* insig band*/ {
            bpc_interval[i_sls] = 0x7fffffff;
            ref[i_sls] = 0;
            p_significant_error[i_sls] = 0;
          }
        }

        if (band_type[sfb_sls] == ImplicitBand) {
          if (dummy<j) dummy = j;
        }  
      } /* for (; i<sfb_offset[s]; i++) */
    
      if (band_type[sfb_sls] == ImplicitBand) {
        p_bpc_maxbitplane[sfb_sls] = dummy;
        significant_band[sfb_sls] = 1;
      } else  /* INSIG band */ {
        significant_band[sfb_sls] = 0;
      }

    } 
  } /* for (s = 0; s < total_sfb+num_osf_sfbs; s++) */
  
  /* oversampling range */
  for (grp=0; grp<quantInfo->num_window_groups; grp++) {
    for (sfb=quantInfo->num_sfb; 
         sfb<quantInfo->num_sfb+quantInfo->num_osf_sfb; sfb++) {
      sfb_sls = grp * quantInfo->num_sfb + grp * quantInfo->num_osf_sfb + sfb;
      band_type[sfb_sls] = ExplicitBand;
      significant_band[sfb_sls] = 0;
      for (i=quantInfo->sls_sfb_offset[sfb_sls]; 
           i<quantInfo->sls_sfb_offset[sfb_sls+1]; i++) {		
        bpc_interval[i] = 0x7fffffff;
        ref[i] = 0;
        p_significant_error[i] = 0;
      }

    }
  }

  for (grp=0; grp<quantInfo->num_window_groups; grp++) {
    init = 0;
    for (sfb=0; sfb<quantInfo->num_sfb+quantInfo->num_osf_sfb; sfb++) {
      s = grp * quantInfo->num_sfb + grp * quantInfo->num_osf_sfb + sfb;
      if (band_type[s] != ImplicitBand) {
        if (!init) {
          p_bpc_maxbitplane[s] = read_bits(&coder,5) - 1;
          init ++;
        } else {
          m = 0;
          while (read_bits(&coder,1) == 0)
            m++;
          if (m) {
            if (read_bits(&coder,1))
              m = -m;
          }
          p_bpc_maxbitplane[s] = m0 - m;
        }
        m0 = p_bpc_maxbitplane[s];
      }
      if (p_bpc_maxbitplane[s]>=0) {
        if (read_bits(&coder,1)==0) {
          p_bpc_L[s] = 2;
        } else {
          if (read_bits(&coder,1)==0)
            p_bpc_L[s] = 1;
          else
            p_bpc_L[s] = 3;
        }
      }
    }
  }

  isTrunc = 0;
  frame_type = read_bits(&coder,1);

  for (grp=0; grp<quantInfo->num_window_groups; grp++)
      for (sfb=0; sfb<quantInfo->num_sfb+quantInfo->num_osf_sfb; sfb++) {
		  s = grp * quantInfo->num_sfb + grp * quantInfo->num_osf_sfb + sfb;
 	      for (i=quantInfo->sls_sfb_offset[s]; i<quantInfo->sls_sfb_offset[s+1]; i++) {
			  cur_bp[i] = p_bpc_maxbitplane[s];
			  distance_to_lazy[i] = p_bpc_L[s]+1;  
		      p_significant_error_bak[i] = p_significant_core[i] = p_significant_error[i];
			  amp_rec[i] = 0;
			  sig_rec[i] = 1;
		  }
	  }

  ii = 0;
  noise_floor_reached = 0;
  is_lazy = 0;
  while(!noise_floor_reached) {
    noise_floor_reached = 1;
    for (grp=0; grp<quantInfo->num_window_groups; grp++)
      for (sfb=0; sfb<quantInfo->num_sfb+quantInfo->num_osf_sfb; sfb++) {
        s = grp * quantInfo->num_sfb + grp * quantInfo->num_osf_sfb + sfb;
        if ( (p_bpc_maxbitplane[s] >= ii) && (p_bpc_maxbitplane[s] - p_bpc_L[s] > 0) ) {
          
          noise_floor_reached = 0;
          for (i=quantInfo->sls_sfb_offset[s]; i<quantInfo->sls_sfb_offset[s+1]; i++) {
            int freq;
            int int_cx;

 
            if (bpc_interval[i] > amp_rec[i]+ bpc_mask[cur_bp[i]])  /*null context int_cx = 2*/ {
              int_cx = (bpc_interval[i] >= amp_rec[i]+ 2*bpc_mask[cur_bp[i]]) ? 1:0 ;
              freq = cbac_model(i,
                                frame_type,
                                sampling_rate,
                                distance_to_lazy[i],
                                p_significant_error_bak,
                                significant_band[s],
                                int_cx,
                                osf);
              sym = ac_decode_symbol (&coder, freq);
              amp_rec[i] += sym*bpc_mask[cur_bp[i]];
              if ((p_significant_error[i]==0) && sym) {
                sym = ac_decode_symbol (&coder, half_freq);
                sig_rec[i] = (sym)? -1:1;
                p_significant_error[i] = 1;
              }
              if (ac_bits(&coder)  >= total_bits) 
                {
                  goto CBAC_END;
                }
              
            } 
    		cur_bp[i]--;
		    distance_to_lazy[i]--;
         } /*for (i=sfb_offset[s]; i<sfb_offset[s+1]; i++)*/
        } /*if (p_bpc_maxbitplane[s] >= ii)*/
      }	/*	for (s=0;s<total_sfb;s++)*/
    memcpy(p_significant_error_bak, 
           p_significant_error,
           1024*osf*sizeof(int));
    
    ii++;
    /* LAZY_CODING */
    if ( ( (ii==CBAC_LAZY) && (frame_type==0) ) || ( (ii==BPGC_LAZY) && (frame_type==1) ) ) {
      is_lazy = 1;
      goto LAZYMODE_CODING;
    }

  } /* while*/
  
LAZYMODE_CODING:

  if (is_lazy) {
    read_bits(&coder,2); /*skip terminationg string*/
    while(!noise_floor_reached) {
      noise_floor_reached = 1;
      for (grp=0; grp<quantInfo->num_window_groups; grp++)
        for (sfb=0; sfb<quantInfo->num_sfb+quantInfo->num_osf_sfb; sfb++) {
          s = grp * quantInfo->num_sfb + grp * quantInfo->num_osf_sfb + sfb;
          if ( (p_bpc_maxbitplane[s] >= ii) && (p_bpc_maxbitplane[s] - p_bpc_L[s] > 0) ) {
            noise_floor_reached = 0;
            for (i=quantInfo->sls_sfb_offset[s]; i<quantInfo->sls_sfb_offset[s+1]; i++) {
              if  (bpc_interval[i] > amp_rec[i]+ bpc_mask[cur_bp[i]]) /*null context int_cx = 2*/ {
                sym = read_bits(&coder,1);
                amp_rec[i] += sym*bpc_mask[cur_bp[i]];
                if ((p_significant_error[i]==0) && sym) {
                  sym = read_bits(&coder,1);
                  sig_rec[i] = (sym)? -1 : 1;
                  p_significant_error[i] = 1;
                }
                if (ac_bits(&coder)  > total_bits+GARBAGE_BIT) 
                  {
                    isTrunc = 1; 
                    goto FILL_ELEMENT;
                  }
              } 
			  cur_bp[i]--;
			  distance_to_lazy[i]--;
            } /*for (i=sfb_offset[s]; i<sfb_offset[s+1]; i++)*/
          } /*if (p_bpc_maxbitplane[s] >= ii)*/
        }		/*	for (s=0;s<total_sfb;s++)*/
    
      ii++;
    } /* while*/
  }

FILL_ELEMENT:
  
  if (isTrunc) {
    for (grp=0; grp<quantInfo->num_window_groups; grp++)
      for (sfb=0; sfb<quantInfo->num_sfb+quantInfo->num_osf_sfb; sfb++) {
        s = grp * quantInfo->num_sfb + grp * quantInfo->num_osf_sfb + sfb;
        if ( (p_bpc_maxbitplane[s] >= ii) && (p_bpc_maxbitplane[s] - p_bpc_L[s] > 0) ) {
	  
          for (i=quantInfo->sls_sfb_offset[s]; i<quantInfo->sls_sfb_offset[s+1]; i++) {

  			  if (distance_to_lazy[i] <0) distance_to_lazy[i] = 0;

            if ((p_significant_error[i]) /*(s>0) && (i != sfb_offset[s-1])*/) {
				if (amp_rec[i] >= 0){ 
					amp_rec[i] += (bpgc_reconst[distance_to_lazy[i]] >> (23-cur_bp[i])) ;
              }
            }
          } /*for (i=sfb_offset[s]; i<sfb_offset[s+1]; i++)*/
        } /*if (p_bpc_maxbitplane[s] >= ii)*/
      }		/*	for (s=0;s<total_sfb;s++)*/  
  }

  /* LOW ENERGY MODE */

  if (!isTrunc)
  {		
    for (grp=0; grp<quantInfo->num_window_groups; grp++)
      for (sfb=0; sfb<quantInfo->num_sfb+quantInfo->num_osf_sfb; sfb++) {
        s = grp * quantInfo->num_sfb + grp * quantInfo->num_osf_sfb + sfb;
        lazy_bp = p_bpc_maxbitplane[s] - p_bpc_L[s];
        if ((p_bpc_maxbitplane[s] >= 0) && (lazy_bp <= 0)) {
          for (i=quantInfo->sls_sfb_offset[s]; i<quantInfo->sls_sfb_offset[s+1]; i++) {
            bin = 0;
            sym = 0;
	  
            while (ac_decode_symbol(&coder,silence_freq[-lazy_bp][bin])==1) {
              sym++;
              if (sym==(1<<(p_bpc_maxbitplane[s]+1))-1) break;
              bin++;
              if (bin >2) bin = 2;
            }
            amp_rec[i] += sym;
            if ((p_significant_error[i] == 0)&&sym) {
              sig_rec[i] = (ac_decode_symbol(&coder,half_freq))? -1:1;
              p_significant_error[i] = 1;
            }
            if (ac_bits(&coder)  >= total_bits) {
              goto LERG_END;
            }
          }
        }
      }
  }

INVERSE_ERROR_MAPPING:

  
  for (grp=0; grp<quantInfo->num_window_groups; grp++) {
    for (sfb=0; sfb<quantInfo->num_sfb+quantInfo->num_osf_sfb; sfb++) {
      sfb_aac = grp * quantInfo->num_sfb + sfb;
      sfb_sls = grp * quantInfo->num_sfb + grp * quantInfo->num_osf_sfb + sfb;

      if (band_type[sfb_sls] == ExplicitBand) {
        for (i=quantInfo->sls_sfb_offset[sfb_sls]; 
             i<quantInfo->sls_sfb_offset[sfb_sls+1]; i++) {
          rec_spectrum[i] = amp_rec[i]*sig_rec[i];
        }
      } else {
        for (i=0; i<quantInfo->sls_sfb_width[sfb_sls]; i++) {		
          i_aac = quantInfo->aac_sfb_offset[sfb_aac]+i;
           i_sls = quantInfo->sls_sfb_offset[sfb_sls]+i;
          if (1/*(amp_rec[i_sls])||(!isTrunc)*/) {
            rec_spectrum[i_sls] = ref[i_sls] + amp_rec[i_sls]*sig_rec[i_sls];
          } else { 					
				rec_spectrum[i_sls] = invQuantFunction(abs(quant_aac[i_aac]), overall_scale[sfb_aac]);
          }
          rec_spectrum[i_sls] *=  (quant_aac[i_aac] >=0)? 1:-1;

        }
      } 
    }
  }
 
  return;

  	/* for ambiguity detection */
	while(!noise_floor_reached)
	{
		noise_floor_reached = 1;
    for (grp=0; grp<quantInfo->num_window_groups; grp++)
      for (sfb=0; sfb<quantInfo->num_sfb+quantInfo->num_osf_sfb; sfb++) {
        s = grp * quantInfo->num_sfb + grp * quantInfo->num_osf_sfb + sfb;
			if ((p_bpc_maxbitplane[s] >= ii) 
				&& (p_bpc_maxbitplane[s] - p_bpc_L[s] >0)
				)
			{
				
				noise_floor_reached = 0;
        for (i=quantInfo->sls_sfb_offset[s]; i<quantInfo->sls_sfb_offset[s+1]; i++) {
					int sym;
					int freq;
					int int_cx;
					
					if (bpc_interval[i] > amp_rec[i]+ bpc_mask[cur_bp[i]])  /*null context int_cx = 2*/
					{
						int_cx = (bpc_interval[i] >= amp_rec[i]+ 2*bpc_mask[cur_bp[i]])?1:0 ;
						freq = cbac_model(i,frame_type,sampling_rate,distance_to_lazy[i],p_significant_error_bak,significant_band[s],int_cx,osf);
						sym = ac_decode_symbol_with_ambiguity_check(&coder, freq);

						if(sym==-1)
						{
							isTrunc=1;
							goto FILL_ELEMENT;

						}

						amp_rec[i] += sym*bpc_mask[cur_bp[i]];	

						if ((p_significant_error[i]==0) && sym)
						{
							sym = ac_decode_symbol_with_ambiguity_check(&coder, half_freq);
							if(sym==-1)
							{
								isTrunc=1;
								amp_rec[i] = 0; /* discard decoded samples */
								goto FILL_ELEMENT;
							}
							sig_rec[i] = (sym)? -1:1;
		 					p_significant_error[i] = 1;
						}

CBAC_END:
						i=i;
					}
 				  cur_bp[i]--;
     			  distance_to_lazy[i]--;
				} /*for (i=sfb_offset[s]; i<sfb_offset[s+1]; i++)*/
			} /*if (p_bpc_maxbitplane[s] >= ii)*/
		}	/*	for (s=0;s<total_sfb;s++)*/
		memcpy(	p_significant_error_bak, p_significant_error,1024*osf*sizeof(int));

		ii++;

		if ( ((ii==CBAC_LAZY) &&(frame_type==0)) || ((ii==BPGC_LAZY) &&(frame_type==1)) )
		{
			is_lazy = 1;
			break;
		}

	} /* while*/

	if (is_lazy)
	{
		read_bits(&coder,2); /*skip terminationg string*/
		while(!noise_floor_reached)
		{
			noise_floor_reached = 1;
			for (grp=0; grp<quantInfo->num_window_groups; grp++)
        for (sfb=0; sfb<quantInfo->num_sfb+quantInfo->num_osf_sfb; sfb++) {
          s = grp * quantInfo->num_sfb + grp * quantInfo->num_osf_sfb + sfb;
				if ((p_bpc_maxbitplane[s] >= ii) 
					&& (p_bpc_maxbitplane[s] - p_bpc_L[s] >0)
					)
				{
					
					noise_floor_reached = 0;
          for (i=quantInfo->sls_sfb_offset[s]; i<quantInfo->sls_sfb_offset[s+1]; i++) {
						int sym;

						if  (bpc_interval[i] > amp_rec[i]+ bpc_mask[cur_bp[i]]) /*null context int_cx = 2*/
						{
							sym = read_bits(&coder,1);

							amp_rec[i] += sym*bpc_mask[cur_bp[i]];
							if ((p_significant_error[i]==0) && sym)
							{
								sym = read_bits(&coder,1);

								sig_rec[i] = (sym)? -1:1;
		 						p_significant_error[i] = 1;
							}
						}
						if (ac_bits(&coder)  > total_bits+GARBAGE_BIT) 
						{
							isTrunc = 1;
							goto FILL_ELEMENT;
						}
						cur_bp[i]--;
						distance_to_lazy[i]--;
					} /*for (i=sfb_offset[s]; i<sfb_offset[s+1]; i++)*/
				} /*if (p_bpc_maxbitplane[s] >= ii)*/
			}		/*	for (s=0;s<total_sfb;s++)*/
			ii++;

		} /* while*/
	}

	/*low energy mode decoding*/
	if (!isTrunc)
	{
    for (grp=0; grp<quantInfo->num_window_groups; grp++)
      for (sfb=0; sfb<quantInfo->num_sfb+quantInfo->num_osf_sfb; sfb++) {
        s = grp * quantInfo->num_sfb + grp * quantInfo->num_osf_sfb + sfb;
			lazy_bp = p_bpc_maxbitplane[s] - p_bpc_L[s];
			if ((p_bpc_maxbitplane[s] >= 0) && (lazy_bp <= 0))
			{
        for (i=quantInfo->sls_sfb_offset[s]; i<quantInfo->sls_sfb_offset[s+1]; i++) {
					int symTemp; 
					bin = 0;
					sym=0;				
					
					while (1) {
						symTemp = ac_decode_symbol_with_ambiguity_check (&coder, silence_freq[-lazy_bp][bin]);

						if(symTemp==-1)
						{
							isTrunc = 1;
							goto INVERSE_ERROR_MAPPING;
						}
						else if(symTemp==1) { 
							sym++;
							if (sym==(1<<(p_bpc_maxbitplane[s]+1))-1) 
								break;
							bin++;
							if (bin >2) 
								bin = 2;
						}
						else break;
					}
					amp_rec[i] += sym;
					if ((p_significant_error[i] == 0)&&sym)
					{
						symTemp = ac_decode_symbol_with_ambiguity_check (&coder, half_freq);
						if(symTemp==-1)
						{
							isTrunc = 1;
							amp_rec[i] = 0; /* discard decoded sample */
							goto INVERSE_ERROR_MAPPING;
						}
						sig_rec[i] = (symTemp)? -1:1;
		 				p_significant_error[i] = 1;
					}
LERG_END:
					i=i;
				}
			}
		}
	}
	goto INVERSE_ERROR_MAPPING;

}



#ifndef ADIF2MP4
int ll_byte_align(int ll_layer)
{
  int i=0;

  if(ll_nbits[ll_layer] <= 16){
    ll_cword[ll_layer] = (ll_cword[ll_layer]<<16) | ll_getshort(ll_layer);
    ll_nbits[ll_layer] += 16;
  }

  while (ll_nbits[ll_layer] & ((1<<3)-1)) {
    ll_nbits[ll_layer] -= 1;
    i += 1;
  }
  return(i);
}
#else

int ll_byte_align(int ll_layer)
{
  int i=0;

  if(ll_nbits <= 16){
    ll_cword = (ll_cword<<16) | ll_getshort(ll_layer);
    ll_nbits += 16;
  }
 
  while (ll_nbits & ((1<<3)-1)) {
    ll_nbits -= 1;
    i += 1;
  }
  return(i);
}
#endif




void LIT_ll_read(LIT_LL_Info *ll_Info, int is_left)
{
	int i;

	ll_byte_align(0);
	ll_Info->stream_len = ll_getbits(8,0);
	ll_Info->stream_len += ll_getbits(8,0)<<8;
	
	if (is_left)	
		ll_getbits(4,0);		/* element instance tag */

	for (i = 0; i<ll_Info->stream_len;i++)
		ll_Info->stream[i] = ll_getbits(8,0);
}

