/*******************************************************************************
This software module was originally developed by

Institute for Infocomm Research and Fraunhofer IIS

and edited by

-

in the course of development of ISO/IEC 14496 for reference purposes and its
performance may not have been optimized. This software module is an
implementation of one or more tools as specified by ISO/IEC 14496. ISO/IEC gives
You a royalty-free, worldwide, non-exclusive, copyright license to copy,
distribute, and make derivative works of this software module or modifications
thereof for use in implementations of ISO/IEC 14496 in products that satisfy
conformance criteria (if any). Those intending to use this software module in
products are advised that its use may infringe existing patents. ISO/IEC have no
liability for use of this software module or modifications thereof. Copyright is
not released for products that do not conform to audiovisual and image-coding
related ITU Recommendations and/or ISO/IEC International Standards.

#ifdef NOT_PUBLISHED

Assurance that the originally developed software module can be used (1) in
ISO/IEC 14496 once ISO/IEC 14496 has been adopted; and (2) to develop ISO/IEC
14496:
Institute for Infocomm Research and Fraunhofer IIS grant ISO/IEC all
rights necessary to include the originally developed software module or
modifications thereof in ISO/IEC 14496 and to permit ISO/IEC to offer You a
royalty-free, worldwide, non-exclusive, copyright license to copy, distribute,
and make derivative works for use in implementations of ISO/IEC 14496 in
products that satisfy conformance criteria (if any), and to the extent that such
originally developed software module or portions of it are included in ISO/IEC
14496. To the extent that Institute for Infocomm Research and Fraunhofer IIS 
own patent rights that would be required to make, use, or sell the
originally developed software module or portions thereof included in ISO/IEC
14496 in a conforming product, Institute for Infocomm Research and Fraunhofer IIS 
will assure the ISO/IEC that they are willing to negotiate
licenses under reasonable and non-discriminatory terms and conditions with
applicants throughout the world. ISO/IEC gives You a free license to this
software module or modifications thereof for the sole purpose of developing
ISO/IEC 14496.

#endif

Institute for Infocomm Research and Fraunhofer IIS retain full right to
modify and use the code for their own purpose, assign or donate the code
to a third party and to inhibit third parties from using the code for products
that do not conform to MPEG-related ITU Recommendations and/or ISO/IEC
International Standards. This copyright notice must be included in all copies or
derivative works.

Copyright (c) ISO/IEC 2005.
*******************************************************************************/




#ifndef DE_LOSSLESS_H
#define DE_LOSSLESS_H

#include "all.h"
#include "acod.h"

#include "cbac.h"
#include "int_mdct_defs.h"

#define MAX_SHORT_IN_LONG_BLOCK 8


typedef struct LIT_LL_Info_t
{
	int type_PCM; /* PCM types, 0/1/2/3 = 16/20/24/32 bits per sample*/

	int bpc_L[MAXBANDS];
	int bpc_maxbitplane[MAXBANDS];
	int bank_type[MAXBANDS];
	double CoreScaling;

	int core_significance_signaling;
	int core_significance[MAXBANDS];

	
	int bpc_sig_maxbitplane[MAX_OSF*BLOCK_LEN_LONG];
	int bpc_interval[MAX_OSF*BLOCK_LEN_LONG];
	int significant_error[MAX_OSF*BLOCK_LEN_LONG];
	int error_spectrum[MAX_OSF*BLOCK_LEN_LONG];
	int aac_abs_ref_spectrum[MAX_OSF*BLOCK_LEN_LONG];
	int significant_band[MAXBANDS];

	unsigned char *stream;
	long stream_len;
} LIT_LL_Info;


typedef struct LIT_LL_quantInfo_t
{
  Info *info; 
  /* NEW: use complete Info structure from AAC. 
     This might make several other values obsolte. Todo: check */

	int * quantCoef;
  int num_window_groups;
  int num_sfb;
  int num_osf_sfb;
  int total_sfb_per_group;
	int total_sfb;
	int table[MAXBANDS];
	int scale_factor[MAXBANDS];

  /*added in enhancer*/
  byte max_sfb;/*[MAX_TIME_CHANNELS];*/
  int osf_sfb_width;
  int sfb_offset_lossless_only_short[MAXBANDS];  /* todo: replace by general table */
  int aac_sfb_offset[MAXBANDS]; /* sfb_offset for AAC core layer */
  int sls_sfb_offset[MAXBANDS]; /* sfb_offset for SLS lossless enhancement layer (AAC bands + osf bands), todo: osf>1 */
  int aac_sfb_width[MAXBANDS]; 
  int sls_sfb_width[MAXBANDS]; 
} LIT_LL_quantInfo;

void calc_sfb_offset_table(LIT_LL_quantInfo *quantInfo,
                           int osf);

void LIT_ll_Dec(LIT_LL_Info *ll_Info,
                LIT_LL_quantInfo *quantInfo,
                byte * mask,
                byte hasmask,
                byte ** cb_mask,
                int *rec_spectrum,
                Ch_Info *channelInfo,
                int sampling_rate,
                int osf,
                short * block_type_lossless_only,
                int *msStereoIntMDCT,
                int aac_core_present,
                int ch,
                byte * group
                );

int ll_byte_align(int i);

void LIT_ll_read(LIT_LL_Info *ll_Info, int is_left);
void lit_ll_deinterleave(int inptr[], int outptr[], byte * group, LIT_LL_quantInfo *quantInfo);

void lle_main_element(	int channel_number	
                        ,Ch_Info *cip	
                        ,LIT_LL_Info **ll_info
                        /*					,bitfile *lz */
                        /*						,uint32_t *zbuffer_size */
						);

void lle_extension_element(	int channel_number	
                            ,Ch_Info *cip	
                            ,LIT_LL_Info **ll_info
                            /* 		,bitfile *lz */ 
                            /* 	,uint32_t *zbuffer_size */
                            ,int sls
                            ,int layer_num);

void lle_channel_pair_element(	int is_base_element
								,Ch_Info *cip
								,LIT_LL_Info **ll_info
							/* 	,bitfile *lz */ 
							/* 	,uint32_t *zbuffer_size */
								,int layer_num);

void lle_individual_channel_stream(	int is_base_element
									,int channel_is_left
									,Ch_Info *cip
									,LIT_LL_Info *ll_info
								/* 	,bitfile *lz */ 
								/* 	,uint32_t zbuffer_size */
									,int layer_num);

#endif
