/************************* MPEG-2 NBC Audio Decoder **************************
 *                                                                           *
"This software module was originally developed in the course of 
development of the MPEG-2 NBC/MPEG-4 Audio standard ISO/IEC 13818-7, 
14496-1,2 and 3. This software module is an implementation of a part of one or more 
MPEG-2 NBC/MPEG-4 Audio tools as specified by the MPEG-2 NBC/MPEG-4 
Audio standard. ISO/IEC  gives users of the MPEG-2 NBC/MPEG-4 Audio 
standards free license to this software module or modifications thereof for use in 
hardware or software products claiming conformance to the MPEG-2 NBC/MPEG-4
Audio  standards. Those intending to use this software module in hardware or 
software products are advised that this use may infringe existing patents. 
The original developer of this software module and his/her company, the subsequent 
editors and their companies, and ISO/IEC have no liability for use of this software 
module or modifications thereof in an implementation. Copyright is not released for 
non MPEG-2 NBC/MPEG-4 Audio conforming products.The original developer
retains full right to use the code for his/her  own purpose, assign or donate the 
code to a third party and to inhibit third party from using the code for non 
MPEG-2 NBC/MPEG-4 Audio conforming products. This copyright notice must
be included in all copies or derivative works." 
Copyright(c)1996.
 *                                                                           *
 ****************************************************************************/

/*******************************************************************************************
 *
 * MS stereo coding module
 *
 * Authors:
 * CL    Chuck Lueck, TI <lueck@ti.com>
 * RSY   Rongshan Yu, I2R
 *
 * Changes:
 * 22-jan-98   CL   Initial revision.  Lacks psychoacoustics for MS stereo.
 * 06-sep-98   CL   Disable MS stereo for bands which use PNS (MPEG4 only).
 * Oct-2003	   RSY  Modified for Scalable Lossless coding 
 ***************************************************************************************/
 
#include "lit_ms.h"
#include "int_win_dctiv.h"


void LL_MSEncode(int *p_spectrum_left, 
                 int *p_spectrum_right,
                 LIT_LL_quantInfo *quantInfo,
                 byte * mask,
                 byte hasmask,
                 int msStereoIntMDCT)
{  
  int grp, sfb, i;

  for (grp=0; grp<quantInfo->num_window_groups; grp++) {
    for (sfb=0; sfb<quantInfo->num_sfb; sfb++) {
      int sfb_aac = grp * quantInfo->num_sfb + sfb;
      int sfb_sls = grp * quantInfo->num_sfb + grp * quantInfo->num_osf_sfb + sfb;


      if (msStereoIntMDCT) {

        /* M/S is default for StereoIntMDCT */
        /* -> switch off M/S for L/R bands in encoder */
        if ( (!hasmask) || (hasmask && !mask[sfb_aac]) ) {
          for (i=quantInfo->sls_sfb_offset[sfb_sls]; i<quantInfo->sls_sfb_offset[sfb_sls+1]; i++) {
            InverseIntMidSideINT32(&p_spectrum_left[i],   /* M -> L */
                                   &p_spectrum_right[i]); /* S -> R */
          }
        }
        
      } else {
      
        /* L/R is default for MonoIntMDCT */
        /* -> switch on M/S for L/R bands in encoder */
        if ( hasmask && mask[sfb_aac] ) {
          for (i=quantInfo->sls_sfb_offset[sfb_sls]; i<quantInfo->sls_sfb_offset[sfb_sls+1]; i++) {
            IntMidSideINT32(&p_spectrum_left[i],   /* L -> M */
                            &p_spectrum_right[i]); /* R -> S */
          }
        }
      }
    }
  } 
}


void LL_MSDecode(int *p_spectrum_left, 
                 int *p_spectrum_right,
                 LIT_LL_quantInfo *quantInfo,
                 byte * mask,
                 byte hasmask,
                 int msStereoIntMDCT)
{  
  int grp, sfb, i;

  for (grp=0; grp<quantInfo->num_window_groups; grp++) {
    for (sfb=0; sfb<quantInfo->num_sfb; sfb++) {
      int sfb_aac = grp * quantInfo->num_sfb + sfb;
      int sfb_sls = grp * quantInfo->num_sfb + grp * quantInfo->num_osf_sfb + sfb;

      if (msStereoIntMDCT) {
        
        /* M/S is default for StereoIntMDCT */
        /* -> switch on M/S for L/R bands in decoder */
        if ( (!hasmask) || (hasmask && !mask[sfb_aac]) ) {
          for (i=quantInfo->sls_sfb_offset[sfb_sls]; i<quantInfo->sls_sfb_offset[sfb_sls+1]; i++) {
            IntMidSideINT32(&p_spectrum_left[i],   /* L -> M */
                            &p_spectrum_right[i]); /* R -> S */
          }
        }

      } else {
 
        /* L/R is default for MonoIntMDCT */
        /* -> switch off M/S for L/R bands in encoder */
        if ( hasmask && mask[sfb_aac] ) {
          for (i=quantInfo->sls_sfb_offset[sfb_sls]; i<quantInfo->sls_sfb_offset[sfb_sls+1]; i++) {
            InverseIntMidSideINT32(&p_spectrum_left[i],   /* M -> L */
                                   &p_spectrum_right[i]); /* S -> R */
          }
        }
        
      }
    }
  }
}

