/************************* MPEG-2 NBC Audio Decoder **************************
 *                                                                           *
"This software module was originally developed by 
AT&T, Dolby Laboratories, Fraunhofer Gesellschaft IIS 
and edited by
Yoshiaki Oikawa (Sony Corporation),
Mitsuyuki Hatanaka (Sony Corporation)
Mike Coleman (Five Bats Research)
in the course of development of the MPEG-2 NBC/MPEG-4 Audio standard ISO/IEC 13818-7, 
14496-1,2 and 3. This software module is an implementation of a part of one or more 
MPEG-2 NBC/MPEG-4 Audio tools as specified by the MPEG-2 NBC/MPEG-4 
Audio standard. ISO/IEC  gives users of the MPEG-2 NBC/MPEG-4 Audio 
standards free license to this software module or modifications thereof for use in 
hardware or software products claiming conformance to the MPEG-2 NBC/MPEG-4
Audio  standards. Those intending to use this software module in hardware or 
software products are advised that this use may infringe existing patents. 
The original developer of this software module and his/her company, the subsequent 
editors and their companies, and ISO/IEC have no liability for use of this software 
module or modifications thereof in an implementation. Copyright is not released for 
non MPEG-2 NBC/MPEG-4 Audio conforming products.The original developer
retains full right to use the code for his/her  own purpose, assign or donate the 
code to a third party and to inhibit third party from using the code for non 
MPEG-2 NBC/MPEG-4 Audio conforming products. This copyright notice must
be included in all copies or derivative works." 
Copyright(c)1996,1997.

2000-10-18 : <kaehleof@iis.fhg.de> started using libaudio (AFsp)
 *                                                                           *
 ****************************************************************************/
/* Oct-2003  modified by RSY,I2R for MPEG-4 audio scalable lossless (SLS) coding */
/* April-2005 modified by Chan Ti Eu (Institute of Infocomm Research, A*STAR) for */
/* multiple SLS elementary stream support										  */

#include "all.h"
#include "port.h" 
#include "lit_ll_dec.h"
#include "inv_int_mdct.h"

#include <fcntl.h>
#include <sys/stat.h>

#ifdef WIN32
#include <io.h>
#else
#include <unistd.h>
#endif

#include <stdio.h>

#ifndef USELIB_AFsp
#include "aiff_support.h"
#else
#include <libtsp.h>
#include <libtsp/AFpar.h>
#endif

#ifdef ENABLE_RTPLAY
#include <au_channel.h>
#endif

#if USELIB_ISOMP4
#include "MP4Movies.h"
#endif

#define	PATH_LEN    128

#ifdef BSAC
int codec_type;
int bsac_sf;
int bsac_ch;
#endif 

#ifdef USELIB_AFsp
static	AFILE*  outfd[Chans];
#else
#ifdef ENABLE_RTPLAY
static  hAudioChannel  outfd[Chans];
#else
static  int     outfd[Chans];
#endif
#endif
static	char	output_path[PATH_LEN];
static	FILE*	inputfile;
static	FILE*	ll_inputfile;
#if USELIB_ISOMP4
char* mp4filename;
static  char    *MP4AacBp; 
#define MAX_LLE_TRACKS 32
static  int    MP4AazBp[MAX_LLE_TRACKS]; 
#endif

#if (defined USELIB_AFsp)||(defined ENABLE_RTPLAY)
static	unsigned int   output_multifiles = 0;
#else
static  unsigned long  output_frames     = 0;
#endif
static	char*          output_format     = "raw";
static	unsigned short output_channels   = 1;
static	unsigned short output_wordsize   = 16;
static	unsigned long  output_sampleRate = 48000;
static	MC_Info*       output_MC_Info    = 0;
static  long           byte_count=0;
static  int            osf               = 1;
static  int			   aac_core_present  = 1; /*default: aac core present */



char ll_bitFileName[255];
char audioFileName[255];
char AAC_inputfile[255];
int numChannel;
float fSample;
long fileNumSample;
int type_PCM;


#ifdef ENABLE_RTPLAY
static  int            output_realtime   = 0;
static  AuChanInfo     outinfo;
#endif 


int              (*getbyte)(void);
int              (*ll_getbyte)(int ll_layer);

void
usage(char *s)
{
  PRINT(SE, "MPEG-4 SLS Decoder\n");
  PRINT(SE, "usage:  %s [options] chan_file pcm_file\n", s);
#ifdef ENABLE_RTPLAY
  PRINT(SE, "	  if pcm_file is '-' the output is played realtime\n");
#endif
  PRINT(SE, "	options are:\n");
  PRINT(SE, "	-i program_number (ADIF header present, 0 is first program)\n");
#ifdef   ADTS
  PRINT(SE, "	-t  (ADTS header present)\n");
#endif
  PRINT(SE, "	-s sampling_frequency, Hz   (48 kHz is default)\n");
  PRINT(SE, "	-a  (create AIFF output files)\n");
    PRINT(SE, "	-f oversampling factor for lossless enhancement (dflt: 1, options: 1,2,4)\n");
	PRINT(SE, "	-n no aac core present\n");
#if (defined USELIB_AFsp)||(defined ENABLE_RTPLAY)
  PRINT(SE, "	-w  (create WAV output files)\n");
  PRINT(SE, "	-3  (create 24bit output files)\n");
  PRINT(SE, "	-m  (create multiple output files)\n");
#endif
#ifdef ENABLE_RTPLAY
  PRINT(SE, "	-ole [1-100] (parameter to control volume)\n");
#endif
  PRINT(SE, "	-D [a-Z]    (enable debugging printouts)\n");
  PRINT(SE, "	-p profile  (Main or LC, Main is default)\n");
  myexit(0);
}

int
initio(int argc, char *argv[], long* fileNumSample)
{
  int i, j, *p;
  char *cmd, *s;
  int getbyte_MP2(void);
  int ll_getbyte_MP2(void);
  int getbyte_MP4(void);
    int ll_getbyte_MP4(void);

  /* set defaults */
  getbyte = &getbyte_MP2;
  ll_getbyte = &ll_getbyte_MP2;
  adif_header_present = 0;


#ifdef   ADTS
  adts_header_present = 0;
#endif
#if USELIB_ISOMP4
  mpeg4_header_present = 0;
#endif

  default_config = 1;
  current_program = -1;
  mc_info.profile = LC_Profile;
  mc_info.sampling_rate_idx = Fs_48;

  /* save cmd 
   * ARGBEGIN is so clever that it throws away argv[0] and 
   * increments the argv pointer
   */
  cmd = argv[0];

  ARGBEGIN {


  default:
    PRINT(SE, "unknown option\n");
    usage(cmd);
#ifdef BSAC
  case 'b':
    codec_type = 1;
    break;
#endif
  case 'a':
    output_format = "aiff";
    break;
#if (defined USELIB_AFsp)||(defined ENABLE_RTPLAY)
  case 'w':
    output_format = "wav";
    break;
  case '3':
    output_wordsize = 24;
    break;
  case 'm':
    output_multifiles = 1;
    break;
#endif
  case 'f':
    s = ARGF();
    osf = atoi(s);
    break;
  case 'n':
    aac_core_present = 0;
#if USELIB_ISOMP4
	myexit("'no aac core' option is not supported");
#endif
    break;
#ifdef ENABLE_RTPLAY
  case 'o':
    if (!strcmp(ARGF(),"le")) {
      s = ARGF();
      outinfo.typeInfo.rt.level = atoi(s);
    }
    break;
#endif
  case 'i':
    adif_header_present = 1;
    s = ARGF();
    current_program = atoi(s);
    if((current_program < 0) || (current_program > 15)) {
      PRINT(SE, "program number out of range: %d\n", current_program);
      myexit("PCE");
    }
    if (debug['v'])
      PRINT(SE, "program with tag %d selected\n", current_program);
    break;
  case 'p':
    s = ARGF();
    p = &mc_info.profile;
    if ((!strncmp(s, "Main", 4)) || (!strncmp(s, "main", 4))) {
      *p = Main_Profile; 
      break;
    }
    if ((!strncmp(s, "LC", 2)) || (!strncmp(s, "lc", 2))) {
      *p = LC_Profile; 
      break;
    }
    if ((!strncmp(s, "LTP", 3)) || (!strncmp(s, "ltp", 3))) {
      *p = LTP_Profile;
      break;
    }
    PRINT(SE, "Unsupported profile %s\n", s);
    myexit("profile");
    break;
#ifdef   ADTS
  case 't':
    adts_header_present=1;
    break;
#endif
  case 's':
    j = atoi(ARGF());
    for (i=0; i<(1<<LEN_SAMP_IDX); i++) {
      if (j == samp_rate_info[i].samp_rate)
	break;
    }
    if (i == (1<<LEN_SAMP_IDX)) {
      PRINT(SE, "Unsupported sampling frequency %d\n", j);
      myexit("samling_rate_idx");
    }
    mc_info.sampling_rate_idx = i;
    output_sampleRate = samp_rate_info[i].samp_rate;
    break;
  case 'D':
    for (s = ARGF(); *s; s++) {
      debug[(int)(*s)] = 1;
      PRINT(SE, "debug option %c enabled\n", *s);
    }
    break;
  case 'l':
    {
      s = ARGF();
      ll_inputfile = fopen(s,"rb");    
	  if (ll_inputfile == 0)
	{
	  PRINT(SE, "cant open %s\n", s);
	  myexit("open");
	}
    }
    break;
  } 
  ARGEND

    output_sampleRate /= osf;
  for (i=0; i<(1<<LEN_SAMP_IDX); i++) {
    if (output_sampleRate == samp_rate_info[i].samp_rate)
      break;
  }
  if (i == (1<<LEN_SAMP_IDX)) {
    PRINT(SE, "Unsupported sampling frequency %d\n", output_sampleRate);
    myexit("samling_rate_idx");
  }
  mc_info.sampling_rate_idx = i;
  output_sampleRate = samp_rate_info[i].samp_rate;

  mc_info.osf = osf;
	mc_info.aac_core_present = aac_core_present;

  if((argc != 1)&&(argc != 2)) {
    usage(cmd);
  }
#if USELIB_ISOMP4
	mp4filename=argv[0];
	ll_inputfile=1;
#endif
      if (aac_core_present) {		
        /* open AAC file */    
        inputfile = fopen(argv[0], "rb"); 
        if(inputfile == 0) {
          PRINT(SE, "cant open %s\n", argv[0]);
          myexit("open");
        }
      }  
      /* in decoding mode: open AAZ file for reading */
#if 1
	
	 if (ll_inputfile == 0) 
	{
	  char ll_path[128];

	  strcpy(ll_path,argv[0]);
	  strcpy(strrchr(ll_path,'.'),"\0");
	  strcat(ll_path,".aaz");
	  ll_inputfile = fopen(ll_path,"rb");
	  if (ll_inputfile == 0)
	    {
	      PRINT(SE, "cant open %s\n", ll_path);
	      myexit("open");
	    }
	}
#endif     
      /* check if bitstream has an ADIF header */
      if (aac_core_present) {
	char adif_id[4];

	fread(adif_id, sizeof(char), 4, inputfile);
	fseek(inputfile, 0, SEEK_SET);
	if(!strncmp(adif_id, "ADIF", 4)) {
	  adif_header_present = 1;
	  if (debug['n'])
	    PRINT(SE, "ADIF header present\n");
	}
#if USELIB_ISOMP4
	else {
		  mpeg4_header_present = 1;
	    getbyte = &getbyte_MP4;
        ll_getbyte = &ll_getbyte_MP4;

#if 0
		if (!MP4OpenMovieFile( &moov, argv[0], MP4OpenMovieNormal)) {
	    /* seems to be a .MP4 */
	  
	    if (debug['n'])
	      PRINT(SE, "MPEG4 header present\n");
	    /* select first (and only) prog in decoderSpecificInfo */
	    current_program = -1;
	  } 
#endif
	}        
#endif
      }


#if USELIB_ISOMP4
	if (!mpeg4_header_present)
	{
#endif
		if (ll_inputfile == 0)
		{
		char ll_path[128];

		strcpy(ll_path,argv[0]);
		strcpy(strrchr(ll_path,'.'),"\0");
		strcat(ll_path,".aaz");
		ll_inputfile = fopen(ll_path,"rb");
		if (ll_inputfile == 0)
		{
			PRINT(SE, "cant open %s\n", ll_path);
			myexit("open");
		}
		}
#if USELIB_ISOMP4
	}
#endif

#ifdef ADTS
      if ((adif_header_present == 0)
#if USELIB_ISOMP4
	  &&(mpeg4_header_present == 0)
#endif
	  ) {
	adts_header_present = 1; /* assume adts header */
      }
#endif

#if USELIB_ISOMP4||ADTS
      if (
#ifdef ADTS
	  ((adif_header_present == 1)&&(adts_header_present == 1))||
#endif
#if   USELIB_ISOMP4
	  ((adif_header_present == 1)&&(mpeg4_header_present == 1))
#ifdef ADTS
	  ||((adts_header_present == 1)&&(mpeg4_header_present == 1))
#endif
#endif
	  )
	{
	  printf("Header Name Specification Error!! Please specify one header name(ADIF or ADTS or MPEG4).\n");
	  exit(0);
	}
#endif
  
      if (argc==1) {  /* only one filename on commandline => show header */
	return 1;
      }
	
      /* two filenames on commandline => prepare output file */
#ifdef ENABLE_RTPLAY
      if (!strcmp(argv[1],"-")) { /* prepare realtime output */
	output_realtime = 1;
	output_multifiles = 0;
	return argc;
      }
#endif

      if (strlen(argv[1]) + strlen("_f00.pcm") + 1 > PATH_LEN) {
	PRINT(SE, "output path too long %s\n", argv[1]);
	myexit("create");
      }
      
	  strcpy(output_path, argv[1]);

return argc;
}


void set_parameters(int sampling_freq, int osf
#ifdef ADIF2MP4
					, int *sampling_freq_index, int *ll_sampling_freq_index
#endif
					,int core_present
					)
{
	int i;
	
	output_sampleRate = sampling_freq;

      output_sampleRate /= osf;
    for (i=0; i<(1<<LEN_SAMP_IDX); i++) {
      if (output_sampleRate == samp_rate_info[i].samp_rate)
	break;
    }
    if (i == (1<<LEN_SAMP_IDX)) {
      PRINT(SE, "Unsupported sampling frequency %d\n", output_sampleRate);
      myexit("sampling_rate_idx");
    }
    mc_info.sampling_rate_idx = i;
#ifdef ADIF2MP4
	*sampling_freq_index = i;
#endif
    output_sampleRate = samp_rate_info[i].samp_rate;

    mc_info.osf = osf;
	mc_info.aac_core_present = core_present;

#ifdef ADIF2MP4
    for (i=0; i<(1<<LEN_SAMP_IDX); i++) {
      if (sampling_freq == samp_rate_info[i].samp_rate)
	break;
    }
    *ll_sampling_freq_index = i;
#endif
}


/* -- functions for output-files -- */
static void
write_fext(Ch_Info *cip, char c, int n)
{
  cip->fext = (char *)mal1(5);
  sprintf(cip->fext, "_%c%02d", c, n);
}

int
open_output_files(MC_Info *mip)
{
  int i, j;
  char file[PATH_LEN];

#ifdef USELIB_AFsp

  int format_id;

  struct {
    char *str;
    int fmt;
  } fmtstr[] = {    /* these types are also "auto-detected" from output-argument */
    /*{"au",FTW_AU*256},
      {"snd",FTW_AU*256},*/
    {"wav",FTW_WAVE*256},
    {"aiff",FTW_AIFF_C*256},
    /*{"aif",FTW_AIFF_C*256},
      {"aifc",FTW_AIFF_C*256},*/
    {"pcm",FTW_NH_EB*256},      /* no header big-endian */
    {"raw",FTW_NH_EB*256},      /* no header big-endian */
    {NULL,-1}
  };
#else
#ifdef ENABLE_RTPLAY
  AuChanType auChnType = TYPE_RT;
  AuChanError err;
  int format_id;
  struct {
    char *str;
    int fmt;
  } fmtstr[] = {
    {"wav",TYPE_RIFF},
    {"aiff",TYPE_AIFF},
    {"pcm",TYPE_PCM},
    {"raw",TYPE_PCM},
    {NULL,-1}
  };
#endif
#endif

  output_MC_Info = mip;
  output_sampleRate = samp_rate_info[mip->sampling_rate_idx].samp_rate; 
  j=0;

#if (defined USELIB_AFsp)||(defined ENABLE_RTPLAY)
  for (i=strlen(output_path);i>=(signed int)(strlen(output_path)-5);i--) {
    if (output_path[i]=='.') {
      j=-1;
      while ( fmtstr[++j].str ) {
	if (!strcmp(fmtstr[j].str,(output_path+i+1))) {
	  output_format=output_path+i+1;
	  *(output_path+i)=0;
	  break;
	}
      }
    }
  }

  i=-1;
  while ( fmtstr[++i].str )
    if (!strcmp(fmtstr[i].str, output_format)) break;

  if ( fmtstr[i].str == NULL ) {
    PRINT(SE, "Internal Error: Unknown output-format\n");
    myexit("format");
  }

  format_id = fmtstr[i].fmt;

#ifdef  USELIB_AFsp
#ifdef BSAC
  if(mip->pcmWordLength == 2 || mip->pcmWordLength == 3) //Suggest update here
	  format_id += FD_INT24;
  else if(mip->pcmWordLength == 1)
	  format_id += FD_INT16; // jie.zhan 2007.4.17 
  //currently 8bit is not supported
  /*else if(mip->pcmWordLength == 0)
	  format_id += FD_INT8;*/
#elif I2R_LOSSLESS
  if(mip->pcmWordLength == 2 || mip->pcmWordLength == 3)
	  format_id += FD_INT24;
  else if(mip->pcmWordLength == 1)
	  format_id += FD_INT16;
  //currently 8bit is not supported
  /*else if(mip->pcmWordLength == 0)
	  format_id += FD_INT8;*/
#endif
#endif
#endif 

  /* if FCenter == 1 (center speaker present)
   *	position    index   extension
   *	left	    1	    01
   *	center	    0	    00
   *	right	    2	    02
   * if FCenter == 0 (center speaker absent)
   *	position    index   extension
   *	left	    0	    01
   *	right	    1	    02
   */
  for (i=((FCenter==1) ? 0 : 1); i<FChans; i++) {
    write_fext(&mip->ch_info[j], 'f', i);
    j++;
  }
  for (i=0; i<SChans; i++) {
    write_fext(&mip->ch_info[j], 's', i);
    j++;
  }
  /* if BFCenter == 1 (center speaker present)
   *	position    index   extension
   *	left	    3	    00
   *	center	    5	    02
   *	right	    4	    01
   * if BCenter == 0 (center speaker absent)
   *	position    index   extension
   *	left	    3	    00
   *	right	    4	    01
   */
  for (i=0; i<BChans; i++) {
    write_fext(&mip->ch_info[j], 'b', i);
    j++;
  }
  for (i=0; i<LChans; i++) {
    write_fext(&mip->ch_info[j], 'l', i);
    j++;
  }

#if (defined USELIB_AFsp)||(defined ENABLE_RTPLAY)
  if (!output_multifiles) {
    j=0;
    for (i=0; i<Chans; i++) 
      if (mip->ch_info[i].present) j++;
    if (j>2) {
      output_multifiles = 1;
      PRINT(SE, "warning: no standards for %i channels in one file... using multiple\n", j);
    } else {
      output_channels = j;
    }
  }
#endif
#ifdef ENABLE_RTPLAY
  if (output_realtime) {
    auChnType = TYPE_RT;
    if ((output_channels>2)||(output_multifiles)) {
      PRINT(SE, "error: can't play so many channels realtime\n");
      myexit(0);
    }
    outinfo.valid = 1;
    outinfo.bitsPerSample = 16;
    outinfo.sampleRate = output_sampleRate;
    outinfo.nChannels = output_channels;
    outinfo.nSamples = LN2;
    outinfo.fpScaleFactor = 32768.0;
    if (!outinfo.typeInfo.rt.level) outinfo.typeInfo.rt.level = 50; /* output Volume */
    outinfo.typeInfo.rt.outputDrain = OUT_SPEAKER;
    err = AuChannelOpen(&outfd[0], NULL/*outname*/, (AuChanMode)AU_CHAN_WRITE, &auChnType, &outinfo);
    if (err != AU_CHAN_OK ) {
      PRINT(SE, "AuChannelOpen reported an error\n" );
      myexit(0);
    }
  } else {
    auChnType = format_id;
    outinfo.valid = 1;
    outinfo.bitsPerSample = output_wordsize;
    outinfo.sampleRate = output_sampleRate;
    outinfo.nChannels = output_channels;
    outinfo.nSamples = LN2;
    outinfo.fpScaleFactor = 32768.0;
#endif

#if (defined USELIB_AFsp)||(defined ENABLE_RTPLAY)
    if (output_multifiles) { /* output several files */
#endif
      j=0;
      for (i=0; i<Chans; i++) {
	if (!(mip->ch_info[i].present))
	  continue;

	j++;

	strcpy(file, output_path);

	// I2R: output filenames appended with channel numbers (28 May 07)
	strcat(file, itoa(i,mip->ch_info[i].fext,10));//mip->ch_info[i].fext);

	strcat(file, ".");
	strcat(file, output_format);

	if (debug['o'])
	  PRINT(SE, "opening file %s at position %d\n", file, i);

#ifdef ENABLE_RTPLAY
	err = AuChannelOpen(&outfd[i], file, (AuChanMode)AU_CHAN_WRITE, &auChnType, &outinfo);
	if (err != AU_CHAN_OK ) {
	  PRINT(SE, "AuChannelOpen reported an error\n" );
	  myexit("create");
	}
#else
#ifdef USELIB_AFsp
	outfd[i] = AFopenWrite(file, format_id, output_channels, output_sampleRate*mip->osf, (FILE*)NULL );
	if(outfd[i] == NULL) {
	  PRINT(SE, "cant create %s\n", file);
	  myexit("create");
	}
#else  /* no AFsp-lib support */
#ifdef WIN32
	outfd[i] = _open (file, _O_CREAT | _O_TRUNC | _O_BINARY | _O_WRONLY, _S_IWRITE);
#else
	outfd[i] = creat(file, 0666);
#endif
	if(outfd[i] < 0) {
	  PRINT(SE, "cant create %s\n", file);
	  myexit("create");
	}
	if ( !strcmp(output_format,"aiff") ) {
	  /* write dummy header and position for audio data */
	  if ( WriteAIFFHeader(outfd[i], output_channels, output_frames, output_wordsize, output_sampleRate) != 0 ) {
	    PRINT( SE, "cannot write AIFF header for %s\n", file );
	    myexit( "aiff-header" );
	  }
	}
#endif
#endif
      }
#if (defined USELIB_AFsp)||(defined ENABLE_RTPLAY)
    } else {   /* place multiple channels in one file */
      strcpy(file, output_path);
      strcat(file, ".");
      strcat(file, output_format);

      if (debug['o'])
	PRINT(SE, "opening file %s with %d channels\n", file, output_channels);
#ifdef USELIB_AFsp
      outfd[0] = AFopenWrite(file, format_id, output_channels, output_sampleRate*mip->osf, (FILE*)NULL );
        
      if(outfd[0] == NULL) {
	PRINT(SE, "cant create %s\n", file);
	myexit("create");
      }
#else
      err = AuChannelOpen(&outfd[0], file, (AuChanMode)AU_CHAN_WRITE, &auChnType, &outinfo);
      if (err != AU_CHAN_OK ) {
	PRINT(SE, "AuChannelOpen reported an error\n" );
	myexit(0);
      }
#endif
    }
#endif
#ifdef ENABLE_RTPLAY
  }
#endif
  return(1);
}

#if !((defined USELIB_AFsp)||(defined ENABLE_RTPLAY))
void
fmtchan(char *p, float *data, int stride)
{
  int i, c;
  float s;

  for(i=0; i<LN2; i++) {
    s = *data++;
    if(s < 0) {
      s -= .5;
      if(s < -0x7fff)
        s = (float) -0x7fff;
    } else {
      s += .5;
      if(s > 0x7fff)
        s = (float) 0x7fff;
    }
    c = (int) s;

    p[0] = (c >> 8) & 0xff;
    p[1] = c & 0xff;

    p += stride;
  }
}
#endif


void
writeout(Float **data, MC_Info *mip,int len)
{
  int i;
  static int opened = 0;

#ifndef USELIB_AFsp
  int stride, size;
  char obuf[LN2*2];
#endif
#if (defined USELIB_AFsp)||(defined ENABLE_RTPLAY)
  int j;
  Float fl_obuf[MAX_OSF*LN2*Chans];
#endif
   
  if (!opened) {
    open_output_files(mip);
    opened=1;
  }

#ifndef USELIB_AFsp
  stride=2;
  size = sizeof(obuf);
#endif
#if (defined ENABLE_RTPLAY)||(defined USELIB_AFsp)
  if (!output_multifiles) { /* multiple channels into one file */
    int k=0;
    for (i=0; i<Chans; i++) {
      if (!(mip->ch_info[i].present))
	continue;
      for (j=0; j<len*output_channels; j++)
	fl_obuf[j*output_channels+k]=data[i][j];
      k++;
    }
#ifdef ENABLE_RTPLAY
    AuChannelWriteFloat( outfd[0], fl_obuf, len*output_channels, &i /*written bytes*/);
#endif
#ifdef USELIB_AFsp
    AFwriteData(outfd[0], fl_obuf, len*output_channels);
#endif
  } else {
#endif
    for (i=0; i<Chans; i++) {
      if (!(mip->ch_info[i].present))
	continue;
#ifdef USELIB_AFsp
      AFwriteData(outfd[i], data[i], len);
#else
#ifdef ENABLE_RTPLAY
      AuChannelWriteFloat( outfd[i], data[i], len, &j /*written bytes*/);
#else
      fmtchan(obuf, data[i], stride);
      if(write(outfd[i], obuf, size) != size) {
	PRINT(SE, "write error\n");
	myexit("write");
      }
#endif
#endif
    }
#if ((defined USELIB_AFsp)||(defined ENABLE_RTPLAY))
  }
#else
  if ( !strcmp(output_format,"aiff") )
    output_frames += len;
#endif
}


/* -- functions for reading the inputfile -- */
void
restartio(
#if USELIB_ISOMP4
	  char *p
#endif
	  )
{
  cword = 0;
  nbits = 0;
  inerror = 0;

#if USELIB_ISOMP4
    if ( mpeg4_header_present == 1 )
      MP4AacBp = p;
#endif
}

void
ll_restartio(char *ll_p ,int i)
{

#ifndef ADIF2MP4 
    ll_cword[i] = 0;
    ll_nbits[i] = 0;
    ll_inerror[i] = 0;
#else
  ll_cword = 0;
  ll_nbits = 0;
  ll_inerror = 0;
#endif

#if USELIB_ISOMP4
  if ( mpeg4_header_present == 1 )
#if USELIB_ISOMP4
	  MP4AazBp[i] = ll_p;
#endif
#endif
}


int 
getbyte_MP2(void)
{
  int c;
  static unsigned short errorCounter = 0;
  c = getc(inputfile);
  if(c < 0) {
    errorCounter++;
    if ( errorCounter > 3 ) {
      myexit((debug['n']) ?
             (char *)"getbyte: End of Bitstream" : (char *)0);
    }
    c = 0;
  }

  return c;
}

int 
ll_getbyte_MP2(void)
{
  int c;
  static unsigned short errorCounter = 0;
  c = getc(ll_inputfile);
  if(c < 0) {
    errorCounter++;
    if ( errorCounter > 3 ) {
      myexit((debug['n']) ?
             (char *)"getbyte: End of Bitstream" : (char *)0);
    }
    c = 0;
  }

  return c;
}


#if USELIB_ISOMP4
int getbyte_MP4(void)
{
	unsigned char c;
  c = *MP4AacBp++;
  return c;
}

int ll_getbyte_MP4(int ll_layer)
{
	unsigned char c;

	c = *((char*)MP4AazBp[ll_layer]);
		MP4AazBp[ll_layer]++;

    return c;
}
#endif /* USELIB_ISOMP4 */

#ifndef ADIF2MP4
long
getshort(void)
{
  int c1, c2;

  c1 = (*getbyte)();
  c2 = (*getbyte)();
  byte_count += 2;
  if(c2 < 0)
    c2 = 0;
  if(c1 < 0)
    c1 = 0;

  if (debug['G'])
    PRINT(SE,"get: %02x %x %ld\n", c1, c2, byte_count);
  return (c1<<8) | c2;
}

long
ll_getshort(int ll_layer)
{
  int c1, c2;

    c1 = (*ll_getbyte)(ll_layer);
    c2 = (*ll_getbyte)(ll_layer);
  byte_count += 2;
  if(c2 < 0)
    c2 = 0;
  if(c1 < 0)
    c1 = 0;

  if (debug['G'])
    PRINT(SE,"get: %02x %x %ld\n", c1, c2, byte_count);
  return (c1<<8) | c2;
}


long
getbits(int n)
{
  if(nbits <= 16){
    cword = (cword<<16) | getshort();
    nbits += 16;
  }

  nbits -= n;
  return (cword>>nbits) & ((1<<n)-1);
}

long
ll_getbits(int n, int ll_layer)
{
  if(ll_nbits[ll_layer] <= 16){
    ll_cword[ll_layer] = (ll_cword[ll_layer]<<16) | ll_getshort(ll_layer);
    ll_nbits[ll_layer] += 16;
  }

  ll_nbits[ll_layer] -= n;
  return (ll_cword[ll_layer]>>ll_nbits[ll_layer]) & ((1<<n)-1);
}
#endif

long getlongbits(int n)
{
  long v1, v2;
  if (n > 16) {
    v1 = getbits(8);
    v2 = getbits(n-8);
    v2 = (v1<<(n-8)) | v2;
  }
  else {
    v2 = getbits(n);
  }
  return v2;
}

/* -- nice exit functions -- */
#ifndef ADIF2MP4
void
myexit(char *s)
{
  countInstructionsPrint();
  closefiles();
  if (debug['v'])
    PRINT(SE,"%lx %d\n", cword, nbits);
  if (s)
    PRINT(SE,"\n%s\n", s);
  exit(s ? 1 : 0);
}
#endif

void
closefiles()
{
  int i;
#if !((defined USELIB_AFsp)||(defined ENABLE_RTPLAY))
  if ( !strcmp(output_format,"aiff") ) 
    {
#endif
      for ( i = 0; i < Chans; i++ )
	{
	  if ( !outfd[i] )
	    continue;
#ifdef USELIB_AFsp
	  AFclose(outfd[i]);
#else
#ifdef ENABLE_RTPLAY
	  AuChannelClose( outfd[i] );
#else
	  WriteAIFFHeader( outfd[i], output_channels, output_frames, output_wordsize, output_sampleRate );
	  close( outfd[i] );
#endif
#endif
	}
#if !((defined USELIB_AFsp)||(defined ENABLE_RTPLAY))
    }
#endif
}


#ifdef BSAC
int getBSACFrameBst(unsigned char *bst, int *len)
{
// 	if (fread(bst, sizeof(char), 2, inputfile) != 2)
// 		return 0;
// 	*len = (bst[0]<<3) | ((bst[1]>>5) & 0x07);
// 
//	fread(bst+2, sizeof(char), *len-2, inputfile);
	memcpy(bst,MP4AacBp, sizeof(char)*2);
	*len = (bst[0]<<3) | ((bst[1]>>5) & 0x07);
	memcpy(bst+2, MP4AacBp+2, sizeof(char)*(*len-2));
	MP4AacBp+=*len;
	return 1;
}
#endif






