#ifdef  VERSION2
/**********************************************************************
MPEG-4 Audio VM

This software module was originally developed by
  Y.B. Thomas Kim and S.H. Park (Samsung AIT)
and edited by
  Y.B. Thomas Kim (Samsung AIT) on 1997-11-06

in the course of development of the MPEG-2 AAC/MPEG-4 Audio standard
ISO/IEC 13818-7, 14496-1,2 and 3. This software module is an
implementation of a part of one or more MPEG-2 AAC/MPEG-4 Audio tools
as specified by the MPEG-2 AAC/MPEG-4 Audio standard. ISO/IEC gives
users of the MPEG-2 AAC/MPEG-4 Audio standards free license to this
software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-2 AAC/ MPEG-4 Audio
standards. Those intending to use this software module in hardware or
software products are advised that this use may infringe existing
patents. The original developer of this software module and his/her
company, the subsequent editors and their companies, and ISO/IEC have
no liability for use of this software module or modifications thereof
in an implementation. Copyright is not released for non MPEG-2
AAC/MPEG-4 Audio conforming products. The original developer retains
full right to use the code for his/her own purpose, assign or donate
the code to a third party and to inhibit third party from using the
code for non MPEG-2 AAC/MPEG-4 Audio conforming products. This
copyright notice must be included in all copies or derivative works.

Copyright (c) 1997.

	SAMSUNG_2005-09-30 : EncTf_bsac_encode()
										   sam_tf_quantize_spectrum()
											

**********************************************************************/
#include <stdio.h>
#include <math.h>

#ifdef VERSION2
#include "sam_encode.h"
#endif




#define  ID_SCE  0x00
#define  ID_CPE  0x01
#define  MAX_PRED_SFB  40

static int num_of_channels;
static int bitrate;
static int sampling_frequency_index;
static int sampling_frequency;
static int frame_length_flag;
static int block_size_samples;
static int  fs_index[16]={
  96000, 88200, 64000, 48000, 44100, 32000, 24000, 22050,
  16000, 12000, 11025,  8000,     0,     0,     0,     0
};

#ifdef __cplusplus
extern "C" {
#endif

static int my_log2(int value);

#ifdef __cplusplus
}
#endif



static   HANDLE_BSBITBUFFER tmpBitBuf;
static   HANDLE_BSBITBUFFER dynpartBuf;

static   long  samplingRate;
static double pow_quant[1024]; /* SAMSUNG_2005-09-30 */



void sam_init_encode_bsac(
    int numChannel,
    int sampling_rate,
    int bit_rate,
    int frame_size
)
{
  int  i;
  num_of_channels = numChannel;
  block_size_samples = frame_size;
  if(frame_size == 1024)
    frame_length_flag = 0;
  else
    frame_length_flag = 1;
  for(i = 0; i < 12; i++) if(sampling_rate == fs_index[i]) {
    sampling_frequency_index = i;
    break;
  }
  sampling_frequency = sampling_rate;
  bitrate = bit_rate/num_of_channels;
  if(bitrate > 64000) {
    printf("BSAC: Maximum bitrate 64kbps/ch!\n");
    bitrate = 64000;
  }
  sam_scale_bits_init_enc(sampling_rate, block_size_samples);
/* SAMSUNG_2005-09-30 */
	for (i=0;i<1024;i++){ 
    pow_quant[i]=pow((double)i, ((double)4.0/(double)3.0)); 
  }
}

int startf=1;
int sam_encode_frame(
  int num_of_chan,
	int ch_type, /* SAMSUNG_2005-09-30 */
  WINDOW_SEQUENCE windowSequence,
  WINDOW_SHAPE  window_shape,
  int sfb_offset[],
  int nr_of_sfs,
  int num_window_groups,
  int window_group_length[],
  int quant[][1024],
  int scfacs[][MAX_SCFAC_BANDS],
	MSInfo      *msInfo, /* SAMSUNG_2005-09-30 */
  HANDLE_BSBITSTREAM fixed_stream,
  int i_ch,
  /*int w_flag,*/
  int avr_bits,
	int mc_ext_size, /* SAMSUNG_2005-09-30 : extension parts length  */
	int mc_enabled, /* SAMSUNG_2005-09-30 */
	int ext_flag
	,TNS_INFO* tns_info[]
	,MS_Info* ms_info[]
	) /* SAMSUNG_2005-09-30 */
{
  int i, k, ch, w, g, s;
  int b = 0;
  int sfb, maxSfb;
  int frameSize;
  int abits;
  int nch;
  int cband;
  int nr_of_sfb = 0;
  int end_cband[8];
  int stereo_mode;
  /*int stereo_info[MAX_SCFAC_BANDS];*/
  int stereo_info[8][MAX_SCFAC_BANDS]; /* SAMSUNG_2005-09-30 */
  int ModelIndex[2][8][32];
  int scalefactors[2][MAX_SCFAC_BANDS];
  int scf[2][8][MAX_SCFAC_BANDS];
  int swb_offset[8][MAX_SCFAC_BANDS];
  int sample_buf[2][1024];
  int *sample[2][8];
	int w_flag = (fixed_stream)? 1 : 0; /* SAMSUNG_2005-09-30 */

  /*nch = i_ch + 1;*/
  nch = i_ch + num_of_chan; /* SAMSUNG_2005-09-30 */

  /* initialize */
  stereo_mode = 2;
  //SLS_BSAC
  //stereo_mode = msInfo->ms_mask;  /* SAMSUNG_2005-09-30 */
  

/*  for(sfb = 0; sfb < MAX_SCFAC_BANDS; sfb++) {
    scalefactors[i_ch][sfb] = scfacs[i_ch][sfb];
    stereo_info[sfb] = 0;
  }
  if(w_flag) {
    nch = 1;
    if(num_of_chan == 2) {
      for(i = 0; i < MAX_SCFAC_BANDS; i++) 
        scalefactors[1][i] = scfacs[1][i];
      nch = 2;
    }
  }*/
/* SAMSUNG_2005-09-30 */
	for(sfb = 0; sfb < MAX_SCFAC_BANDS; sfb++) {
		for(ch = i_ch; ch < nch; ch++) {		
			scalefactors[ch][sfb] = scfacs[ch][sfb];
		}		
	}
	nr_of_sfb = nr_of_sfs / num_window_groups;

	//SLS_BSAC
	/*for(g = 0; g < num_window_groups; g++)
		for(sfb = 0; sfb < nr_of_sfb; sfb++)
			stereo_info[g][sfb] = msInfo->ms_used[g][sfb];*/
	for(g = 0; g < num_window_groups; g++)
		for(sfb = 0; sfb < nr_of_sfb; sfb++)
			stereo_info[g][sfb] = 1;


	if(ch_type == LFE)
		maxSfb = 4;
	else
/* ~SAMSUNG_2005-09-30 */
		maxSfb = nr_of_sfs / num_window_groups;
	//SLS_BSAC
	// - 3;

  for(ch = i_ch; ch < nch; ch++) {

    if(windowSequence == EIGHT_SHORT_SEQUENCE) {
      int short_block_size=block_size_samples/8;

      s = 0;
      /*nr_of_sfb = nr_of_sfs / num_window_groups;*/
      for(g = 0; g < num_window_groups; g++) { 
        sample[ch][g] = &(sample_buf[ch][short_block_size * s]);
        s += window_group_length[g];

        swb_offset[g][0] = 0;
        for(sfb = 0; sfb < nr_of_sfb; sfb++)
          swb_offset[g][sfb+1] = sfb_offset[sfb+1] * window_group_length[g];
        end_cband[g] = (swb_offset[g][maxSfb]+31)/32;
      }

      /* reshuffling */
      s = 0;
      for(g = 0; g < num_window_groups; g++) {
        for (i = 0; i < short_block_size; i+=4) {
          int offset = (128 * s) + (i * window_group_length[g]);          
          for(w=0; w<window_group_length[g]; w++, b++)  
					for (k = 0; k < 4; k++) 
            sample_buf[ch][offset+4*w+k] = quant[ch][short_block_size*(w+s)+i+k];
        }
        s += window_group_length[g];
      }

    } else {

      nr_of_sfb = nr_of_sfs;
      num_window_groups = 1;
      window_group_length[0] = 1;
      swb_offset[0][0] = 0;
      for(sfb = 0; sfb < nr_of_sfb; sfb++)
        swb_offset[0][sfb+1] = sfb_offset[sfb+1];

      sample[ch][0] = &(quant[ch][0]);

      end_cband[0] = (swb_offset[0][maxSfb]+31)/32;
    }

    for (g = 0; g < num_window_groups; g++) 
      for (cband = 0; cband < 32; cband++) 
        ModelIndex[ch][g][cband] = 0;

    for(g = 0; g < num_window_groups; g++) {
      int bal, max;

      for (cband = 0; cband < end_cband[g]; cband++) {
        max = 0;
        for (i = cband*32; i < (cband+1)*32; i++) {
          if (abs(sample[ch][g][i])>max) 
            max = abs(sample[ch][g][i]);
        }
        if(max > 0) bal = my_log2(max);
        else    bal = 0;
        if(bal >= 8)
          ModelIndex[ch][g][cband] = bal + 7;
        else
          ModelIndex[ch][g][cband] = bal * 2  - 1;
        if (ModelIndex[ch][g][cband] < 0) 
          ModelIndex[ch][g][cband] = 0; 
      }
    }

    for (g = 0; g < num_window_groups; g++) {
      for (sfb = 0; sfb < maxSfb; sfb++) {
        scf[ch][g][sfb] = scalefactors[ch][(g*nr_of_sfb)+sfb];
      }
    }
  }

  if(avr_bits == 0)
    abits = 10000;
  else
    abits = avr_bits - 8;

  /********** B I T S T R E A M   M A K I N G *********/
  if(w_flag) {
    /*frameSize = sam_encode_bsac(bitrate, windowSequence, window_shape, sample,
      scf, maxSfb, num_window_groups, window_group_length, swb_offset, nr_of_sfb,
      ModelIndex, stereo_mode, stereo_info, abits, i_ch, nch);*/		
		/* SAMSUNG_2005-09-30 */
    frameSize = sam_encode_bsac(bitrate, ch_type, windowSequence, window_shape, sample,
      scf, maxSfb, num_window_groups, window_group_length, swb_offset, nr_of_sfb,
      ModelIndex, stereo_mode, stereo_info, abits+mc_ext_size, i_ch, nch, mc_enabled, ext_flag
	,tns_info
	,ms_info
	,w_flag
	 );

   abits = frameSize;
	 
  }

  /*frameSize = sam_encode_bsac(bitrate, windowSequence, window_shape, sample,
      scf, maxSfb, num_window_groups, window_group_length, swb_offset, nr_of_sfb,
      ModelIndex, stereo_mode, stereo_info, abits, i_ch, nch);*/
	/* SAMSUNG_2005-09-30 */
  frameSize = sam_encode_bsac(bitrate, ch_type, windowSequence, window_shape, sample,
      scf, maxSfb, num_window_groups, window_group_length, swb_offset, nr_of_sfb,      
	  ModelIndex, stereo_mode, stereo_info, abits+mc_ext_size, i_ch, nch, mc_enabled, ext_flag
	,tns_info
	,ms_info
	,0
	);

/*
if(startf) {
printf("INIT\n");
for(i = 0; i < 16; i++)
printf("%2d: %d\n", i, sample[0][0][i]);
}
printf("SIZE:%d, %d\n", frameSize, abits);
startf = 0;
*/
  /* 20060705 */
  if(w_flag) {
	if(mc_enabled)
	{
		if(!ext_flag) /* SAMSUNG_2005-09-30 */
		{
			sam_frame_length_rewrite2bs(frameSize+mc_ext_size, 0, 11);
		}
		else
		{
			sam_frame_length_rewrite2bs(frameSize, 0, 11);
		}
	}
	else
	{
		if((frameSize % 8)) {
		  frameSize += sam_putbits2bs(0x00, (8 - (frameSize % 8)));
		}
		sam_frame_length_rewrite2bs(frameSize, 0, 11);
	}
	if(!mc_enabled) /* SAMSUNG_2005-09-30 */
		sam_bsflush(fixed_stream, frameSize);
	else
	{
		if(!ext_flag)
			sam_bsflushAll(fixed_stream);/* SAMSUNG_2005-09-30 : flush all of channel bistream */
	}

  } else {
    if(frameSize % 8) 
       frameSize += (8 - (frameSize % 8));
  }

  return frameSize;
}

static int my_log2(int value)
{
  int  i, step;

  if(value < 0) {
    fprintf(stderr, "BSC:my_log2:error : %d\n", value);
    return 0;
  }
  if(value == 0)  return 0;

  step = 2;
  for(i = 1; i < 24; i++) {
    if(value < step) return i;
    step *= 2;
  }
  return ( 1 + (int)(log10((double)value)/log10(2.0)));
}

#endif  /* VERSION 2 */
