/************************* MPEG-2 AAC Audio Decoder **************************
 *                                                                           *
 "This software module was originally developed by 
 Fraunhofer Gesellschaft IIS / University of Erlangen (UER)
 and edited by
 in the course of 
 development of the MPEG-2 AAC/MPEG-4 Audio standard ISO/IEC 13818-7, 
 14496-1,2 and 3. This software module is an implementation of a part of one or more 
 MPEG-2 AAC/MPEG-4 Audio tools as specified by the MPEG-2 AAC/MPEG-4 
 Audio standard. ISO/IEC  gives users of the MPEG-2 AAC/MPEG-4 Audio 
 standards free license to this software module or modifications thereof for use in 
 hardware or software products claiming conformance to the MPEG-2 AAC/MPEG-4
 Audio  standards. Those intending to use this software module in hardware or 
 software products are advised that this use may infringe existing patents. 
 The original developer of this software module and his/her company, the subsequent 
 editors and their companies, and ISO/IEC have no liability for use of this software 
 module or modifications thereof in an implementation. Copyright is not released for 
 non MPEG-2 AAC/MPEG-4 Audio conforming products.The original developer
 retains full right to use the code for his/her  own purpose, assign or donate the 
 code to a third party and to inhibit third party from using the code for non 
 MPEG-2 AAC/MPEG-4 Audio conforming products. This copyright notice must
 be included in all copies or derivative works." 
 Copyright(c)1999.
 *                                                                           *
 ****************************************************************************/

#ifndef _TF_MAIN_HANDLE_H_INCLUDED
#define _TF_MAIN_HANDLE_H_INCLUDED

#include <stdlib.h> /* might contain min and max definitions. RS 2001-10-18 */ 
#ifndef min
#define min(a,b) ((a) < (b) ? (a) : (b)) 
#endif

#ifndef max
#define max(a,b) ((a) > (b) ? (a) : (b))
#endif




/* name the audio channels */

enum CHANN_ASS { 
  MONO_CHAN    = 0,
  LEFT_CHAN    = 0, 
  RIGHT_CHAN   = 1,
  MAX_CHANNELS = 48
};



typedef enum  { 
  ONLY_LONG_SEQUENCE, 
  LONG_START_SEQUENCE, 
  EIGHT_SHORT_SEQUENCE,
  LONG_STOP_SEQUENCE,
  NUM_WIN_SEQ        
} WINDOW_SEQUENCE ;    


enum _MAX { 
  MAX_GRANULES            =  3,
  NSFB_LONG               = 53,
  NSFB_SHORT              = 16,
  MAX_SHORT_IN_LONG_BLOCK =  8,
  MAX_SHORT_WINDOWS       =  8
};

/* if static memory allocation is used, this value tells the max. nr of 
   audio channels to be supported */

#define MAX_TIME_CHANNELS (MAX_CHANNELS)

/* max. number of scale factor bands */

#define MAX_SCFAC_BANDS ((NSFB_SHORT+1)*MAX_SHORT_IN_LONG_BLOCK)

#define SFB_NUM_MAX MAX_SCFAC_BANDS

#define	MAX_CHANNEL_ELEMENT	10  /* SAMSUNG_2005-09-30 : maximum channel elements for BSAC Multichannel */



#endif  /* #ifndef _TF_MAIN_HANDLE_H_INCLUDED */

#define DEBUG 1
#define CHANNEL  1
#define NUM_COEFF  1024
#define MAGIC_NUMBER  0.4054
#define MAX_QUANT 8192
#define SF_OFFSET 100
#define abs(A) ((A) < 0 ? (-A) : (A))
#define sgn(A) ((A) > 0 ? (1) : (-1))


typedef enum {
	CENTER_FRONT,		/* 1 */
	LR_FRONT,				/* 2 */
	REAR_SUR,				/* 1 */
	LR_SUR,					/* 2 */
	LFE,						/* 1 */
	LR_OUTSIDE,			/* 2 */	
	RESERVED 	
}CH_CFG ;


typedef enum {
  WS_FHG,
  WS_DOLBY,
#ifdef VERSION2
  WS_ZPW,
#endif 
  N_WINDOW_SHAPES
} WINDOW_SHAPE;


typedef struct {
  int scale_factor[SFB_NUM_MAX];
  int book_vector[SFB_NUM_MAX];
  int quant[NUM_COEFF];
} QuantInfo;

typedef struct {
  /* energies of each scalefactor band */
  int  pns_sfb_nrg[SFB_NUM_MAX];
  /* PNS on/off flag for each scalefactor band */
  int  pns_sfb_flag[SFB_NUM_MAX];
} PnsInfo;


typedef struct {
  int ms_mask;
  int ms_used[MAX_SHORT_WINDOWS][SFB_NUM_MAX];
} MSInfo;
  
typedef struct tagBsBitBufferStruct *HANDLE_BSBITBUFFER;
typedef struct tagBsBitStreamStruct *HANDLE_BSBITSTREAM;

#define MAXBANDS 128
#define CChans 8

typedef struct
{
    int is_present;	/* right channel uses intensiy stereo */
    int is_used[MAXBANDS];
    int sign[2*(MAXBANDS+1)];
    int fac[2*(MAXBANDS+1)];
    int bot[2*(MAXBANDS+1)];
    int top[2*(MAXBANDS+1)];
} IS_Info;

typedef struct
{
    int is_present;  
    int ms_used[MAXBANDS];
} MS_Info;

typedef struct
{
    int present;	/* channel present */
    int tag;		/* element tag */
    int cpe;		/* 0 if single channel or lfe, 1 if channel pair */
	int lfe;		/* 1 if lfe channel */
    int	common_window;	/* 1 if common window for cpe */
    int	ch_is_left;	/* 1 if left channel of cpe */
    int	paired_ch;	/* index of paired channel in cpe */
    int widx;		/* window element index for this channel */
    IS_Info is_info;	/* IS information */
    MS_Info ms_info;    /* MS information */
    int ncch;		/* number of coupling channels for this ch */
    int cch[CChans];	/* coupling channel idx */
    int cc_dom[CChans];	/* coupling channel domain */
    int cc_ind[CChans];	/* independently switched coupling channel flag */
    char *fext;		/* filename extension */
} Ch_Info;

/**************************/
/* Structure definitions  */
/**************************/
#define TNS_MAX_ORDER  20
#define LEN_TNS_NFILTL 2
#define NSHORT 8
typedef struct {
  int order;                           /* Filter order */
  int direction;		       /* Filtering direction */
  int coefCompress;		       /* Are coeffs compressed? */
  int length;                          /* Length, in bands */                     
  double aCoeffs[TNS_MAX_ORDER+1];     /* AR Coefficients */
  double kCoeffs[TNS_MAX_ORDER+1];     /* Reflection Coefficients */
  int index[TNS_MAX_ORDER+1];	       /* Coefficient indices */
} TNS_FILTER_DATA;


typedef struct {
  int numFilters;				/* Number of filters */
  int coefResolution;				/* Coefficient resolution */
  TNS_FILTER_DATA tnsFilter[1<<LEN_TNS_NFILTL];	/* TNS filters */
} TNS_WINDOW_DATA;


typedef struct {
  int tnsDataPresent;
  int tnsMinBandNumberLong;
  int tnsMinBandNumberShort;
  int tnsMaxBandsLong;
  int tnsMaxBandsShort;
  int tnsMaxOrderLong;
  int tnsMaxOrderShort;
  TNS_WINDOW_DATA windowData[NSHORT];	/* TNS data per window */
} TNS_INFO;