/**********************************************************************
MPEG-4 Audio VM
Encoder frame work



This software module was originally developed by

Heiko Purnhagen (University of Hannover)

and edited by

in the course of development of the MPEG-2 NBC/MPEG-4 Audio standard
ISO/IEC 13818-7, 14496-1,2 and 3. This software module is an
implementation of a part of one or more MPEG-2 NBC/MPEG-4 Audio tools
as specified by the MPEG-2 NBC/MPEG-4 Audio standard. ISO/IEC gives
users of the MPEG-2 NBC/MPEG-4 Audio standards free license to this
software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-2 NBC/ MPEG-4 Audio
standards. Those intending to use this software module in hardware or
software products are advised that this use may infringe existing
patents. The original developer of this software module and his/her
company, the subsequent editors and their companies, and ISO/IEC have
no liability for use of this software module or modifications thereof
in an implementation. Copyright is not released for non MPEG-2
NBC/MPEG-4 Audio conforming products. The original developer retains
full right to use the code for his/her own purpose, assign or donate
the code to a third party and to inhibit third party from using the
code for non MPEG-2 NBC/MPEG-4 Audio conforming products. This
copyright notice must be included in all copies or derivative works.

Copyright (c) 1996.



Source file: encoder.c (formerly mp4enc.c)


Required libraries:
libtsp.a		AFsp audio file library

Required modules:
common.o		common module
cmdline.o		command line module
bitstream.o		bits stream module
enc_tf.o		encoder core (tf-based)

Authors:
HP    Heiko Purnhagen, Uni Hannover <purnhage@tnt.uni-hannover.de>
BG    Bernhard Grill, Uni Erlangen <grl@titan.lte.e-technik.uni-erlangen.de>
NI    Naoki Iwakami, NTT <iwakami@splab.hil.ntt.jp>
CL    Chuck Lueck, TI <lueck@ti.com>
RG    Ralf Geiger, FhG/IIS
MCF	  Matt Fellers, Dolby Laboratories, Inc.

Changes:
13-jun-96   HP    first version
14-jun-96   HP    added debug code
17-jun-96   HP    added bit reservoir control / switches -r -b
18-jun-96   HP    added delay compensation
19-jun-96   HP    using new ComposeFileName()
25-jun-96   HP    changed frameNumBit
26-jun-96   HP    improved handling of switch -o
04-jul-96   HP    joined with t/f code by BG (check "DISABLE_TF")
14-aug-96   HP    adapted to new cmdline module, added mp4.h
16-aug-96   HP    adapted to new enc.h
                  added multichannel signal handling
21-aug-96   HP    added NBC_RM4 switches by BG
26-aug-96   HP    CVS
28-aug-96   NI    moved "static CmdLineSwitch switchList[]" into main routine.
29-aug-96   HP    moved "switchlist[]" back again
                  NI stated, that it just was a locally required bug-fix:
                    This table is moved into the main routine,
                    because there was a run-time problem on a
                    SGI (IRIX-5.3) system (using g++).
30-oct-96   HP    additional frame work options
15-nov-96   HP    adapted to new bitstream module
18-nov-96   HP    changed int to long where required
                  added bit stream header options
                  removed NBC_RM4 switches (new -nh, -ms, -bi options)
10-dec-96   HP    new -ni, -vr options, added variable bit rate
23-jan-97   HP    added audio i/o module
07-jun-97   CL    Renamed this file (mp4enc.c) to encoder.c for 
                  use in AAC technical report encoder.  Removed
                  support for lpc and parametric codecs.  
07-apr-98   RG    added switch -lf to indicate presence of LFE channel   
27-may-98   MCF   Added File I/O for independent coupling channel support.
24-jul-98   MCF   Added support for dynamic range control. 
**********************************************************************/

/* Oct-2003  modified by RSY,I2R for MPEG-4 audio scalable lossless (SLS) coding */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <math.h>

//#include <libtsp.h>	/* AFsp audio file library */

#include "libtsp.h"	/* AFsp audio file library */

#include "common.h"	/* common module */
#include "cmdline.h"	/* command line module */
#include "bitstream.h"	/* bit stream module */
#include "audio.h"	/* audio i/o module */

#include "aac_qc.h"
#include "mp4.h"	/* frame work declarations */
#include "interface.h"
#include "drc.h"
#include "enc.h"	/* encoder cores */

#include "cbac.h" /* fish*/
#include "int_mdct_defs.h"

#ifdef USELIB_ISOMP4
#include "../MP4container/MP4moviewrapper.h"
#endif
/* ---------- declarations ---------- */


/* ---------- variables ---------- */

/* command line module */
static float bitRate;				   
static int varBitRate;
static int lfePresent;                    
static int pnsSfbStart;                 
static int bitReservSize;
static int bitReservInit;				  
static int bitReservInitUsed;
static float regionStart;
static float regionDurat;
static int regionDuratUsed;
static char *encPara;
static char *outFileName;
static char *cplFileName;
static int outFileNameUsed;
static int cplFileNameUsed;
static char *oriPath,*bitPath;
static int oriPathUsed,bitPathUsed;
static char *oriExt,*bitExt;
static int oriExtUsed;
static int noInfo;
static int noHeader;
static int cplenabl = 0;
static int isdisable = 0;
static int msdisable = 0;
static int tnsenable_switch = 0;
static char *magicString;
static int mainDebugLevel,audioDebugLevel,bitDebugLevel,cmdDebugLevel,quantDebugLevel;
static int *varArgIdx;
static int totalNumChans;
static int dynrng_enabl;
static double dynrng_import[DR_READLEN];
static short dynrng_shrtblk[MAX_SHORT_WINDOWS];
static short dynrng_sgn[MAX_SHORT_WINDOWS];

#ifdef BSAC
int codec_type;
int bsac_ch;
int bsac_sf;
#endif

#ifdef I2R_LOSSLESS
static float ll_bitRate;
static char * ll_bitExt;
static int coredisable = 0;
static int msStereoIntMDCTMode; /* 0: always off, 1: always on, 2: dynamic (default) */ 
#endif


#ifdef PLOTMTV
int frame; /* global for debugging */
#else 
static  int frame;
#endif 

static double cplmix[MAX_TIME_CHANNELS] = {0.7, 0.7, 0.7, 0.7, 0.7, 0.7};
static int numCplChannel;
static int cplMixMode = 0;
static int cplMixModeUsed = 0;

static int osf; /* oversampling factor */

static CmdLinePara paraList[] = {
  {&varArgIdx,NULL,"<audio file(s)>"},
  {NULL,NULL,NULL}
};

static CmdLineSwitch switchList[] = {
  {"h",NULL,NULL,NULL,NULL,"print help"},
  {"r",&bitRate,"%f","64000",NULL,"bit rate [bit/s]"},
#ifdef I2R_LOSSLESS
  {"llr",&ll_bitRate,"%f","30700000",NULL,"lossless enhance layer bit rate [bit/s]"},
  {"osf",&osf,"%d","1",NULL,"oversampling factor for lossless enhancement (dflt: 1, options: 1,2,4)"},
#endif
  {"vr",&varBitRate,NULL,NULL,NULL,"enable variable bit rate"},
  {"lf",&lfePresent,NULL,NULL,NULL,"LFE channel present"},             
  {"b",&bitReservSize,"%d","8184",NULL,"bit reservoir size [bit]"},
  {"bi",&bitReservInit,"%d",NULL,&bitReservInitUsed,
   "initial bit reservoir bits\n"
  "(dflt: bit reservoir size)"},
  {"c",&encPara,"%s","",NULL,
   "encoder parameter string"},
  {"dr",&dynrng_enabl,NULL,NULL,NULL,"enable dynamic range control metadata"},
  {"cp", &cplFileName,"%s",NULL,&cplFileNameUsed,"Independent couple chan file name, \n"
   "(must be same samplerate as input channels)"},
  {"cpn", &cplMixMode, "%d", 0, &cplMixModeUsed, "Independent couple mix mode:\n"
   "0:  Cpl chan mixed to all chans w. 6dB atten. (default)\n"
   "1:  Cpl chan mixed to left and right w. 3dB atten.\n"
   "2:  Cpl chan mixed to center (or l/r if no center) \n"
   "    w. 0dB atten."},
#ifdef BSAC
  {"bsac",&codec_type, NULL, NULL, NULL, "use BSAC core codec"},
#endif
#ifdef MPEG4V1
  {"pns",&pnsSfbStart,"%d","1000",NULL,"start band for pns"},
#endif

  {"ts",&regionStart,"%f","0",NULL,"start time of encoded region [sec]"},
  {"td",&regionDurat,"%f",NULL,&regionDuratUsed,
   "duration of encoded region [sec]\n"
   "(dflt: until end of file)"},
  {"o",&outFileName,"%s",NULL,&outFileNameUsed,"output file name"},
  {"po",&oriPath,"%s",NULL,&oriPathUsed,
   "default path original audio files (dflt: $" MP4_ORI_PATH_ENV ")"},
  {"pb",&bitPath,"%s",NULL,&bitPathUsed,
   "default path bit stream files (dflt: $" MP4_BIT_PATH_ENV ")"},
  {"eo",&oriExt,"%s",NULL,&oriExtUsed,
   "original audio file extension\n"
   "supported file formats: .au, .snd, .wav, .aif, .raw\n"
   "(dflt: $" MP4_ORI_FMT_ENV " / " MP4_ORI_EXT ")\n"
   "(raw format: $" MP4_ORI_RAW_ENV ")"},
  {"eb",&bitExt,"%s",".aac",NULL,"bit stream file extension"},
#ifdef I2R_LOSSLESS
  {"ll_eb",&ll_bitExt,"%s",".aaz",NULL,"lossless bit stream file extension"},
  {"ncore",&coredisable,NULL,NULL,NULL,"disable AAC core (default enable)"}, 
  {"msStereoIntMDCTMode",&msStereoIntMDCTMode,"%d","2",NULL,"0: always off, 1: always on, 2: dynamic (default)"}, 
#endif
  {"ni",&noInfo,NULL,NULL,NULL,"disable bit info string"},
  {"nh",&noHeader,NULL,NULL,NULL,"disable mp4 bit stream header"},
  /*  {"ms",&magicString,"%s",MP4_MAGIC,NULL,"bit stream magic string"},*/
  {"nms",&msdisable,NULL,NULL,NULL,"disable ms stereo (default on)"}, 
#ifndef I2R_LOSSLESS
  {"nis",&isdisable,NULL,NULL,NULL,"disable intensity stereo (default on)"}, 
#endif
  {"tns",&tnsenable_switch,NULL,NULL,NULL,"enable tns (default off)"}, 
  {"d",&mainDebugLevel,"%i","0",NULL,"main debug level"},
  {"da",&audioDebugLevel,"%i","0",NULL,"audio file debug level"},
  {"db",&bitDebugLevel,"%i","0",NULL,"bit stream debug level"},
  {"dc",&cmdDebugLevel,"%i","0",NULL,"cmd line debug level"},
  {"dq",&quantDebugLevel,"%i","0",NULL,"quantizer  debug level"},

  {NULL,NULL,NULL,NULL,NULL,NULL}
};

/* ---------- functions ---------- */

/* Encode() */
/* Encode audio file and generate bit stream. */
/* (This function evaluates the global xxxDebugLevel variables !!!) */

static int Encode (
  char *audioFileName,		/* in: audio file name */
  char *bitFileName,		/* in: bit stream file name */
#ifdef I2R_LOSSLESS
  char *ll_bitFileName,
#endif

  float bitRate,		/* in: bit rate [bit/sec] */
  int varBitRate,		/* in: variable bit rate */
  int lfePresent,               /* in: lfe channel present */
  int bitReservSize,		/* in: bit reservoir size [bit] */
  int bitReservInit,            /* in: number of bits in reservoir */
  char *encPara,		/* in: encoder parameter string */
  const char *info,			/* in: info string for bit stream */
  int noHeader,			/* in: disable bit stream header */
  char *magicString,		/* in: bit stream magic string */
  char *cplFileName,		/* in: name of coupling channel */
  float regionStart,		/* in: start time of region */
  float regionDurat		/* in: duration of region */
#ifdef I2R_LOSSLESS
  ,float ll_bitRate
#endif
  )
				/* returns: 0=OK  1=error */
{ 
  float fSample;
  float cplfSample;
  long fileNumSample;
  long cplFileNumSample;
  int cplNumSample, samp;
  long totNumSample;
  int numChannel;
  int frameNumSample,delayNumSample;

  int frameMinNumBit,frameMaxNumBit;
  long fSampleLong,bitRateLong;
  BsBitBuffer *bitBuf;
#ifdef I2R_LOSSLESS
	BsBitBuffer *ll_bitBuf;
#endif
  float **sampleBuf;
  float *cplSampleBuf[MAX_CPL_CHANNELS];
  BsBitStream *bitStream;
#ifdef I2R_LOSSLESS
  BsBitStream *ll_bitStream;
#endif
  AudioFile *audioFile;
  AudioFile *cplAudioFile;
  BsBitBuffer *bitHeader;
  FILE *tmpFile;
  int startupNumFrame,startupFrame;
  int numSample;
  int frameNumBit,frameAvailNumBit,usedNumBit;
  int minBitsNoOverflow;
  int headerNumBit;
  int padNumBit;
  long totDataNumBit,totPadNumBit;
  int minUsedNumBit,maxUsedNumBit;
  int minPadNumBit,maxPadNumBit;
  int minReservNumBit,maxReservNumBit;
  long totReservNumBit;
  int ch,i;
  long startSample;
  long encNumSample;
  DRC_INTF dvars_inpt;
  char *dr_metadata_name = "metadata.txt";
  FILE *dr_metadata;
#ifdef I2R_LOSSLESS
  int type_PCM = 0;
  long ll_bits_to_use;
  int frameLength = 0;
#endif

#ifdef USELIB_ISOMP4
MP4MovieStruct movieOut;
unsigned char mp4filename[255]={0};
unsigned int sampleBufLen[NUM_LAYERS]={0}, iAACBufSize=0, iAAZBufSize=0;
unsigned int duration;
unsigned int corepresent=!coredisable,  AOT=(!coredisable)?37:38;
unsigned int totalLayers=corepresent+1;
#endif


  /* init */
  if (mainDebugLevel >= 3) {
    printf("Encode:\n");
    printf("audioFileName=\"%s\"\n",audioFileName);
    printf("bitFileName=\"%s\"\n",bitFileName);
    printf("bitRate=%.3f\n",bitRate);
    printf("varBitRate=%d\n",varBitRate);
    printf("lfePresent=%d\n",lfePresent);     
    printf("bitReservSize=%d\n",bitReservSize);
    printf("bitReservInit=%d\n",bitReservInit);
    printf("encPara=\"%s\"\n",encPara?encPara:"(NULL)");
    printf("info=\"%s\"\n",info);
    printf("noHeader=%d\n",noHeader);
    /* printf("magicString=\"%s\"\n",magicString); */
    printf("regionStart=%.6f\n",regionStart);
    printf("regionDurat=%.6f\n",regionDurat);
  }

  BsInit(0,bitDebugLevel);
  AudioInit(getenv(MP4_ORI_RAW_ENV),	   /* headerless file format */
	    audioDebugLevel);

  /* check audio file */
  if ((tmpFile=fopen(audioFileName,"rb"))==NULL) {
    CommonWarning("Encode: error opening audio file %s",audioFileName);
    return 1;
  }
  fclose(tmpFile);

  /* open metadata file and set up dynamic range parameter buffers */
  if (dynrng_enabl)
  {
    if ((dr_metadata = fopen(dr_metadata_name,"rb"))==NULL) {
      CommonWarning("Encode: error importing metadata file %s", dr_metadata_name);
      return 1;
	}
    dvars_inpt.dyn_rng_sgn = dynrng_sgn;
    dvars_inpt.dyn_rng_ctl = dynrng_shrtblk;
  }

  /* open audio file */
  audioFile = AudioOpenRead(audioFileName,&numChannel,&fSample,
			    &fileNumSample
			);
  fileNumSample /= osf;

  if (audioFile==NULL)
    CommonExit(1,"Encode: error opening audio file %s "
	       "(maybe unknown format)", 
		   audioFileName);
#ifdef I2R_LOSSLESS
  type_PCM = AudioType(audioFile);
  fSample = fSample / osf;
#endif
	if (cplFileNameUsed) /* check audio file */
	{
       if ((tmpFile=fopen(cplFileName,"rb"))==NULL) {
	       CommonWarning("Encode: error opening coupling channel audio file %s",cplFileName);
	       return 1;
	   }
	   fclose(tmpFile);
	   cplenabl = 1;
	   totalNumChans = numChannel + MAX_CPL_CHANNELS;
	}  	
	else
	{
		cplenabl = 0;
		totalNumChans = numChannel;
	}

	if (cplenabl)						/* open coupling channel file */
	{
  		numCplChannel = 1;
		cplAudioFile = AudioOpenRead(cplFileName,&numCplChannel,&cplfSample,
			&cplFileNumSample);			/* cplFileNumSample is ignored */
		if (cplAudioFile==NULL)
		CommonExit(1,"Encode: error opening coupling channel audio file %s "
			"(maybe unknown format)", cplFileName);
		if (fSample != cplfSample)
		CommonExit(1,"Encode: error opening coupling channel audio file %s "
			"(samplerate differs from input channels)", cplFileName);
		if (numCplChannel != 1)
		CommonExit(1,"Encode: error opening coupling channel audio file %s "
			"(currently only supports one coupling channel)", cplFileName);
	}

  startSample = (long)(regionStart*fSample+0.5);
  if (regionDurat >= 0)
    /* encNumSample = (long)(regionDurat*fSample+0.5); */
    encNumSample = (long)(numChannel*regionDurat*fSample+0.5);
  else
    if (fileNumSample == 0)
      encNumSample = -1;
    else
      encNumSample = max(0,fileNumSample-startSample);

  /* init encoder */
  bitHeader = BsAllocBuffer(BITHEADERBUFSIZE);
  bitRateLong = (long)(bitRate+0.5);
  fSampleLong = (long)(fSample+0.5);

#ifdef I2R_LOSSLESS
	isdisable = 1;
	//msdisable = 0;
/* 	tnsdisable = 1; */
#endif

  EncTfInit(numChannel,
	    fSample,
	    bitRate,
	    encPara,
	    quantDebugLevel,
            bitReservInit,
            varBitRate,
	    &frameNumSample,
	    &delayNumSample,
	    bitHeader,
	    cplenabl,
	    isdisable,
	    msdisable,
	    !tnsenable_switch,
	    pnsSfbStart
#ifdef I2R_LOSSLESS
  	   ,type_PCM
	   ,coredisable
	    ,osf
#endif

		);

#ifdef BSAC
  //SLS_BSAC
  if (codec_type == 1)
  sam_init_encode_bsac(numChannel,(int)fSample,(int)bitRate,1024);
#endif
  frameNumBit = (int)(bitRate*frameNumSample/fSample+0.5);
#ifdef I2R_LOSSLESS
  ll_bits_to_use = (int)(ll_bitRate*frameNumSample/fSample+0.5);
#endif

  /* variable bit rate: minimum 8 bit/frame (1 byte) */
  /*                    to allow end_of_bitstream detection in decoder */
  frameMinNumBit = varBitRate ? 8 : frameNumBit;
  frameMaxNumBit = frameNumBit+bitReservSize;

  if (mainDebugLevel >= 3) {
    printf("fSample=%.3f Hz  (int=%ld)\n",fSample,fSampleLong);
    printf("bitRate=%.3f bit/sec  (int=%ld)\n",bitRate,bitRateLong);
    printf("bitReservSize=%d bit  (%.6f sec)\n",
	   bitReservSize,bitReservSize/bitRate);
    printf("bitReservInit=%d bit\n",bitReservInit);
    printf("frameNumSample=%d  (%.6f sec/frame)\n",
	   frameNumSample,frameNumSample/fSample);
    printf("delayNumSample=%d  (%.6f sec)\n",
	   delayNumSample,delayNumSample/fSample);
    printf("frameNumBit=%d\n",frameNumBit);
    printf("frameMinNumBit=%d\n",frameMinNumBit);
    printf("frameMaxNumBit=%d\n",frameMaxNumBit);
    printf("bitHeaderNumBit=%ld\n",BsBufferNumBit(bitHeader));
    printf("fileNumSample=%ld  (%.3f sec  %.3f frames)\n",
	   fileNumSample,fileNumSample/fSample,
	   fileNumSample/(float)frameNumSample);
    printf("startSample=%ld\n",startSample);
    printf("encNumSample=%ld  (%.3f sec  %.3f frames)\n",
	   encNumSample,encNumSample/fSample,
	   encNumSample/(float)frameNumSample);
  }

  /* allocate buffers */
  bitBuf = BsAllocBuffer(frameMaxNumBit);
#ifdef I2R_LOSSLESS
	ll_bitBuf = BsAllocBuffer(ll_bits_to_use+8); // I2R (27 Jun 07)
#endif

  if ((sampleBuf=(float**)malloc(numChannel*sizeof(float*)))==NULL)
    CommonExit(1,"Encode: memory allocation error");
  for (ch=0; ch<numChannel; ch++)
    if ((sampleBuf[ch]=(float*)malloc(MAX_OSF*frameNumSample*sizeof(float)))==NULL)
      CommonExit(1,"Encode: memory allocation error");

  if (cplenabl)
  {
    if ((cplSampleBuf[0]=(float*)malloc(frameNumSample*sizeof(float)))==NULL)
      CommonExit(1,"Encode: memory allocation error");
  }

  #ifdef USELIB_ISOMP4
 {
	char* dot = strchr(ll_bitFileName,'.');
	*dot= '\0'; dot = NULL;
	strcat(mp4filename,ll_bitFileName);
	strcat(mp4filename,".mp4");	// I2R (27 Jun 07)
 }
  if( MP4CreateNewMovie(&movieOut, !coredisable, (unsigned int) fSample, 
	  AOT, numChannel, type_PCM == 0 ? 8 : 16+(type_PCM-1)*4, osf, totalLayers) )  ("create movie file err") ;
// ???? 16+type_PCM*4

#else

  /* disable any header for now */
  noHeader=1;

  /* open bit stream file */

  if (!noHeader)
    bitStream = BsOpenFileWrite(bitFileName,magicString,(char*)info);
  else
    bitStream = BsOpenFileWrite(bitFileName,NULL, NULL);

  if (bitStream==NULL)
    CommonExit(1,"Encode: error opening bit stream file %s",bitFileName);

#ifdef I2R_LOSSLESS
  ll_bitStream = BsOpenFileWrite(ll_bitFileName,NULL, NULL);
  if (ll_bitStream==NULL)
    CommonExit(1,"Encode: error opening bit stream file %s",ll_bitFileName);
	  BsPutBit(ll_bitStream,type_PCM,2);
	  switch (osf)
		{
			case 1:
				frameLength = 0;
				break;
			case 2:
				frameLength = 1;
				break;
			case 4:
				frameLength = 2;
				break;
		}	
	  BsPutBit(ll_bitStream,frameLength,3);
	  ByteAlign(ll_bitStream);
#endif
	

  /* write bit stream header */

  if (!noHeader) {
    if (BsPutBuffer(bitStream,bitHeader))
      CommonExit(1,"Encode: error writing bit stream header");
    ByteAlign(bitStream);  /* byte align for AAC */
  }

  BsFreeBuffer(bitHeader);


  headerNumBit = BsCurrentBit(bitStream);

  if (mainDebugLevel >= 3)
    printf("headerNumBit=%d\n",headerNumBit);

#endif

  /* num frames to start up encoder due to delay compensation */
  startupNumFrame = (delayNumSample+frameNumSample-1)/frameNumSample;

  /* seek to beginning of first (startup) frame (with delay compensation) */
  AudioSeek(audioFile,
	    startSample+delayNumSample-startupNumFrame*frameNumSample);

  if (cplenabl)
  {
  /* seek to beginning of first (startup) frame (with delay compensation) */
	AudioSeek(cplAudioFile,
	    startSample+delayNumSample-startupNumFrame*frameNumSample);
  }

  if (mainDebugLevel >= 3)
    printf("startupNumFrame=%d\n",startupNumFrame);

  /* process audio file frame by frame */
  frame = -startupNumFrame;
  totNumSample = 0;
  totPadNumBit = 0;
  frameAvailNumBit = bitReservInit;
  minBitsNoOverflow = 0;
  minUsedNumBit = minPadNumBit = minReservNumBit = frameMaxNumBit;
  maxUsedNumBit = maxPadNumBit = maxReservNumBit = 0;
  totReservNumBit = 0;
  
  printf("MPEG-4 Audio Scalable Lossless (SLS) Coding Reference Encoder \n");

  do {
    if (mainDebugLevel >= 1 && mainDebugLevel <= 3) {
      printf("\rframe %4d ",frame);
      fflush(stdout);
    }
    if (mainDebugLevel > 3)
      printf("frame %4d\n",frame);

    /* check for startup frame */
    /* startupFrame = frame < 0;*/
	startupFrame=0; /* no such thing as startupFrame in lossless coding now...*/

    /* read audio file */
    numSample = AudioReadData(audioFile,sampleBuf,osf*frameNumSample);
    numSample /= osf;
    totNumSample += numSample;
			
	if (cplenabl)
	{
	    /* read audio file */
		cplNumSample = AudioReadData(cplAudioFile,cplSampleBuf,frameNumSample);
		for (samp = cplNumSample; samp < frameNumSample; samp++)
		{
			cplSampleBuf[0][samp] = 0.0;
		}

		SetCplMix(cplmix, numChannel);
	}

/* Read and format dynamic range words */

	dvars_inpt.dynrng_enabl = dynrng_enabl;
	if (dynrng_enabl)
	{
		ReadDynRngWords(&dvars_inpt, dr_metadata);
	}

    /* compute available number of bits */
    if (!startupFrame) {
      /* frameAvailNumBit contains number of bits in reservoir */
      /* variable bit rate: don't exceed bit reservoir size */
      if (frameAvailNumBit > bitReservSize)
	frameAvailNumBit = bitReservSize;
 
      /* Add to frameAvailNumBit the number of bits for this frame */
      frameAvailNumBit += frameNumBit;

      if (mainDebugLevel >= 5)
	printf("frameAvailNumBit=%d\n",frameAvailNumBit);

      /* For constant rate, compute minimum number of bits needed to avoid 
      overflowing bit reservoir.  For variable rate, use at least frameMinNumBit */
      minBitsNoOverflow = varBitRate ?
	                  frameMinNumBit : (frameAvailNumBit - bitReservSize);
      if (mainDebugLevel >= 5) printf("minBitsNoOverflow=%d\n",minBitsNoOverflow);  /*  YT for debug */

    }

    /* Encode frame */
    EncTfFrame(sampleBuf,
	       startupFrame ? (BsBitBuffer*)NULL : bitBuf,
#ifdef I2R_LOSSLESS
			startupFrame ? (BsBitBuffer*)NULL : ll_bitBuf,
#endif
	       startupFrame ? 0 : (frameAvailNumBit),
	       frameNumBit,
               minBitsNoOverflow,  /* use at least this number of bits */
	       frameMaxNumBit,
		   lfePresent,
		   cplmix,
		   cplSampleBuf[0],
		   &dvars_inpt
#ifdef I2R_LOSSLESS
		   ,ll_bits_to_use
	       ,osf
	       ,msStereoIntMDCTMode
#endif
		   );

    if (!startupFrame) {
      usedNumBit = BsBufferNumBit(bitBuf);

      if (mainDebugLevel >= 5)
	printf("usedNumBit=%d\n",usedNumBit);

#ifdef USELIB_ISOMP4
	if(corepresent)	
		memcpy((unsigned char*)*(movieOut.sampleBufH[0]),bitBuf->data,bitBuf->numBit/8);	 
// {
// 	static framecount=0;
// 	FILE *fp=fopen("SLSTest.txt", "ab");
// 	fwrite(ll_bitBuf->data, sizeof(char),ll_bitBuf->numBit/8, fp);
// 	fclose(fp);
// 
//}	
	memcpy((unsigned char*)*(movieOut.sampleBufH[corepresent]),ll_bitBuf->data,ll_bitBuf->numBit/8);	
	if(corepresent)	
		sampleBufLen[0]=bitBuf->numBit/8;
			
	sampleBufLen[corepresent]=ll_bitBuf->numBit/8;

	MP4SetNextAUintoMovie(&movieOut, sampleBufLen, 
				numSample, totalLayers);
#else

	  /* write bit stream */
      if (usedNumBit > frameAvailNumBit)
	CommonExit(1,"Encode: more bits used than available in frame+buffer");
      if (BsPutBuffer(bitStream,bitBuf))
		CommonExit(1,"Encode: error writing bit stream data");
#ifdef I2R_LOSSLESS 
      if (BsPutBuffer(ll_bitStream,ll_bitBuf))
	CommonExit(1,"Encode: error writing lossless bit stream data");
#endif

#endif
      /* write padding bits */
      padNumBit = 0;
      if (usedNumBit < minBitsNoOverflow) {
	padNumBit = minBitsNoOverflow - usedNumBit;
#ifdef BSAC
	if (codec_type == 1)
	padNumBit = 0;//SLS_BSAC
#endif
	if (mainDebugLevel >= 5)
	  printf("padNumBit=%d\n",padNumBit);

        if (padNumBit!=0) CommonExit(1,"Should be no padding bits for AAC (fill_elements)."); 

	for (i=0; i<padNumBit; i++)

	  if (BsPutBit(bitStream,0,1))
	    CommonExit(1,"Encode: error writing bit stream padding bits");

	totPadNumBit += padNumBit;
      } /* if (usedNumBit < minBitsNoOverflow) */

      /* Adjust available bit counts */
      if (padNumBit != 0 ) {
	CommonExit(1,"PADDING BITS!\n"); 
      }
      frameAvailNumBit -= usedNumBit;    /* Subtract bits used */
      frameAvailNumBit -= padNumBit;     /* Subtract bits padded */

      /* Keep track of stats */
      if (minUsedNumBit > usedNumBit)
	minUsedNumBit = usedNumBit;
      if (maxUsedNumBit < usedNumBit)
	maxUsedNumBit = usedNumBit;
      if (minPadNumBit > padNumBit)
	minPadNumBit = padNumBit;
      if (maxPadNumBit < padNumBit)
	maxPadNumBit = padNumBit;
      if (minReservNumBit > frameAvailNumBit)
	minReservNumBit = frameAvailNumBit;
      if (maxReservNumBit < frameAvailNumBit)
	maxReservNumBit = frameAvailNumBit;
    }

    frame++;
	printf("\r%d %",(int)(frame*(long)frameNumSample*100/encNumSample));
  } while (((encNumSample<0)&&(numSample==frameNumSample)) ||
	   ((encNumSample>=0)&&((frame)*(long)frameNumSample < encNumSample)));

  printf(" \n");


  /* free encoder memory */
    EncTfFree();

  /* close audio file */
  AudioClose(audioFile);

  /* close coupling channel file */
  if (cplenabl)
  {
	AudioClose(cplAudioFile);
    free(cplSampleBuf[0]);
  }
  

#ifdef USELIB_ISOMP4
	MP4OutputMovie(&movieOut,mp4filename, totalLayers);
#else
    totDataNumBit = BsCurrentBit(bitStream)-headerNumBit-totPadNumBit;

  if (mainDebugLevel >= 3) {
    printf("totNumFrame=%d\n",frame);
    printf("totNumSample=%ld\n",totNumSample);
    printf("totNumBit=%ld\n",BsCurrentBit(bitStream));
    printf("totDataNumBit=%ld  (%.3f bit/frame  %.3f bit/sec)\n",
	   totDataNumBit,totDataNumBit/(float)frame,
	   totDataNumBit/(float)frame*fSample/frameNumSample);
    printf("totPadNumBit=%ld\n",totPadNumBit);
    printf("lastPadNumBit=%d\n",frameAvailNumBit);
    printf("minUsedNumBit=%d\n",minUsedNumBit);
    printf("maxUsedNumBit=%d\n",maxUsedNumBit);
    printf("minPadNumBit=%d\n",minPadNumBit);
    printf("maxPadNumBit=%d\n",maxPadNumBit);
    printf("minReservNumBit=%d\n",minReservNumBit);
    printf("maxReservNumBit=%d\n",maxReservNumBit);
    printf("avgReservNumBit=%.1f\n",totReservNumBit/(float)frame);
  }



  /* close bit stream file */
  if (BsClose(bitStream))
    CommonExit(1,"Encode: error closing bit stream file");

#ifdef I2R_LOSSLESS
    if (BsClose(ll_bitStream))
    CommonExit(1,"Encode: error closing bit stream file");
#endif

#endif

  /* close metadata file */
  if (dynrng_enabl)
  {
	fclose(dr_metadata);
  }

  /* free buffers */
  if (numChannel>1)
    for (ch=0; ch<numChannel; ch++)
      free(sampleBuf[ch]);
  free(sampleBuf);
  BsFreeBuffer(bitBuf);
#ifdef I2R_LOSSLESS
	BsFreeBuffer(ll_bitBuf);
#endif
  return 0;
}


int ByteAlign(BsBitStream* ptrBs) {

  int len, i,j;
  len = BsCurrentBit( ptrBs );
	   
  j = (8 - (len%8))%8;

  if ((len % 8) == 0) j = 0;
  for( i=0; i<j; i++ ) {
    BsPutBit( ptrBs, 0, 1 ); 
  }
  return j;
}
  

void SetCplMix(double *cplmix, int numChannel)
{
	int ch;

	for (ch = 0; ch < MAX_TIME_CHANNELS; ch++)
	{
		cplmix[ch] = 0.0;
	}
	if ((numChannel - lfePresent) % 2)	/* Center channel present */
	{
		switch (cplMixMode)
		{
			case 0:
			for (ch = 0; ch < MAX_TIME_CHANNELS; ch++)
			{
				cplmix[ch] = 0.5;
			}
			break;
		case 1:
			cplmix[0] = 0.1;
			cplmix[1] = 0.707;
			cplmix[2] = 0.707;
			break;
		case 2:
			cplmix[0] = 1.0;
			break;					    
		}
	}
	else								/* No center channel present */
	{
		switch (cplMixMode)
		{
			case 0:
			for (ch = 0; ch < MAX_TIME_CHANNELS; ch++)
			{
				cplmix[ch] = 0.5;
			}
			break;
		case 1:
			cplmix[0] = 0.707;
			cplmix[1] = 0.707;
			break;
		case 2:
			cplmix[0] = 1.0;
			cplmix[1] = 1.0;
			break;					    
		}
	}
} /* end SetCplMix */


/* read metadata from text file.  There are 4 dynamic range control words for each group of
   256 samples.  The input dyn range words maps to dB with the following bounds:
   dynrng_import[i] = -1.0 results in -24dB of gain reduction	   (floating pt input)
   dynrng_import[i] = 0.0 results in unity gain (0dB)
   dynrng_import[i] = +1.0 results in +24dB of gain boost

   the output dyn range words map as follows:
   dynrng_shrtblk[i] = -128 results in -31.75dB of gain reduction  (0.25 dB increment steps)
   dynrng_shrtblk[i] = 0 results in unity gain (0dB)
   dynrng_shrtblk[i] = +127 results in +31.75dB of gain boost

   Format this data into AAC metadata words for dynamic range control words, 
   and program reference level words.  The program_reference_level word is derived from a word
   that covers the same range in dB, but has a more course step size of 1dB per step. */

void ReadDynRngWords(DRC_INTF *dvars_inpt, FILE *dr_metadata)
{
	short ref_level_import, i, cnt;
	double drtemp;
	double fact = 0.39685026299;	/* fact = 2^(-128/24) / 2^(-4) */
	double ifact;
	static short ref_level_import_sav;
	static double dynrng_import_sav;

#ifndef WIN32
	/* Needed for byte-swapping */
	/*	unsigned long *longptr;
	unsigned long *longptrhigh;
	double *doubleptr;
	unsigned long longval;
	unsigned long longvalhigh;*/
	int j;
	unsigned char *byteptr,*byteptr2;
	unsigned char byteval;
#endif

	ifact = 1.0 / fact;

	if (dynrng_enabl)
	{
		if (fread(&ref_level_import, sizeof(short), 1, dr_metadata) != 1)
		{
			ref_level_import = ref_level_import_sav;
		}
#ifndef WIN32
		/* byte swap this short */
		else {
		  byteptr = (unsigned char *)(&ref_level_import);
		  byteval = *byteptr; 
		  *byteptr = *(byteptr+1);
		  *(byteptr+1) = byteval;
		}
#endif

		if (fread(dynrng_import, sizeof(double), 4, dr_metadata) != 4)
		{
			for (i = 0; i < 4; i++)
			{
				dynrng_import[i] = dynrng_import_sav;
			}
		}
#ifndef WIN32
		/* byte swap these doubles */
		else {
			for (i = 0; i < 4; i++)
			{
			  byteptr = (unsigned char *)(&dynrng_import[i]);
			  byteptr2 = byteptr + sizeof(double) -1;
			  for (j=0;j<(sizeof(double)/2);j++) {
			    byteval = *byteptr;
			    *byteptr = *(byteptr2);
			    *(byteptr2) = byteval;
			    byteptr++;
			    byteptr2--;
			  }
			}
		}
#endif

		cnt = 0;
		for (i = 0; i < 8; i++)
		{
			if ((i % 2) == 0)
			{
				if (dynrng_import[cnt] < 0.0)
				{
					drtemp = fact * pow(10, (8.0 + 32.0 * dynrng_import[cnt++]) * 0.05);
					dynrng_sgn[i] = 1;
					dynrng_shrtblk[i] = -(short)(24.0 * log10(drtemp) / LG2);
				}
				else
				{
					drtemp = ifact * pow(10, (-8.0 + 32.0 * dynrng_import[cnt++]) * 0.05);
					dynrng_sgn[i] = 0;
					dynrng_shrtblk[i] = (short)(24.0 * log10(drtemp) / LG2);
				}
			}
			else
			{
				dynrng_shrtblk[i] = dynrng_shrtblk[i - 1];
				dynrng_sgn[i] = dynrng_sgn[i - 1];
			}
		}

		dvars_inpt->prog_ref_level = (ref_level_import << 2);
		ref_level_import_sav = ref_level_import;
		dynrng_import_sav = dynrng_import[0];
	}
}

	
/* ---------- main ---------- */

int main (int argc, char *argv[])
{
  char *progName = "<no program name>";
  int result;
  char oriFileName[STRLEN], bitFileName[STRLEN], fullCplFileName[STRLEN];
#ifdef I2R_LOSSLESS
  char ll_bitFileName[STRLEN];
#endif
  char infoDate[STRLEN];
  int i,j,len;
  int fileIdx;
  char *info,*infoTail;
  time_t curTime;



  /* evaluate command line  */
  CmdLineInit(0);
  result = CmdLineEval(argc,argv,paraList,switchList,1,&progName);
  if (result) {
    if (result==1) {
      printf("%s: %s\n",progName,PROGVER);
      CmdLineHelp(progName,paraList,switchList,stdout);
      /*      EncParInfo(stdout);
      EncLpcInfo(stdout);*/
      EncTfInfo(stdout);
      exit (1);
    }
    else
      CommonExit(1,"command line error (\"-h\" for help)");
  }

  if (mainDebugLevel >= 1)
    printf("%s: %s\n",progName,PROGVER);
  if (mainDebugLevel >= 2) {
/*     printf("CVS Id: %s\n",CVSID); */
    /*    printf("%s\n",EncParInfo(NULL));
    printf("%s\n",EncLpcInfo(NULL));*/
    printf("%s\n",EncTfInfo(NULL));
  }

  CmdLineInit(cmdDebugLevel);

  /* calc variable default values */
  if (!bitReservInitUsed)
    bitReservInit = bitReservSize;
  if (!regionDuratUsed)
    regionDurat = -1;
  if (!oriPathUsed)
    oriPath = getenv(MP4_ORI_PATH_ENV);
  if (!bitPathUsed)
    bitPath = getenv(MP4_BIT_PATH_ENV);
  if (!oriExtUsed)
    oriExt = getenv(MP4_ORI_FMT_ENV);
  if (oriExt==NULL)
    oriExt = MP4_ORI_EXT;

  /* check command line options */
  if (bitRate <= 0)
    CommonExit(1,"bit rate <= 0");
  if (bitReservSize < 0)
    CommonExit(1,"bit reservoir size < 0");
  if (bitReservInit < 0)
    CommonExit(1,"bit reservoir initial bits < 0");
  if (bitReservInit > bitReservSize)
    CommonExit(1,"bit reservoir initial bits > size");
  if (regionDuratUsed && regionDurat < 0)
    CommonExit(1,"duration of region < 0");
  if (outFileNameUsed && varArgIdx[0]>=0 && varArgIdx[1]>=0)
    CommonExit(1,"only one input file allowed when using -o");
  /*  if (cplFileNameUsed && varArgIdx[0]>=0 && varArgIdx[1]>=0)
      CommonExit(1,"only one coupling channel file allowed when using -cp");*/
  
  /* generate info string for bit stream */
  len = 1;
  curTime = time((time_t*)NULL);
  len += strftime(infoDate,STRLEN,"%Y/%m/%d %H:%M:%S UTC",gmtime(&curTime));
  len += strlen(PROGVER);
  j = 0;
  for (i=0; i<argc; i++)
    if (varArgIdx[j] == i)
      j++;
    else
      len += strlen(argv[i])+1;
  len += 2*STRLEN;
  len += 5*7+1;
  if ((info=(char*)malloc(len))==NULL)
    CommonExit(1,"memory allocation error");
  strcpy(info,"\n");
  strcat(info,"date: ");
  strcat(info,infoDate);	    
  strcat(info,"\n");
  strcat(info,"prog: ");
  strcat(info,PROGVER);	    
  strcat(info,"\n");
  strcat(info,"para:");
  j = 0;
  for (i=0; i<argc; i++) {
    if (varArgIdx[j] == i)
      j++;
    else {
      strcat(info," ");
      strcat(info,argv[i]);
    }
  }
  strcat(info,"\n");
  infoTail = info+strlen(info);

  /* process all files on command line */
  fileIdx = 0;
  while (varArgIdx[fileIdx] >= 0) {

    /* compose file names */
    if (ComposeFileName(argv[varArgIdx[fileIdx]],0,oriPath,oriExt,oriFileName,
			STRLEN))
      CommonExit(1,"composed file name too long");
    if (outFileNameUsed) {
      if (ComposeFileName(outFileName,0,bitPath,bitExt,bitFileName,
			  STRLEN))
		CommonExit(1,"composed file name too long");
#ifdef I2R_LOSSLESS
	  if (ComposeFileName(outFileName,1,bitPath,ll_bitExt,ll_bitFileName,
			  STRLEN))
		CommonExit(1,"composed file name too long");
#endif

    }
    else
	{
      if (ComposeFileName(argv[varArgIdx[fileIdx]],1,bitPath,bitExt,
			  bitFileName,STRLEN))
			CommonExit(1,"composed file name too long");
#ifdef I2R_LOSSLESS
	  if (ComposeFileName(argv[varArgIdx[fileIdx]],1,bitPath,ll_bitExt,
			  ll_bitFileName,STRLEN))
			CommonExit(1,"composed file name too long");
#endif
	}

	if (cplFileNameUsed)
	{
      if (ComposeFileName(cplFileName,0,oriPath,oriExt,fullCplFileName,
			  STRLEN))
	CommonExit(1,"composed file name too long");
    }
	else if (cplMixModeUsed)
	{
		CommonExit(1,"Must supply coupling channel file name");
	}

    /* complete info string */
    *infoTail = '\0';
    strcat(infoTail,"ori: ");
    strcat(infoTail,oriFileName);
    strcat(infoTail,"\n");
    strcat(infoTail,"bit: ");
    strcat(infoTail,bitFileName);
    strcat(infoTail,"\n");
	strcat(infoTail,fullCplFileName);
	strcat(infoTail, "\n");

    /* encode file */
    if (mainDebugLevel >= 1)
      printf("encoding %s -> %s\n",oriFileName,bitFileName);

    if (Encode(oriFileName,
	       bitFileName,
#ifdef I2R_LOSSLESS
		   ll_bitFileName,
#endif
	       /*	codecMode, */
	       bitRate,
	       varBitRate,
               lfePresent,
	       bitReservSize,
	       bitReservInit,
	       encPara,
	       noInfo ? "" : info,noHeader,
	       magicString,
               fullCplFileName,
	       regionStart,
	       regionDurat
#ifdef I2R_LOSSLESS
		   ,ll_bitRate /*lossless bit_rate*/
#endif
		   ))
      CommonWarning("error encoding audio file %s",oriFileName);

    fileIdx++;
  }
  
  CmdLineEvalFree(paraList);

  if (mainDebugLevel >= 1)
    printf("%s: finished\n",progName);

   return (0);
}

/* end of encoder.c */

