/**********************************************************************
 
This software module was originally developed by Texas Instruments
and edited by         in the course of 
development of the MPEG-2 NBC/MPEG-4 Audio standard
ISO/IEC 13818-7, 14496-1,2 and 3. This software module is an
implementation of a part of one or more MPEG-2 NBC/MPEG-4 Audio tools
as specified by the MPEG-2 NBC/MPEG-4 Audio standard. ISO/IEC gives
users of the MPEG-2 NBC/MPEG-4 Audio standards free license to this
software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-2 NBC/ MPEG-4 Audio
standards. Those intending to use this software module in hardware or
software products are advised that this use may infringe existing
patents. The original developer of this software module and his/her
company, the subsequent editors and their companies, and ISO/IEC have
no liability for use of this software module or modifications thereof
in an implementation. Copyright is not released for non MPEG-2
NBC/MPEG-4 Audio conforming products. The original developer retains
full right to use the code for his/her own purpose, assign or donate
the code to a third party and to inhibit third party from using the
code for non MPEG-2 NBC/MPEG-4 Audio conforming products. This
copyright notice must be included in all copies or derivative works.
 
Copyright (c) 1997.
**********************************************************************/

#include "tns.h"


/* Integer TNS for SLS */

#ifdef WIN32
	typedef __int64 INT64;
#else
	#include <stdint.h>
	typedef int64_t INT64;
#endif

#define SHIFT_INTTNS 21

static int tnsInvQuantCoefFixedPoint(int coef_res, int coef) 
{
  int intTnsCoef_res3[8] = {-2065292, -1816187, -1348023, -717268, 
                            0, 909920, 1639620, 2044572}; /* [-4,...,3] */
  int intTnsCoef_res4[16] = {-2088206, -2017095, -1877294, -1673563, 
                             -1412842, -1104008, -757579, -385351, 
                             0, 436022, 852989, 1232675, 
                             1558488, 1816187, 1994510, 2085664}; /* [-8,...,7] */

  if (coef_res == 3) {
    return intTnsCoef_res3[4+coef];
  }

  if (coef_res == 4) {
    return intTnsCoef_res4[8+coef];
  }
}

/* Decoder transmitted coefficients for one TNS filter */
static void
int_tns_decode_coef( int order, int coef_res, int *coef, int *a )
{
  int i, m;
  int tmp[TNS_MAX_ORDER+1], b[TNS_MAX_ORDER+1];


  /* Inverse quantization */
  for (i=0; i<order; i++)  {
    tmp[i+1] = tnsInvQuantCoefFixedPoint(coef_res, coef[i]);
  }
  /* Conversion to LPC coefficients
   * Markel and Gray,  pg. 95
   */

  /* worst case for order == 12 and all coefficients == 1: 
     6th coefficient raised by 12!/(6!*6!) = 924 
     -> 10 bits headroom required */

  a[0] = 1<<SHIFT_INTTNS;
  for (m=1; m<=order; m++)  {
    b[0] = a[0];
    for (i=1; i<m; i++)  {
	    b[i] = a[i] + ((((((INT64)tmp[m])*a[m-i])>>(SHIFT_INTTNS-1))+1)>>1);
    }
    b[m] = tmp[m];
    for (i=0; i<=m; i++)  {
	    a[i] = b[i];
    }
  }  
}

static void IntTnsInvFilter(int length, int* spec,TNS_FILTER_DATA* filter) {
	int i,j,k=0;
	int order=filter->order;
/* 	double* a=filter->aCoeffs; */
  int* coef = &filter->index[1];
  int a[TNS_MAX_ORDER+1];
	int* temp;
  INT64 temp_accu;

  int_tns_decode_coef(order, DEF_TNS_COEFF_RES, coef, a);


    temp = (int *) malloc (length * sizeof (int));
    if (!temp) {
      CommonExit( 1, "TnsInvFilter: Could not allocate memory for TNS array\nExiting program...\n");
    }

	/* Determine loop parameters for given direction */
  if (filter->direction) 
  {
    /* Startup, initial state is zero */
    temp[length-1]=spec[length-1];
    for (i=length-2;i>(length-1-order);i--) {
      temp[i]=spec[i];
      temp_accu = 0;
      k++;
      for (j=1;j<=k;j++) {
        temp_accu += ((INT64)temp[i+j]) * a[j];
      }
      spec[i] += (int)( ( ( temp_accu >> (SHIFT_INTTNS-1) ) + 1) >> 1 );
    }
    
    /* Now filter the rest */
    for	(i=length-1-order;i>=0;i--) {
      temp[i]=spec[i];
      temp_accu = 0;
      for (j=1;j<=order;j++) {
        temp_accu += ((INT64)temp[i+j]) * a[j]; 
      }
      spec[i] += (int)( ( ( temp_accu >> (SHIFT_INTTNS-1) ) + 1) >> 1 );
    }
    
    
  } else {
    /* Startup, initial state is zero */
    temp[0]=spec[0];
    for (i=1;i<order;i++) {
      temp[i]=spec[i];
      temp_accu = 0;
      for (j=1;j<=i;j++) {
        temp_accu += ((INT64)temp[i-j]) * a[j];
      }
      spec[i] += (int)( ( ( temp_accu >> (SHIFT_INTTNS-1) ) + 1) >> 1 );
    }
    
    /* Now filter the rest */
    for	(i=order;i<length;i++) {
      temp[i]=spec[i];
      temp_accu = 0;
      for (j=1;j<=order;j++) {
        temp_accu += ((INT64)temp[i-j])*a[j];
      }
      spec[i] += (int)( ( ( temp_accu >> (SHIFT_INTTNS-1) ) + 1) >> 1 );
    }
  }
  
  free(temp);
}


/*****************************************************/
/* IntTnsEncodeFilterOnly:                              */
/* This is a stripped-down version of TnsEncode()    */
/* which performs TNS analysis filtering only        */
/*****************************************************/
void IntTnsEncodeFilterOnly(int numberOfBands,          /* Number of bands per window */
			 int maxSfb,                 /* max_sfb */
			 enum WINDOW_TYPE blockType, /* block type */
			 int* sfbOffsetTable,        /* Scalefactor band offset table */
			 int* spec,               /* Spectral data array */
			 TNS_INFO* tnsInfo)          /* TNS info */
{
  int numberOfWindows,windowSize;
  int startBand,stopBand,order;    /* Bands over which to apply TNS */
  int w;
  int startIndex,length;

  switch( blockType ) {
    case ONLY_SHORT_WINDOW :
      numberOfWindows = NSHORT;
      windowSize = SN2;
      startBand = tnsInfo->tnsMinBandNumberShort;
      stopBand = numberOfBands; 
      startBand = min(startBand,tnsInfo->tnsMaxBandsShort);
      stopBand = min(stopBand,tnsInfo->tnsMaxBandsShort);
      break;
      
    default: 
      numberOfWindows = 1;
      windowSize = LN2;
      startBand = tnsInfo->tnsMinBandNumberLong;
      stopBand = numberOfBands;
      startBand = min(startBand,tnsInfo->tnsMaxBandsLong);
      stopBand = min(stopBand,tnsInfo->tnsMaxBandsLong);
      break;
  }
  
  /* Make sure that start and stop bands < maxSfb */
  /* Make sure that start and stop bands >= 0 */
  startBand = min(startBand,maxSfb);
  stopBand = min(stopBand,maxSfb);
  startBand = max(startBand,0);
  stopBand = max(stopBand,0);
  
  
  /* Perform filtering for each window */
  for(w=0;w<numberOfWindows;w++) 
  {
    TNS_WINDOW_DATA* windowData = &tnsInfo->windowData[w];
    TNS_FILTER_DATA* tnsFilter = windowData->tnsFilter;
    
    startIndex = w * windowSize + sfbOffsetTable[startBand];
    length = sfbOffsetTable[stopBand] - sfbOffsetTable[startBand];
    
    if (tnsInfo->tnsDataPresent  &&  windowData->numFilters) {  /* Use TNS */
      IntTnsInvFilter(length,&spec[startIndex],tnsFilter);
    }
  }  
}







