/**********************************************************************
MPEG-4 Audio VM



This software module was originally developed by

Fraunhofer Gesellschaft IIS / University of Erlangen (UER

and edited by

in the course of development of the MPEG-2 NBC/MPEG-4 Audio standard
ISO/IEC 13818-7, 14496-1,2 and 3. This software module is an
implementation of a part of one or more MPEG-2 NBC/MPEG-4 Audio tools
as specified by the MPEG-2 NBC/MPEG-4 Audio standard. ISO/IEC gives
users of the MPEG-2 NBC/MPEG-4 Audio standards free license to this
software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-2 NBC/ MPEG-4 Audio
standards. Those intending to use this software module in hardware or
software products are advised that this use may infringe existing
patents. The original developer of this software module and his/her
company, the subsequent editors and their companies, and ISO/IEC have
no liability for use of this software module or modifications thereof
in an implementation. Copyright is not released for non MPEG-2
NBC/MPEG-4 Audio conforming products. The original developer retains
full right to use the code for his/her own purpose, assign or donate
the code to a third party and to inhibit third party from using the
code for non MPEG-2 NBC/MPEG-4 Audio conforming products. This
copyright notice must be included in all copies or derivative works.

Copyright (c) 1996.

-----
This software module was modified by

Tadashi Araki (Ricoh Company, ltd.)
Tatsuya Okada (Waseda Univ.)
Itaru Kaneko (Graphics Communication Laboratories)
Hiroyuki Ohara (Ricoh Company, ltd.)

and edited by

in the course of development of the MPEG-2 NBC/MPEG-4 Audio standard
ISO/IEC 13818-7, 14496-1,2 and 3.

Almost all part of the function EncTf_psycho_acoustic() is made by
T. Araki and T. Okada, H. Ohara and its copyright belongs to Ricoh.
The function psy_get_absthr() is made by I. Kaneko
and its copyright belongs to Graphics Communication Laboratories.

Copyright (c) 1997.


Source file: 

$Id: psych.c,v 1.2 2004/05/10 16:11:34 ggr Exp $
$Id: psych.c,v 1.2 2004/05/10 16:11:34 ggr Exp $
$Id: psych.c,v 1.2 2004/05/10 16:11:34 ggr Exp $

**********************************************************************/

/* CREATED BY :  Bernhard Grill -- August-96  */
#include <math.h>

#include "tf_main.h"
#include "psych.h"
#include "common.h"

/* dummy psychoacoustic module */
/*
  static int sfb_width_long[36+1] = { 
        4,  4,  4,  4,  4,  4,  4,  4,  4,  6,  6,  6, 
        6,  8,  8,  8, 10, 10, 12, 12, 14, 16, 18, 20, 
       22, 24, 26, 28, 32, 36, 40, 44, 48, 54, 62, 70, 342
    };
    static int sfb_width_short[12+1] = {
        4,  4,  4,  4,  4,  6,  6,  8,  8, 10, 12, 16, 42
    };
    static double fixed_ratio_long[36+1] = { 
        0.04, 0.04, 0.04, 0.04, 0.04, 0.04, 0.04, 0.04, 0.04, 0.04, 0.05, 0.05,
        0.05, 0.06, 0.06, 0.06, 0.09, 0.09, 0.15, 0.15, 0.15, 0.15, 0.15, 0.15,
        0.15, 0.15, 0.20, 0.20, 0.20, 0.20, 0.20, 0.25, 0.30, 0.30, 0.30, 0.40, 
        1.00
    };
*/

 

static SR_INFO sr_info_aac[MAX_SAMPLING_RATES+1] =
{

/* added by T. Araki and H. Ohara (1998.09.03) */
  { 8000, 40, 15,
     { /*  cb_width_long[NSFB_LONG] */
      12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 16, 16, 16, 16, 16,
      16, 16, 20, 20, 20, 20, 24, 24, 24, 28, 28, 32, 36, 36, 40, 44, 48, 52,
      56, 60, 64, 80
     },
     { /* cb_width_short[NSFB_SHORT] */
      4, 4, 4, 4, 4, 4, 4, 8, 8, 8, 8, 12, 16, 20, 20
     }
  },
  { 11025, 43, 15,
      { /*  cb_width_long[NSFB_LONG] */
      8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 12, 12, 12, 12, 12, 12, 12, 12, 12,
      16, 16, 16, 16, 20, 20, 20, 24, 24, 28, 28, 32, 36, 40, 40, 44, 48,
      52, 56, 60, 64, 64, 64
      },
      { /* cb_width_short[NSFB_SHORT] */
       4, 4, 4, 4, 4, 4, 4, 4, 8, 8, 12, 12, 16, 20, 20
      }
  },
  { 12000, 43, 15,
      { /*  cb_width_long[NSFB_LONG] */
      8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 12, 12, 12, 12, 12, 12, 12, 12, 12,
      16, 16, 16, 16, 20, 20, 20, 24, 24, 28, 28, 32, 36, 40, 40, 44, 48,
      52, 56, 60, 64, 64, 64
      },
      { /* cb_width_short[NSFB_SHORT] */
       4, 4, 4, 4, 4, 4, 4, 4, 8, 8, 12, 12, 16, 20, 20
      }
  },
  { 16000, 43, 15,
      { /*  cb_width_long[NSFB_LONG] */
      8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 12, 12, 12, 12, 12, 12, 12, 12, 12,
      16, 16, 16, 16, 20, 20, 20, 24, 24, 28, 28, 32, 36, 40, 40, 44, 48,
      52, 56, 60, 64, 64, 64
      },
      { /* cb_width_short[NSFB_SHORT] */
       4, 4, 4, 4, 4, 4, 4, 4, 8, 8, 12, 12, 16, 20, 20
      }
  },
  { 22050, 47, 15,
     { /*  cb_width_long[NSFB_LONG] */
      4,  4,  4,  4,  4,  4,  4,  4,  4,  4,  4,  8,  8,  8,  8,  8,  8,  8,
      8,  8,  8,  12, 12, 12, 12, 16, 16, 16, 20, 20, 24, 24, 28, 28, 32,
      36, 36, 40, 44, 48, 52, 52, 64, 64, 64, 64, 64
     },
     { /* cb_width_short[NSFB_SHORT] */
      4,  4,  4,  4,  4,  4,  4,  8,  8,  8, 12, 12, 16, 16, 20
     }
  },
  { 24000, 47, 15,
    { /*  cb_width_long[NSFB_LONG] */
      4,  4,  4,  4,  4,  4,  4,  4,  4,  4,  4,  8,  8,  8,  8,  8,  8,  8,
      8,  8,  8,  12, 12, 12, 12, 16, 16, 16, 20, 20, 24, 24, 28, 28, 32,
      36, 36, 40, 44, 48, 52, 52, 64, 64, 64, 64, 64
     },
     { /* cb_width_short[NSFB_SHORT] */
      4,  4,  4,  4,  4,  4,  4,  8,  8,  8, 12, 12, 16, 16, 20
     }
  },
  { 32000, 51, 14,
    { /*  cb_width_long[NSFB_LONG] */
      4,  4,  4,  4,  4,  4,  4,  4,  4,  4,  8,  8,  8,  8,  8,  8,  8, 
      12, 12, 12, 12, 16, 16, 20, 20, 24, 24, 28, 28, 32, 32, 32, 32, 32, 32,
      32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32 
     },
     { /* cb_width_short[NSFB_SHORT] */
      4,  4,  4,  4,  4,  8,  8,  8, 12, 12, 12, 16, 16, 16
     }
  },
/* added by T. Araki and H. Ohara (1998.09.03) end */

/* added by T. Araki (1997.07.09) */
  { 44100, 49, 14,
     { /*  cb_width_long[NSFB_LONG] */
      4,  4,  4,  4,  4,  4,  4,  4,  4,  4,  8,  8,  8,  8,  8,  8,  8, 
      12, 12, 12, 12, 16, 16, 20, 20, 24, 24, 28, 28, 32, 32, 32, 32, 32, 32,
      32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 96 
     }, 
     { /* cb_width_short[NSFB_SHORT] */
      4,  4,  4,  4,  4,  8,  8,  8, 12, 12, 12, 16, 16, 16 
     },
     { /* fixed_ratio_long[NSFB_LONG] */
      0.04, 0.04, 0.04, 0.04, 0.04, 0.04, 0.04, 0.04, 0.04, 0.04, 0.04, 0.04, 
      0.04, 0.04, 0.05, 0.05, 0.05, 0.05, 0.05, 0.06, 0.06, 0.06, 0.06, 0.09, 
      0.09, 0.09, 0.15, 0.15, 0.15, 0.15, 0.15, 0.15, 0.15, 0.15, 0.15, 0.15, 
      0.20, 0.20, 0.20, 0.20, 0.20, 0.20, 0.20, 0.25, 0.30, 0.30, 0.30, 0.40,
      1.00
     },
     { /* fixed_ratio_short[NSFB_SHORT] */
      0.04, 0.04, 0.04, 0.05, 0.07, 0.10, 0.12, 0.15, 0.15, 0.20, 0.25, 0.30, 0.40, 1.00
     }
  },
/* added by T. Araki (1997.07.09) end */

  { 48000, 49, 14,
     { /*  cb_width_long[NSFB_LONG] */
      4,  4,  4,  4,  4,  4,  4,  4,  4,  4,  8,  8,  8,  8,  8,  8,  8, 
      12, 12, 12, 12, 16, 16, 20, 20, 24, 24, 28, 28, 32, 32, 32, 32, 32, 32,
      32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 96 
     }, 
     { /* cb_width_short[NSFB_SHORT] */
      4,  4,  4,  4,  4,  8,  8,  8, 12, 12, 12, 16, 16, 16 
     },
     { /* fixed_ratio_long[NSFB_LONG] */
      0.04, 0.04, 0.04, 0.04, 0.04, 0.04, 0.04, 0.04, 0.04, 0.04, 0.04, 0.04, 
      0.04, 0.04, 0.05, 0.05, 0.05, 0.05, 0.05, 0.06, 0.06, 0.06, 0.06, 0.09, 
      0.09, 0.09, 0.15, 0.15, 0.15, 0.15, 0.15, 0.15, 0.15, 0.15, 0.15, 0.15, 
      0.20, 0.20, 0.20, 0.20, 0.20, 0.20, 0.20, 0.25, 0.30, 0.30, 0.30, 0.40,
      1.00
     },
     { /* fixed_ratio_short[NSFB_SHORT] */
      0.04, 0.04, 0.04, 0.05, 0.07, 0.10, 0.12, 0.15, 0.15, 0.20, 0.25, 0.30, 0.40, 1.00
     }
  },

/* added by T. Araki (1998.09.03) */
  { 64000, 47, 12,
     { /*  cb_width_long[NSFB_LONG] */
      4,4,4,4,4,4,4,4,4,4,4,4,4,4,8,8,8,8,12,12,12,
      16,16,16,20,24,24,28,36,40,40,40,40,40,40,40,
      40,40,40,40,40,40,40,40,40,40,40
     }, 
     { /* cb_width_short[NSFB_SHORT] */
      4,4,4,4,4,4,8,8,8,16,28,36
     }
  },
  { 88200, 41, 12,
     { /*  cb_width_long[NSFB_LONG] */
      4,4,4,4,4,4,4,4,4,4,4,4,4,4,8,8,8,8,8,12,12,12,
      12,12,16,16,24,28,36,44,64,64,64,64,64,64,64,64,64,64,64
     },
     { /* cb_width_short[NSFB_SHORT] */
      4,4,4,4,4,4,8,8,8,16,28,36
     }
  },
  { 96000, 41, 12,
     { /*  cb_width_long[NSFB_LONG] */
      4,4,4,4,4,4,4,4,4,4,4,4,4,4,8,8,8,8,8,12,12,12,
      12,12,16,16,24,28,36,44,64,64,64,64,64,64,64,64,64,64,64
     },
     { /* cb_width_short[NSFB_SHORT] */
      4,4,4,4,4,4,8,8,8,16,28,36
     }
  },
/* added by T. Araki (1998.09.03) end */
#ifdef I2R_LOSSLESS
  { 192000, 41, 12,
     { /*  cb_width_long[NSFB_LONG] */
      4,4,4,4,4,4,4,4,4,4,4,4,4,4,8,8,8,8,8,12,12,12,
      12,12,16,16,24,28,36,44,64,64,64,64,64,64,64,64,64,64,64
     },
     { /* cb_width_short[NSFB_SHORT] */
      4,4,4,4,4,4,8,8,8,16,28,36
     }
  },

#endif
  { -1 }
};

static SR_INFO sr_info_960[MAX_SAMPLING_RATES+1] =
{
  { 8000  },
  { 11025 },
  { 12000 },
  { 16000 },
  { 22050 },
  { 24000 },
  { 32000 },
  { 44100 },
  { 48000, 49, 14,
     { /*  cb_width_long[NSFB_LONG] */
       4,4,4, 4, 4, 4, 4, 4, 4, 4, 8, 8, 8, 8, 8, 8, 8, 12, 12, 12, 12, 16, 16, 20, 20, 24, 24, 28, 28, 32, 32, 32,
     32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32
       
     }, 
     { /* cb_width_short[NSFB_SHORT] */
       4,4,4, 4, 4, 8, 8, 8, 12,12,12,16,16, 8
     },
     { /* fixed_ratio_long[NSFB_LONG] */
      0.04, 0.04, 0.04, 0.04, 0.04, 0.04, 0.04, 0.04, 0.04, 0.04, 0.04, 0.04, 
      0.04, 0.04, 0.05, 0.05, 0.05, 0.05, 0.05, 0.06, 0.06, 0.06, 0.06, 0.09, 
      0.09, 0.09, 0.15, 0.15, 0.15, 0.15, 0.15, 0.15, 0.15, 0.15, 0.15, 0.15, 
      0.20, 0.20, 0.20, 0.20, 0.20, 0.20, 0.20, 0.25, 0.30, 0.30, 0.30, 0.40,
      1.00
     },
     { /* fixed_ratio_short[NSFB_SHORT] */
      0.04, 0.04, 0.04, 0.05, 0.07, 0.10, 0.12, 0.15, 0.15, 0.20, 0.25, 0.30, 0.40, 1.00
     }
  },
  { 96000 },
  { -1 }
};


static SR_INFO sr_info_NOT_aac[MAX_SAMPLING_RATES+1] =
{
  { 8000  },
  { 11025 },
  { 12000 },
  { 16000 },
  { 22050 },
  { 24000 },
  { 32000 },
  { 44100 },
  { 48000, 37, 13,
    { /*  cb_width_long[NSFB_LONG] */
      4,  4,  4,  4,  4,  4,  4,  4,  4,  6,  6,  6, 
      6,  8,  8,  8, 10, 10, 12, 12, 14, 16, 18, 20, 
      22, 24, 26, 28, 32, 36, 40, 44, 48, 54, 62, 70, 342
    },
    { /* cb_width_short[NSFB_SHORT] */
      4,  4,  4,  4,  4,  6,  6,  8,  8, 10, 12, 16, 42
    },
    { /* fixed_ratio_long[NSFB_LONG] */
      0.04, 0.04, 0.04, 0.04, 0.04, 0.04, 0.04, 0.04, 0.04, 0.04, 0.05, 0.05,
      0.05, 0.06, 0.06, 0.06, 0.09, 0.09, 0.15, 0.15, 0.15, 0.15, 0.15, 0.15,
      0.15, 0.15, 0.20, 0.20, 0.20, 0.20, 0.20, 0.25, 0.30, 0.30, 0.30, 0.40, 
      1.00
    },
    { /* fixed_ratio_short[NSFB_SHORT] */
      0.04, 0.04, 0.04, 0.05, 0.07, 0.10, 0.12, 0.15, 0.15, 0.20, 0.25, 0.30, 
      0.40, 1.00
    }
  },
  { 96000 },
  { -1 }
};

/* added by T. Araki (1997.07.10) */
static PARTITION_TABLE_LONG  part_tbl_long_all[MAX_SAMPLING_RATES+1] =
{

/* added by T. Araki and H. Ohara (1998.09.03) */
  { 8000, 52,
     { /* w_low */
      0, 9, 18, 27, 36, 45, 54, 63, 72, 81, 90, 99, 109, 119, 129, 139, 149,
      160, 171, 182, 193, 205, 217, 229, 242, 255, 269, 283, 298, 313, 329,
      346, 364, 382, 401, 421, 442, 465, 489, 515, 542, 571, 602, 635, 671,
      709, 750, 794, 842, 894, 950, 1010
     },
     { /* w_high */
      8, 17, 26, 35, 44, 53, 62, 71, 80, 89, 98, 108, 118, 128, 138, 148,
      159, 170, 181, 192, 204, 216, 228, 241, 254, 268, 282, 297, 312, 328,
      345, 363, 381, 400, 420, 441, 464, 488, 514, 541, 570, 601, 634, 670,
      708, 749, 793, 841, 893, 949, 1009, 1023
     },
     { /* width */
      9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 10, 10, 10, 10, 10, 11, 11, 11, 11,
      12, 12, 12, 13, 13, 14, 14, 15, 15, 16, 17, 18, 18, 19, 20, 21, 23, 24,
      26, 27, 29, 31, 33, 36, 38, 41, 44, 48, 52, 56, 60, 14
     },
     { /* bval */
      0.18, 0.53, 0.89, 1.24, 1.59, 1.94, 2.29, 2.63, 2.98, 3.31, 3.65, 3.99,
      4.35, 4.71, 5.05, 5.39, 5.74, 6.10, 6.45, 6.79, 7.13, 7.48, 7.82, 8.17,
      8.51, 8.85, 9.20, 9.54, 9.88, 10.22, 10.56, 10.91, 11.25, 11.58, 11.91,
      12.24, 12.58, 12.92, 13.26, 13.59, 13.93, 14.26, 14.60, 14.93, 15.27,
      15.60, 15.93, 16.26, 16.60, 16.93, 17.26, 17.47
     },
     { /* qsthr */
      46.82, 46.82, 46.82, 41.82, 41.82, 41.82, 38.82, 38.82, 38.82, 33.82,
      33.82, 34.28, 32.28, 32.28, 32.28, 32.28, 32.69, 32.69, 32.69, 32.69,
      33.07, 33.07, 33.07, 33.42, 33.42, 33.74, 33.74, 34.04, 34.04, 34.32,
      34.58, 34.83, 34.83, 35.06, 35.29, 35.50, 35.89, 36.08, 36.43, 36.59,
      36.90, 37.19, 37.46, 37.84, 38.07, 38.40, 38.71, 39.09, 39.44, 39.76,
      40.06, 33.74
     }
  },
  { 11025, 56,
     { /* w_low */
       0,7,14,21,28,35,42,49,56,63,70,77,84,91,98,106,
       114,122,130,138,147,156,165,174,184,194,204,215,226,238,250,263,
       276,290,305,321,338,356,375,395,416,439,463,489,517,547,580,615,
       653,694,738,786,837,892,951,1015
     },
     { /* w_high */
       6,13,20,27,34,41,48,55,62,69,76,83,90,97,105,113,
       121,129,137,146,155,164,173,183,193,203,214,225,237,249,262,275,
       289,304,320,337,355,374,394,415,438,462,488,516,546,579,614,652,
       693,737,785,836,891,950,1014,1023
     },
     { /* width */
       7,7,7,7,7,7,7,7,7,7,7,7,7,7,8,8,8,8,8,
       9,9,9,9,10,10,10,11,11,12,12,13,13,14,15,16,17,18,19,20,
       21,23,24,26,28,30,33,35,38,41,44,48,51,55,59,64,9
     },
     { /* bval */
       0.19,0.57,0.95,1.33,1.71,2.08,2.45,2.82,3.18,3.54,
       3.89,4.24,4.59,4.92,5.28,5.65,6.01,6.36,6.7,7.06,
       7.42,7.77,8.11,8.46,8.82,9.16,9.5,9.85,10.19,10.54,
       10.88,11.22,11.56,11.9,12.24,12.59,12.94,13.28,13.62,13.96,
       14.29,14.63,14.96,15.29,15.63,15.96,16.3,16.63,16.97,17.3,
       17.64,17.97,18.3,18.64,18.97,19.16
     },
     { /* qsthr */
       45.73,45.73,45.73,40.73,40.73,37.73,37.73,37.73,32.73,32.73,
       32.73,30.73,30.73,30.73,31.31,31.31,31.31,31.31,31.31,31.82,
       31.82,31.82,31.82,32.28,32.28,32.28,32.69,32.69,33.07,33.07,
       33.42,33.42,33.74,34.04,34.32,34.58,34.83,35.06,35.29,35.5,
       35.89,36.08,36.43,36.75,37.05,37.46,37.72,38.07,38.4,38.71,
       39.09,39.35,39.68,39.98,40.34,31.82
     }
  },
  { 12000, 57,
     { /* w_low */
       0,6,12,18,24,30,36,42,48,54,60,66,73,80,
       87,94,101,108,115,123,131,139,147,155,164,173,182,192,
       202,213,224,236,248,261,274,288,303,319,336,354,373,393,
       415,438,463,490,519,550,584,620,659,701,746,795,848,905,966
     },
     { /* w_high */
       5,11,17,23,29,35,41,47,53,59,65,72,79,86,
       93,100,107,114,122,130,138,146,154,163,172,181,191,201,
       212,223,235,247,260,273,287,302,318,335,353,372,392,414,
       437,462,489,518,549,583,619,658,700,745,794,847,904,965,1023
     },
     { /* width */
       6,6,6,6,6,6,6,6,6,6,6,7,7,7,7,7,7,7,
       8,8,8,8,8,9,9,9,10,10,11,11,12,12,13,13,
       14,15,16,17,18,19,20,22,23,25,27,29,31,
       34,36,39,42,45,49,53,57,61,58 
    },
     { /* bval */
       0.18,0.53,0.89,1.24,1.59,1.94,2.29,2.63,2.98,3.31,
       3.65,4,4.38,4.75,5.11,5.47,5.82,6.15,6.51,6.88,
       7.24,7.58,7.92,8.27,8.62,8.96,9.31,9.66,10.01,10.36,
       10.71,11.06,11.41,11.75,12.09,12.43,12.77,13.11,13.46,13.8,
       14.13,14.47,14.81,15.14,15.48,15.81,16.15,16.48,16.82,17.15,
       17.48,17.81,18.14,18.48,18.81,19.15,19.47
     },
     { /* qsthr */
       45.06,45.06,45.06,40.06,40.06,40.06,37.06,37.06,
       37.06,32.06,32.06,30.73,30.73,30.73,30.73,30.73,
       30.73,30.73,31.31,31.31,31.31,31.31,31.31,31.82,
       31.82,31.82,32.28,32.28,32.69,32.69,33.07,33.07,
       33.42,33.42,33.74,34.04,34.32,34.58,34.83,35.06,
       35.29,35.7,35.89,36.26,36.59,36.9,37.19,37.59,
       37.84,38.19,38.51,38.81,39.18,39.52,39.83,40.13,39.91
     }
  },
  { 16000, 60,
     { /* w_low */
       0,5,10,15,20,25,30,35,40,45,50,55,60,65,70,
       75,81,87,93,99,105,111,117,124,131,138,145,153,161,169,
       178,187,197,207,218,229,241,254,268,283,299,316,334,353,374,
       396,420,446,474,504,537,572,610,651,695,742,792,846,904,966
     },
     { /* w_high */
       4,9,14,19,24,29,34,39,44,49,
       54,59,64,69,74,80,86,92,98,104,
       110,116,123,130,137,144,152,160,168,177,
       186,196,206,217,228,240,253,267,282,298,
       315,333,352,373,395,419,445,473,503,536,
       571,609,650,694,741,791,845,903,965,1023
     },
     { /* width */
       5,5,5,5,5,5,5,5,5,5,
       5,5,5,5,5,6,6,6,6,6,
       6,6,7,7,7,7,8,8,8,9,
       9,10,10,11,11,12,13,14,15,16,
       17,18,19,21,22,24,26,28,30,33,
       35,38,41,44,47,50,54,58,62,58
     },
     { /* bval */
       0.2,0.59,0.99,1.38,1.77,2.16,2.54,2.92,3.29,3.66,
       4.03,4.39,4.74,5.09,5.43,5.79,6.18,6.56,6.92,7.28,
       7.63,7.96,8.31,8.68,9.03,9.37,9.71,10.07,10.41,10.75,
       11.1,11.45,11.8,12.14,12.48,12.82,13.16,13.51,13.86,14.21,
       14.56,14.9,15.24,15.58,15.91,16.25,16.58,16.92,17.25,17.59,
       17.93,18.26,18.6,18.94,19.27,19.6,19.94,20.27,20.61,20.92
     },
     { /* qsthr */
       43.30,43.10,38.30,38.10,38.00,35.10,35.30,30.00,30.00,28.30,
       28.30,28.30,28.30,28.30,28.30,28.30,28.30,28.00,29.27,29.27,
       29.27,29.27,29.27,29.06,30.06,30.06,30.06,30.73,30.73,30.73,
       31.31,31.31,31.82,31.82,32.28,32.28,32.69,32.69,33.07,33.46,
       33.82,34.12,34.42,34.68,35.15,35.32,35.73,35.91,36.42,36.75,
       37.11,37.34,37.63,38.12,38.17,41.52,41.84,42.13,44.41,44.87
     }    
  },
  { 22050, 63,
     { /* w_low */
      0, 4, 8, 12, 16, 20, 24, 28, 32, 36, 40, 44, 48, 52, 56, 60, 64, 68,
      72, 77, 82, 87, 92, 97, 102, 108, 114, 120, 126, 133, 140, 147, 155,
      163, 172, 181, 191, 201, 212, 224, 237, 251, 266, 282, 299, 318, 338,
      360, 383, 408, 435, 464, 495, 528, 564, 602, 643, 687, 734, 785, 840,
      899, 963
     },
     { /* w_high */
      3, 7, 11, 15, 19, 23, 27, 31, 35, 39, 43, 47, 51, 55, 59, 63, 67, 71,
      76, 81, 86, 91, 96, 101, 107, 113, 119, 125, 132, 139, 146, 154, 162,
      171, 180, 190, 200, 211, 223, 236, 250, 265, 281, 298, 317, 337, 359,
      382, 407, 434, 463, 494, 527, 563, 601, 642, 686, 733, 784, 839, 898,
      962, 1023
     },
     { /* width */
      4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 5, 5, 5, 5, 5, 5,
      6, 6, 6, 6, 7, 7, 7, 8, 8, 9, 9, 10, 10, 11, 12, 13, 14, 15, 16, 17,
      19, 20, 22, 23, 25, 27, 29, 31, 33, 36, 38, 41, 44, 47, 51, 55, 59,
      64, 61
     },
     { /* bval */
      0.22, 0.65, 1.09, 1.52, 1.95, 2.37, 2.79, 3.21, 3.62, 4.02, 4.41, 4.80,
      5.18, 5.55, 5.92, 6.27, 6.62, 6.95, 7.32, 7.71, 8.10, 8.46, 8.82, 9.16,
      9.52, 9.89, 10.25, 10.59, 10.95, 11.31, 11.65, 12.00, 12.35, 12.70,
      13.05, 13.40, 13.74, 14.07, 14.41, 14.76, 15.11, 15.46, 15.80, 16.14,
      16.48, 16.82, 17.16, 17.50, 17.84, 18.17, 18.51, 18.84, 19.17, 19.51,
      19.84, 20.17, 20.50, 20.84, 21.17, 21.50, 21.84, 22.17, 22.48
     },
     { /* qsthr */
      43.30, 43.30, 38.30, 38.30, 38.30, 35.30, 35.30, 30.30, 30.30, 28.30,
      28.30, 28.30, 28.30, 28.30, 28.30, 28.30, 28.30, 28.30, 29.27, 29.27,
      29.27, 29.27, 29.27, 29.27, 30.06, 30.06, 30.06, 30.06, 30.73, 30.73,
      30.73, 31.31, 31.31, 31.82, 31.82, 32.28, 32.28, 32.69, 33.07, 33.42,
      33.74, 34.04, 34.32, 34.58, 35.06, 35.29, 35.70, 35.89, 36.26, 36.59,
      36.90, 37.19, 37.46, 37.84, 38.07, 41.40, 41.71, 42.00, 44.35, 44.68,
      44.98, 50.34, 50.13
     }
  },
  { 24000, 66,
     { /* w_low */
       0,3,6,9,12,15,18,21,24,27,30,
       33,37,41,45,49,53,57,61,65,69,73,
       77,82,87,92,97,102,108,114,120,126,133,
       140,147,155,163,172,181,191,202,214,226,239,
       253,268,285,303,322,343,365,389,415,443,473,
       505,539,576,615,657,702,751,804,861,923,990
     },
     { /* w_high */
       2,5,8,11,14,17,20,23,26,29,32,
       36,40,44,48,52,56,60,64,68,72,76,
       81,86,91,96,101,107,113,119,125,132,139,
       146,154,162,171,180,190,201,213,225,238,252,
       267,284,302,321,342,364,388,414,442,472,504,
       538,575,614,656,701,750,803,860,922,989,1023
     },
     { /* width */
       3,3,3,3,3,3,3,3,3,3,3,
       4,4,4,4,4,4,4,4,4,4,4,
       5,5,5,5,5,6,6,6,6,7,7,
       7,8,8,9,9,10,11,12,12,13,14,
       15,17,18,19,21,22,24,26,28,30,32,
       34,37,39,42,45,49,53,57,62,67,34
     },
     { /* bval */
       0.18,0.53,0.89,1.24,1.59,1.94,2.29,2.63,2.98,3.31,
       3.65,4.03,4.46,4.88,5.29,5.69,6.08,6.46,6.83,7.19,
       7.54,7.88,8.25,8.64,9.02,9.38,9.73,10.09,10.47,10.83,
       11.18,11.53,11.89,12.23,12.57,12.92,13.26,13.61,13.95,14.29,
       14.65,15,15.33,15.66,16,16.34,16.69,17.02,17.36,17.7,
       18.03,18.37,18.7,19.04,19.38,19.71,20.04,20.38,20.71,21.04,
       21.37,21.7,22.04,22.37,22.7,22.95
     },
     { /* qsthr */
       42.05,42.05,42.05,37.05,37.05,37.05,
       34.05,34.05,34.05,29.05,29.05,28.3,
       28.3,28.3,28.3,28.3,28.3,28.3,
       28.3,28.3,28.3,28.3,29.27,29.27,
       29.27,29.27,29.27,30.06,30.06,30.06,
       30.06,30.73,30.73,30.73,31.31,31.31,
       31.82,31.82,32.28,32.69,33.07,33.07,
       33.42,33.74,34.04,34.58,34.83,35.06,
       35.5,35.7,36.08,36.43,36.75,37.05,
       37.33,37.59,40.96,41.19,41.51,43.81,
       44.18,44.52,49.83,50.2,50.54,47.59
     }
  },
  { 32000, 66,
     { /* w_low */
       0,3,6,9,12,15,18,21,24,27,30,
       33,36,39,42,45,48,51,54,57,61,65,
       69,73,77,81,85,90,95,100,105,111,117,
       123,130,137,145,153,162,172,182,193,205,218,
       232,247,263,280,299,319,341,364,389,416,445,
       476,509,544,582,623,668,716,769,827,891,962
     },
     { /* w_high */
       2,5,8,11,14,17,20,23,26,29,32,
       35,38,41,44,47,50,53,56,60,64,68,
       72,76,80,84,89,94,99,104,110,116,122,
       129,136,144,152,161,171,181,192,204,217,231,
       246,262,279,298,318,340,363,388,415,444,475,
       508,543,581,622,667,715,768,826,890,961,1023
     },
     { /* width */
       3,3,3,3,3,3,3,3,3,3,3,
       3,3,3,3,3,3,3,3,4,4,4,
       4,4,4,4,5,5,5,5,6,6,6,
       7,7,8,8,9,10,10,11,12,13,14,
       15,16,17,19,20,22,23,25,27,29,31,
       33,35,38,41,45,48,53,58,64,71,62
     },
     { /* bval */
       0.24,0.71,1.18,1.65,2.12,2.58,3.03,3.48,
       3.92,4.35,4.77,5.19,5.59,5.99,6.37,6.74,
       7.1,7.45,7.8,8.18,8.6,9,9.39,9.76,
       10.11,10.45,10.81,11.19,11.55,11.9,12.25,12.62,
       12.96,13.31,13.66,14.01,14.36,14.71,15.07,15.42,
       15.76,16.1,16.45,16.8,17.14,17.48,17.82,18.15,
       18.49,18.84,19.17,19.51,19.85,20.19,20.53,20.87,
       21.2,21.53,21.86,22.2,22.53,22.86,23.2,23.53,
       23.86,24
     },
     { /* qsthr */
       42.05,42.05,37.05,37.05,34.05,34.05,29.05,29.05,
       29.05,27.05,27.05,27.05,27.05,27.05,27.05,27.05,
       27.05,27.05,27.05,28.3,28.3,28.3,28.3,28.3,
       28.3,28.3,29.27,29.27,29.27,29.27,30.06,30.06,
       30.06,30.73,30.73,31.31,31.31,31.82,32.28,32.28,
       32.69,33.07,33.42,33.74,34.04,34.32,34.58,35.06,
       35.29,35.7,35.89,36.26,36.59,39.9,40.19,40.46,
       42.72,43.07,43.4,48.81,49.09,49.52,59.91,60.34,
       60.79,65.89
     }
  },
/* added by T. Araki and H. Ohara (1998.09.03) end */

  { 44100, 70, 
     { /* w_low */
      0, 2, 4, 6, 8, 10, 12, 14, 16, 18, 20, 22, 24, 26, 28, 30, 32, 34, 36,
      39, 42, 45, 48, 51, 54, 57, 60, 63, 67, 71, 75, 79, 83, 88, 93, 98,
      104, 110, 117, 124, 132, 140, 149, 158, 168, 179, 191, 204, 218, 233,
      249, 266, 284, 304, 325, 348, 372, 398, 426, 456, 489, 525, 564, 607,
      654, 707, 766, 833, 909, 997
     },
     { /* w_high */
      1, 3, 5, 7, 9, 11, 13, 15, 17, 19, 21, 23, 25, 27, 29, 31, 33, 35, 38, 
      41, 44, 47, 50, 53, 56, 59, 62, 66, 70, 74, 78, 82, 87, 92, 97, 103,
      109, 116, 123, 131, 139, 148, 157, 167, 178, 190, 203, 217, 232, 248,
      265, 283, 303, 324, 347, 371, 397, 425, 455, 488, 524, 563, 606, 653,
      706, 765, 832, 908, 996, 1023
     },
     { /* width */
      2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 3, 3, 3, 3, 3, 
      3, 3, 3, 3, 4, 4, 4, 4, 4, 5, 5, 5, 6, 6, 7, 7, 8, 8, 9, 9, 10, 11,
      12, 13, 14, 15, 16, 17, 18, 20, 21, 23, 24, 26, 28, 30, 33, 36, 39,
      43, 47, 53, 59, 67, 76, 88, 27
     },
     { /* bval */
      0.22, 0.65, 1.09, 1.52, 1.95, 2.37, 2.79, 3.21, 3.62, 4.02, 4.41, 4.80,
      5.18, 5.55, 5.92, 6.27, 6.62, 6.95, 7.36, 7.83, 8.28, 8.71, 9.12, 9.52,
      9.89, 10.25, 10.59, 10.97, 11.38, 11.77, 12.13, 12.48, 12.84, 13.22,
      13.57, 13.93, 14.30, 14.67, 15.03, 15.40, 15.76, 16.11, 16.45, 16.79,
      17.13, 17.48, 17.83, 18.18, 18.52, 18.87, 19.21, 19.54, 19.88, 20.22,
      20.56, 20.90, 21.24, 21.57, 21.91, 22.24, 22.58, 22.91, 23.25, 23.58,
      23.91, 24.00, 24.00, 24.00, 24.00, 24.00
     },
     { /* qsthr */
      40.29, 40.29, 35.29, 35.29, 35.29, 32.29, 32.29, 27.29, 27.29, 25.29,
      25.29, 25.29, 25.29, 25.29, 25.29, 25.29, 25.29, 25.29, 27.05, 27.05,
      27.05, 27.05, 27.05, 27.05, 27.05, 27.05, 27.05, 28.30, 28.30, 28.30,
      28.30, 28.30, 29.27, 29.27, 29.27, 30.06, 30.06, 30.73, 30.73, 31.31,
      31.31, 31.82, 31.82, 32.28, 32.69, 33.07, 33.42, 33.74, 34.04, 34.32,
      34.58, 34.83, 35.29, 38.50, 38.89, 39.08, 41.43, 41.75, 42.05, 47.46,
      47.84, 48.19, 58.61, 59.00, 59.52, 69.98, 70.54, 71.08, 71.72, 72.09
     }
  },
  { 48000, 69,
     { /* w_low */
      0, 2, 4, 6, 8, 10, 12, 14, 16, 18, 20, 22, 24, 26, 28, 30, 32, 34, 36,
      38, 41, 44, 47, 50, 53, 56, 59, 62, 66, 70, 74, 78, 82, 87, 92, 97,
      103, 109, 116, 123, 131, 139, 148, 158, 168, 179, 191, 204, 218, 233,
      249, 266, 284, 304, 325, 348, 372, 398, 426, 457, 491, 528, 568, 613,
      663, 719, 782, 854, 938
     },
     { /* w_high */
      1, 3, 5, 7, 9, 11, 13, 15, 17, 19, 21, 23, 25, 27, 29, 31, 33, 35, 37,
      40, 43, 46, 49, 52, 55, 58, 61, 65, 69, 73, 77, 81, 86, 91, 96, 102,
      108, 115, 122, 130, 138, 147, 157, 167, 178, 190, 203, 217, 232, 248,
      265, 283, 303, 324, 347, 371, 397, 425, 456, 490, 527, 567, 612, 662,
      718, 781, 853, 937, 1023
     }, 
     { /* width */
      2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 3, 3, 3, 3, 3,
      3, 3, 3, 4, 4, 4, 4, 4, 5, 5, 5, 6, 6, 7, 7, 8, 8, 9, 10, 10, 11, 12,
      13, 14, 15, 16, 17, 18, 20, 21, 23, 24, 26, 28, 31, 34, 37, 40, 45, 50,
      56, 63, 72, 84, 86
     },
     { /* bval */
      0.24, 0.71, 1.18, 1.65, 2.12, 2.58, 3.03, 3.48, 3.92, 4.35, 4.77, 5.19,
      5.59, 5.99, 6.37, 6.74, 7.10, 7.45, 7.80, 8.20, 8.68, 9.13, 9.55, 9.96,
      10.35, 10.71, 11.06, 11.45, 11.86, 12.25, 12.62, 12.96, 13.32, 13.70,
      14.05, 14.41, 14.77, 15.13, 15.49, 15.85, 16.20, 16.55, 16.91, 17.25,
      17.59, 17.93, 18.28, 18.62, 18.96, 19.30, 19.64, 19.97, 20.31, 20.65,
      20.99, 21.33, 21.66, 21.99, 22.32, 22.66, 23.00, 23.33, 23.67, 24.00,
      24.00, 24.00, 24.00, 24.00, 24.00
     },
     { /* qsthr */
      40.29, 40.29, 35.29, 35.29, 32.29, 32.29, 27.29, 27.29, 27.29, 25.29,
      25.29, 25.29, 25.29, 25.29, 25.29, 25.29, 25.29, 25.29, 25.29, 27.05,
      27.05, 27.05, 27.05, 27.05, 27.05, 27.05, 27.05, 28.30, 28.30, 28.30,
      28.30, 28.30, 29.27, 29.27, 29.27, 30.06, 30.06, 30.73, 30.73, 31.31,
      31.31, 31.82, 32.28, 32.28, 32.69, 33.07, 33.42, 33.74, 34.04, 34.32,
      34.58, 34.83, 38.29, 38.50, 38.89, 41.08, 41.43, 41.75, 47.19, 47.59,
      47.96, 58.30, 58.81, 69.27, 69.76, 70.27, 70.85, 71.52, 70.20
     }
  },

/* added by T. Araki (1998.09.03) */
  { 64000, 67,
     { /* w_low */
      0,2,4,6,8,10,12,14,16,18,20,22,24,26,28,30,32,34,36,38,40,
      42,45,48,51,54,57,60,64,68,72,76,81,86,91,97,103,110,117,
      125,133,142,152,162,173,185,198,212,227,243,260,278,298,319,
      342,367,395,425,459,496,538,585,639,702,775,862,967
     },
     { /* w_high */
      1,3,5,7,9,11,13,15,17,19,21,23,25,27,29,31,33,35,37,39,41,
      44,47,50,53,56,59,63,67,71,75,80,85,90,96,102,109,116,124,
      132,141,151,161,172,184,197,211,226,242,259,277,297,318,341,
      366,394,424,458,495,537,584,638,701,774,861,966,1023
     }, 
     { /* width */
      2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,3,3,3,3,3,3,
      4,4,4,4,5,5,5,6,6,7,7,8,8,9,10,10,11,12,13,14,15,16,17,
      18,20,21,23,25,28,30,34,37,42,47,54,63,73,87,105,57
     },
     { /* bval */
      0.32,0.95,1.57,2.19,2.8,3.4,3.99,4.56,5.12,5.66,6.18,6.68,
      7.16,7.63,8.07,8.5,8.9,9.29,9.67,10.03,10.37,10.77,11.23,
      11.66,12.06,12.44,12.79,13.18,13.59,13.97,14.32,14.69,15.07,
      15.42,15.77,16.13,16.49,16.85,17.2,17.54,17.88,18.23,18.58,
      18.91,19.25,19.6,19.94,20.29,20.63,20.97,21.31,21.64,21.98,
      22.31,22.65,22.98,23.32,23.66,23.99,24,24,24,24,24,24,24,24
     },
     { /* qsthr */
      40.29,40.29,35.29,32.29,32.29,27.29,27.29,25.29,25.29,25.29,
      25.29,25.29,25.29,25.29,25.29,25.29,25.29,25.29,25.29,25.29,
      25.29,27.05,27.05,27.05,27.05,27.05,27.05,28.3,28.3,28.3,28.3,
      29.27,29.27,29.27,30.06,30.06,30.73,30.73,31.31,31.31,31.82,
      32.28,32.28,32.69,33.07,33.42,33.74,37.04,37.32,37.58,39.83,
      40.29,40.5,45.89,46.26,46.75,57.05,57.59,57.96,68.51,69,69.6,
      70.27,70.91,71.67,72.49,69.83
     }
  },
  { 88200, 72,
     { /* w_low */
      0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,20,22,24,26,28,
      30,32,34,36,38,40,43,46,49,52,55,59,63,67,71,76,81,86,92,98,
      105,112,120,128,137,147,157,168,180,193,207,222,238,256,275,
      296,319,345,374,406,443,485,534,592,661,746,852,989
     },
     { /* w_high */
      0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,19,21,23,25,27,29,
      31,33,35,37,39,42,45,48,51,54,58,62,66,70,75,80,85,91,97,104,
      111,119,127,136,146,156,167,179,192,206,221,237,255,274,295,
      318,344,373,405,442,484,533,591,660,745,851,988,1023
     }, 
     { /* width */
      1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,2,2,2,2,2,2,2,2,
      3,3,3,3,3,4,4,4,4,5,5,5,6,6,7,7,8,8,9,10,10,11,12,13,14,15,
      16,18,19,21,23,26,29,32,37,42,49,58,69,85,106,137,35
     },
     { /* bval */
      0,0.44,0.87,1.3,1.73,2.16,2.58,3,3.41,3.82,4.22,4.61,4.99,5.37,
      5.74,6.1,6.45,6.79,7.44,8.05,8.64,9.19,9.7,10.19,10.65,11.08,
      11.48,11.86,12.22,12.64,13.1,13.53,13.93,14.3,14.69,15.11,15.49,
      15.84,16.21,16.58,16.92,17.27,17.62,17.97,18.32,18.67,19.02,19.35,
      19.71,20.05,20.39,20.73,21.08,21.43,21.77,22.11,22.45,22.8,23.13,
      23.47,23.81,24,24,24,24,24,24,24,24,24,24,24
     },
     { /* qsthr */
      37.28,37.28,37.28,32.28,32.28,29.28,29.28,24.28,24.28,24.28,
      22.28,22.28,22.28,22.28,22.28,22.28,22.28,22.28,25.29,25.29,
      25.29,25.29,25.29,25.29,25.29,25.29,25.29,25.29,25.29,27.05,
      27.05,27.05,27.05,27.05,28.3,28.3,28.3,28.3,29.27,29.27,29.27,
      30.06,30.06,30.73,30.73,31.31,31.31,31.82,32.28,35.28,35.69,
      36.07,38.42,38.74,39.04,44.32,44.83,45.06,55.5,55.89,56.43,
      66.9,67.33,67.96,68.51,69.18,69.91,70.66,71.57,72.53,73.64,67.72
     }
  },
  { 96000, 71,
     { /* w_low */
      0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,21,23,25,27,
      29,31,33,35,37,39,41,44,47,50,53,56,60,64,68,73,78,83,89,95,
      102,109,117,125,134,144,154,165,177,190,204,219,235,253,272,
      293,317,343,373,407,446,491,544,608,686,784,911
     },
     { /* w_high */
      0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,20,22,24,26,28,
      30,32,34,36,38,40,43,46,49,52,55,59,63,67,72,77,82,88,94,101,
      108,116,124,133,143,153,164,176,189,203,218,234,252,271,292,
      316,342,372,406,445,490,543,607,685,783,910,1023
     }, 
     { /* width */
      1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,2,2,2,2,2,2,2,2,
      3,3,3,3,3,4,4,4,5,5,5,6,6,7,7,8,8,9,10,10,11,12,13,14,15,16,
      18,19,21,24,26,30,34,39,45,53,64,78,98,127,113
     },
     { /* bval */
      0,0.47,0.95,1.42,1.88,2.35,2.81,3.26,3.7,4.14,4.57,4.98,5.39,
      5.79,6.18,6.56,6.93,7.28,7.63,8.28,8.9,9.48,10.02,10.53,11,11.45,
      11.86,12.25,12.62,12.96,13.36,13.8,14.21,14.59,14.94,15.32,15.71,
      16.08,16.45,16.83,17.19,17.54,17.9,18.26,18.62,18.97,19.32,19.67,
      20.03,20.38,20.72,21.07,21.42,21.77,22.12,22.46,22.8,23.14,23.47,
      23.81,24,24,24,24,24,24,24,24,24,24,24
     },
     { /* qsthr */
      37.28,37.28,37.28,32.28,32.28,29.28,29.28,24.28,24.28,22.28,22.28,
      22.28,22.28,22.28,22.28,22.28,22.28,22.28,22.28,25.29,25.29,25.29,
      25.29,25.29,25.29,25.29,25.29,25.29,25.29,25.29,27.05,27.05,27.05,
      27.05,27.05,28.3,28.3,28.3,29.27,29.27,29.27,30.06,30.06,30.73,
      30.73,31.31,31.31,31.82,35.28,35.28,35.69,38.07,38.42,38.74,44.04,
      44.32,44.83,55.06,55.5,56.08,66.43,67.05,67.59,68.19,68.81,69.52,
      70.34,71.2,72.19,73.31,72.81
     }
  },
/* added by T. Araki (1998.09.03) end */
#ifdef I2R_LOSSLESS
{ 192000, 71,
     { /* w_low */
      0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,21,23,25,27,
      29,31,33,35,37,39,41,44,47,50,53,56,60,64,68,73,78,83,89,95,
      102,109,117,125,134,144,154,165,177,190,204,219,235,253,272,
      293,317,343,373,407,446,491,544,608,686,784,911
     },
     { /* w_high */
      0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,20,22,24,26,28,
      30,32,34,36,38,40,43,46,49,52,55,59,63,67,72,77,82,88,94,101,
      108,116,124,133,143,153,164,176,189,203,218,234,252,271,292,
      316,342,372,406,445,490,543,607,685,783,910,1023
     }, 
     { /* width */
      1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,2,2,2,2,2,2,2,2,
      3,3,3,3,3,4,4,4,5,5,5,6,6,7,7,8,8,9,10,10,11,12,13,14,15,16,
      18,19,21,24,26,30,34,39,45,53,64,78,98,127,113
     },
     { /* bval */
      0,0.47,0.95,1.42,1.88,2.35,2.81,3.26,3.7,4.14,4.57,4.98,5.39,
      5.79,6.18,6.56,6.93,7.28,7.63,8.28,8.9,9.48,10.02,10.53,11,11.45,
      11.86,12.25,12.62,12.96,13.36,13.8,14.21,14.59,14.94,15.32,15.71,
      16.08,16.45,16.83,17.19,17.54,17.9,18.26,18.62,18.97,19.32,19.67,
      20.03,20.38,20.72,21.07,21.42,21.77,22.12,22.46,22.8,23.14,23.47,
      23.81,24,24,24,24,24,24,24,24,24,24,24
     },
     { /* qsthr */
      37.28,37.28,37.28,32.28,32.28,29.28,29.28,24.28,24.28,22.28,22.28,
      22.28,22.28,22.28,22.28,22.28,22.28,22.28,22.28,25.29,25.29,25.29,
      25.29,25.29,25.29,25.29,25.29,25.29,25.29,25.29,27.05,27.05,27.05,
      27.05,27.05,28.3,28.3,28.3,29.27,29.27,29.27,30.06,30.06,30.73,
      30.73,31.31,31.31,31.82,35.28,35.28,35.69,38.07,38.42,38.74,44.04,
      44.32,44.83,55.06,55.5,56.08,66.43,67.05,67.59,68.19,68.81,69.52,
      70.34,71.2,72.19,73.31,72.81
     }
  },
#endif
  { -1 }
};

static PARTITION_TABLE_SHORT  part_tbl_short_all[MAX_SAMPLING_RATES+1] =
{

/* added by T. Araki and H. Ohara (1998.09.03) */
  { 8000, 40,
    { /* w_low */
     0, 2, 4, 6, 8, 10, 12, 14, 16, 18, 20, 22, 24, 26, 28, 30, 32, 34, 36,
     38, 40, 42, 45, 48, 51, 54, 57, 60, 64, 68, 72, 76, 81, 86, 91, 97, 103,
     110, 117, 125
    },
    { /* w_high */
     1, 3, 5, 7, 9, 11, 13, 15, 17, 19, 21, 23, 25, 27, 29, 31, 33, 35, 37,
     39, 41, 44, 47, 50, 53, 56, 59, 63, 67, 71, 75, 80, 85, 90, 96, 102,
     109, 116, 124, 127
    },
    { /* width */
     2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 3, 3, 3,
     3, 3, 3, 4, 4, 4, 4, 5, 5, 5, 6, 6, 7, 7, 8, 3
    },
    { /* bval */
     0.32, 0.95, 1.57, 2.19, 2.80, 3.40, 3.99, 4.56, 5.12, 5.66, 6.18, 6.68,
     7.16, 7.63, 8.07, 8.50, 8.90, 9.29, 9.67, 10.03, 10.37, 10.77, 11.23,
     11.66, 12.06, 12.44, 12.79, 13.18, 13.59, 13.97, 14.32, 14.69, 15.07,
     15.42, 15.77, 16.13, 16.49, 16.85, 17.20, 17.44
    },
    { /* qsthr */
     30.29, 30.29, 25.29, 22.29, 22.29, 17.29, 17.29, 15.29, 15.29, 15.29,
     15.29, 15.29, 15.29, 15.29, 15.29, 15.29, 15.29, 15.29, 15.29, 15.29,
     15.29, 17.05, 17.05, 17.05, 17.05, 17.05, 17.05, 18.30, 18.30, 18.30, 
     18.30, 19.27, 19.27, 19.27, 20.06, 20.06, 20.73, 20.73, 21.31, 17.05
    }
  },
  { 11025, 47,
     { /* w_low */
       0,1,2,3,4,5,6,7,8,9,
       10,11,12,13,14,15,16,17,18,20,
       22,24,26,28,30,32,34,36,38,40,
       43,46,49,52,55,59,63,67,71,76,
       81,86,92,98,105,112,120
     },
     { /* w_high */
       0,1,2,3,4,5,6,7,8,9,
       10,11,12,13,14,15,16,17,19,21,
       23,25,27,29,31,33,35,37,39,42,
       45,48,51,54,58,62,66,70,75,80,
       85,91,97,104,111,119,127
     },
     { /* width */
       1,1,1,1,1,1,1,1,1,1,
       1,1,1,1,1,1,1,1,2,2,
       2,2,2,2,2,2,2,2,2,3,
       3,3,3,3,4,4,4,4,5,5,
       5,6,6,7,7,8,8
     },
     { /* bval */
       0,0.44,0.87,1.3,1.73,2.16,2.58,3,
       3.41,3.82,4.22,4.61,4.99,5.37,5.74,6.1,
       6.45,6.79,7.44,8.05,8.64,9.19,9.7,10.19,
       10.65,11.08,11.48,11.86,12.22,12.64,13.1,13.53,
       13.93,14.3,14.69,15.11,15.49,15.84,16.21,16.58,
       16.92,17.27,17.62,17.97,18.32,18.67,19.02
     },
     { /* qsthr */
       27.28,27.28,27.28,22.28,22.28,19.28,19.28,14.28,
       14.28,14.28,12.28,12.28,12.28,12.28,12.28,12.28,
       12.28,12.28,15.29,15.29,15.29,15.29,15.29,15.29,
       15.29,15.29,15.29,15.29,15.29,17.05,17.05,17.05,
       17.05,17.05,18.3,18.3,18.3,18.3,19.27,19.27,
       19.27,20.06,20.06,20.73,20.73,21.31,21.31
     }
  },
  { 12000, 48,
     { /* w_low */
       0,1,2,3,4,5,6,7,
       8,9,10,11,12,13,14,15,
       16,17,18,19,21,23,25,27,
       29,31,33,35,37,39,41,44,
       47,50,53,56,60,64,68,73,
       78,83,89,95,102,109,117,125
     },
     { /* w_high */
       0,1,2,3,4,5,6,7,
       8,9,10,11,12,13,14,15,
       16,17,18,20,22,24,26,28,
       30,32,34,36,38,40,43,46,
       49,52,55,59,63,67,72,77,
       82,88,94,101,108,116,124,127
     },
     { /* width */
       1,1,1,1,1,1,1,1,1,1,1,1,
       1,1,1,1,1,1,1,2,2,2,2,2,
       2,2,2,2,2,2,3,3,3,3,3,4,
       4,4,5,5,5,6,6,7,7,8,8,3
     },
     { /* bval */
       0,0.47,0.95,1.42,1.88,2.35,2.81,3.26,
       3.7,4.14,4.57,4.98,5.39,5.79,6.18,6.56,
       6.93,7.28,7.63,8.28,8.9,9.48,10.02,10.53,
       11,11.45,11.86,12.25,12.62,12.96,13.36,13.8,
       14.21,14.59,14.94,15.32,15.71,16.08,16.45,16.83,
       17.19,17.54,17.9,18.26,18.62,18.97,19.32,19.55
     },
     { /* qsthr */
       27.28,27.28,27.28,22.28,22.28,19.28,
       19.28,14.28,14.28,12.28,12.28,12.28,
       12.28,12.28,12.28,12.28,12.28,12.28,
       12.28,15.29,15.29,15.29,15.29,15.29,
       15.29,15.29,15.29,15.29,15.29,15.29,
       17.05,17.05,17.05,17.05,17.05,18.3,
       18.3,18.3,19.27,19.27,19.27,20.06,
       20.06,20.73,20.73,21.31,21.31,17.05
     }
  },
  { 16000, 47,
     { /* w_low */
       0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,
       16,17,18,19,20,21,23,25,27,29,31,33,35,
       37,40,43,46,49,52,56,60,64,69,74,79,85,
       91,98,105,113,121
     },
     { /* w_high */
       0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,
       16,17,18,19,20,22,24,26,28,30,32,34,36,
       39,42,45,48,51,55,59,63,68,73,78,84,90,
       97,104,112,120,127
     },
     { /* width */
       1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
       2,2,2,2,2,2,2,2,3,3,3,3,3,4,4,4,5,5,5,6,6,7,7,8,8,7
     },
     { /* bval */
       0,0.63,1.26,1.88,2.5,3.11,3.7,4.28,4.85,5.39,
       5.92,6.43,6.93,7.4,7.85,8.29,8.7,9.1,9.49,9.85,
       10.2,10.85,11.44,11.99,12.5,12.96,13.39,13.78,
       14.15,14.57,15.03,15.45,15.84,16.19,16.57,16.97,
       17.33,17.71,18.09,18.44,18.8,19.17,19.53,19.89,
       20.25,20.61,20.92
     },
     { /* qsthr */
       27.28,27.28,22.28,22.28,19.28,14.28,14.28,12.28,
       12.28,12.28,12.28,12.28,12.28,12.28,12.28,12.28,
       12.28,12.28,12.28,12.28,12.28,15.29,15.29,15.29,
       15.29,15.29,15.29,15.29,15.29,17.05,17.05,17.05,
       17.05,17.05,18.3,18.3,18.3,19.27,19.27,19.27,20.06,
       20.06,20.73,20.73,24.31,24.31,23.73
     }
  },
  { 22050, 46,
     { /* w_low */
      0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19,
      20, 22, 24, 26, 28, 30, 32, 34, 36, 39, 42, 45, 48, 52, 56, 60, 64, 69,
      74, 79, 85, 91, 98, 105, 113, 121
     },
     { /* w_high */
      0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19,
      21, 23, 25, 27, 29, 31, 33, 35, 38, 41, 44, 47, 51, 55, 59, 63, 68, 73,
      78, 84, 90, 97, 104, 112, 120, 127
     },
     { /* width */
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2,
      2, 2, 2, 2, 3, 3, 3, 3, 4, 4, 4, 4, 5, 5, 5, 6, 6, 7, 7, 8, 8, 7
     },
     { /* bval */
      0.00, 0.87, 1.73, 2.58, 3.41, 4.22, 4.99, 5.74, 6.45, 7.12, 7.75, 8.36,
      8.92, 9.45, 9.96, 10.43, 10.87, 11.29, 11.68, 12.05, 12.71, 13.32,
      13.86, 14.35, 14.80, 15.21, 15.58, 15.93, 16.32, 16.75, 17.15, 17.51,
      17.89, 18.30, 18.67, 19.02, 19.37, 19.74, 20.09, 20.44, 20.79, 21.15,
      21.50, 21.85, 22.20, 22.49
     },
     { /* qsthr */
      27.28, 27.28, 22.28, 19.28, 14.28, 12.28, 12.28, 12.28, 12.28, 12.28,
      12.28, 12.28, 12.28, 12.28, 12.28, 12.28, 12.28, 12.28, 12.28, 12.28,
      15.29, 15.29, 15.29, 15.29, 15.29, 15.29, 15.29, 15.29, 17.05, 17.05,
      17.05, 17.05, 18.30, 18.30, 18.30, 18.30, 19.27, 19.27, 22.27, 23.06,
      23.06, 25.73, 25.73, 26.31, 31.31, 30.73
     }
  },
  { 24000, 46,
     { /* w_low */
       0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,
       16,17,18,19,20,22,24,26,28,30,32,34,37,
       40,43,46,49,53,57,61,66,71,76,82,88,95,
       102,110,118,127
     },
     { /* w_high */
       0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,
       16,17,18,19,21,23,25,27,29,31,33,36,39,
       42,45,48,52,56,60,65,70,75,81,87,94,101,
       109,117,126,127
     },
     { /* width */
       1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
       2,2,2,2,2,2,2,3,3,3,3,3,4,4,4,5,5,5,6,6,7,7,8,8,9,1
     },
     { /* bval */
       0,0.95,1.88,2.81,3.7,4.57,5.39,6.18,6.93,
       7.63,8.29,8.91,9.49,10.03,10.53,11.01,11.45,
       11.87,12.26,12.62,13.28,13.87,14.4,14.88,
       15.32,15.71,16.08,16.49,16.94,17.35,17.73,
       18.07,18.44,18.83,19.2,19.57,19.96,20.31,
       20.67,21.04,21.41,21.77,22.13,22.48,22.82,23.01
     },
     { /* qsthr */
       27.28,27.28,22.28,19.28,14.28,12.28,12.28,
       12.28,12.28,12.28,12.28,12.28,12.28,12.28,
       12.28,12.28,12.28,12.28,12.28,12.28,15.29,
       15.29,15.29,15.29,15.29,15.29,15.29,17.05,
       17.05,17.05,17.05,17.05,18.3,18.3,18.3,19.27,
       19.27,22.27,23.06,25.06,25.73,25.73,31.31,
       31.31,31.82,32.28
     }
  },
  { 32000, 44,
     { /* w_low */
       0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,
       16,17,18,19,21,23,25,27,29,31,33,36,39,
       42,45,49,53,57,61,66,71,76,82,88,95,103,111,120
     },
     { /* w_high */
       0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,
       16,17,18,20,22,24,26,28,30,32,35,38,41,
       44,48,52,56,60,65,70,75,81,87,94,102,110,119,127
     },
     { /* width */
       1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
       2,2,2,2,2,2,2,3,3,3,3,4,4,4,4,5,5,5,6,6,7,8,8,9,8
     },
     { /* bval */
       0,1.26,2.5,3.7,4.85,5.92,6.93,7.85,8.7,
       9.49,10.2,10.85,11.45,12,12.5,12.96,13.39,
       13.78,14.15,14.8,15.38,15.89,16.36,16.77,
       17.15,17.5,17.9,18.34,18.74,19.11,19.5,
       19.92,20.3,20.65,21.02,21.4,21.75,22.1,
       22.45,22.8,23.16,23.51,23.85,24
     },
     { /* qsthr */
       27.28,22.28,19.28,14.28,12.28,12.28,12.28,
       12.28,12.28,12.28,12.28,12.28,12.28,12.28,
       12.28,12.28,12.28,12.28,12.28,15.29,15.29,
       15.29,15.29,15.29,15.29,15.29,17.05,17.05,
       17.05,17.05,18.3,18.3,21.3,21.3,24.27,24.27,
       24.27,30.06,30.06,30.73,41.31,41.31,41.82,60.47
     }
  },
/* added by T. Araki and H. Ohara (1998.09.03) end */

  { 44100, 42, 
     { /* w_low */
      0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 20,
      22, 24, 26, 28, 30, 32, 35, 38, 41, 44, 48, 52, 56, 60, 65, 70, 76,
      82, 89, 97, 106, 116
     },
     { /* w_high */
      0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 19, 21,
      23, 25, 27, 29, 31, 34, 37, 40, 43, 47, 51, 55, 59, 64, 69, 75, 81,
      88, 96, 105, 115, 127
     },
     { /* width */
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2,
      2, 2, 3, 3, 3, 3, 4, 4, 4, 4, 5, 5, 6, 6, 7, 8, 9, 10, 12
     },
     { /* bval */
      0.00, 1.73, 3.41, 4.99, 6.45, 7.75, 8.92, 9.96, 10.87, 11.68, 12.39,
      13.03, 13.61, 14.12, 14.59, 15.01, 15.40, 15.76, 16.39, 16.95, 17.45,
      17.89, 18.30, 18.67, 19.02, 19.41, 19.85, 20.25, 20.62, 21.01, 21.43,
      21.81, 22.15, 22.51, 22.87, 23.23, 23.59, 23.93, 24.00, 24.00, 24.00,
      24.00
     },
     { /* qsthr */
      27.28, 22.28, 14.28, 12.28, 12.28, 12.28, 12.28, 12.28, 12.28, 12.28,
      12.28, 12.28, 12.28, 12.28, 12.28, 12.28, 12.28, 12.28, 15.29, 15.29,
      15.29, 15.29, 15.29, 15.29, 15.29, 17.05, 17.05, 20.05, 20.05, 23.30,
      23.30, 23.30, 28.30, 29.27, 29.27, 40.06, 40.06, 40.73, 51.31, 51.82,
      52.28, 53.07
     }
  },
  { 48000, 42,
     { /* w_low */
      0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 19, 21,
      23, 25, 27, 29, 31, 34, 37, 40, 43, 46, 50, 54, 58, 63, 68, 74, 80,
      87, 95, 104, 114, 126
     },
     { /* w_high */
      0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 18, 20, 22,
      24, 26, 28, 30, 33, 36, 39, 42, 45, 49, 53, 57, 62, 67, 73, 79, 86,
      94, 103, 113, 125, 127
     }, 
     { /* width */
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2,
      2, 3, 3, 3, 3, 3, 4, 4, 4, 5, 5, 6, 6, 7, 8, 9, 10, 12, 1
     },
     { /* bval */
      0.00, 1.88, 3.70, 5.39, 6.93, 8.29, 9.49, 10.53, 11.45, 12.26, 12.96,
      13.59, 14.15, 14.65, 15.11, 15.52, 15.90, 16.56, 17.15, 17.66, 18.13,
      18.54, 18.93, 19.28, 19.69, 20.14, 20.54, 20.92, 21.27, 21.64, 22.03,
      22.39, 22.76, 23.13, 23.49, 23.85, 24.00, 24.00, 24.00, 24.00, 24.00,
      24.00
     },
     { /* qsthr */
      27.28, 22.28, 14.28, 12.28, 12.28, 12.28, 12.28, 12.28, 12.28, 12.28,
      12.28, 12.28, 12.28, 12.28, 12.28, 12.28, 12.28, 15.29, 15.29, 15.29,
      15.29, 15.29, 15.29, 15.29, 17.05, 20.05, 20.05, 20.05, 22.05, 23.30,
      28.30, 28.30, 29.27, 39.27, 40.06, 40.06, 50.37, 51.31, 51.82, 52.28,
      53.07, 53.07
     }
  },

/* added by T. Araki (1998.09.03) */
  { 64000, 39,
     { /* w_low */
      0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,18,20,22,24,26,28,
      30,33,36,39,42,46,50,54,59,64,70,77,85,94,105,118
     },
     { /* w_high */
      0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,17,19,21,23,25,27,29,
      32,35,38,41,45,49,53,58,63,69,76,84,93,104,117,127
     }, 
     { /* width */
      1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,2,2,2,2,3,3,3,3,4,4,4,
      5,5,6,7,8,9,11,13,10
     },
     { /* bval */
      0,2.5,4.85,6.93,8.7,10.2,11.45,12.5,13.39,14.15,14.81,15.39,
      15.9,16.36,16.78,17.16,17.82,18.4,18.92,19.39,19.82,20.21,
      20.57,20.98,21.43,21.84,22.22,22.61,23.02,23.39,23.75,24,24,
      24,24,24,24,24,24
     },
     { /* qsthr */
      27.28,19.28,12.28,12.28,12.28,12.28,12.28,12.28,12.28,12.28,
      12.28,12.28,12.28,12.28,12.28,12.28,15.29,15.29,15.29,15.29,
      15.29,18.29,18.29,20.05,22.05,22.05,27.05,28.3,38.3,38.3,
      39.27,49.27,50.06,50.73,51.31,51.82,52.69,53.42,52.28
     }
  },
  { 88200, 37,
     { /* w_low */
      0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,18,20,22,24,26,28,
      30,33,36,39,43,47,52,57,63,70,78,88,100,116
     },
     { /* w_high */
      0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,17,19,21,23,25,27,29,
      32,35,38,42,46,51,56,62,69,77,87,99,115,127
     }, 
     { /* width */
      1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,2,2,2,2,3,3,3,4,4,
      5,5,6,7,8,10,12,16,1
     },
     { /* bval */
      0,3.41,6.45,8.92,10.87,12.39,13.61,14.59,15.4,16.09,16.69,
      17.21,17.68,18.11,18.49,18.85,19.48,20.05,20.55,21.01,21.43,
      21.81,22.15,22.55,22.98,23.36,23.75,24,24,24,24,24,24,24,24,24,24
     },
     { /* qsthr */
      27.28,14.28,12.28,12.28,12.28,12.28,12.28,12.28,12.28,12.28,
      12.28,12.28,12.28,12.28,12.28,12.28,15.29,18.29,18.29,20.29,
      20.29,20.29,25.29,27.05,27.05,37.05,38.3,48.3,49.27,49.27,
      50.06,50.73,51.31,52.28,53.07,54.32,53.07
     }
  },
  { 96000, 36,
     { /* w_low */
      0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,17,19,21,23,25,27,29,
      32,35,38,42,46,51,56,62,69,78,89,103,121
     },
     { /* w_high */
      0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,16,18,20,22,24,26,28,31,
      34,37,41,45,50,55,61,68,77,88,102,120,127
     }, 
     { /* width */
      1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,2,2,2,2,3,3,3,4,4,5,5,
      6,7,9,11,14,18,7
     },
     { /* bval */
      0,3.7,6.93,9.49,11.45,12.96,14.15,15.11,15.9,16.57,17.16,17.67,
      18.13,18.55,18.93,19.6,20.2,20.73,21.21,21.64,22.03,22.39,22.79,
      23.23,23.62,24,24,24,24,24,24,24,24,24,24,24
     },
     { /* qsthr */
      27.28,14.28,12.28,12.28,12.28,12.28,12.28,12.28,12.28,12.28,12.28,
      12.28,12.28,12.28,12.28,15.29,18.29,18.29,20.29,20.29,25.29,25.29,
      27.05,37.05,37.05,48.3,48.3,49.27,49.27,50.06,50.73,51.82,52.69,
      53.74,54.83,50.73
     }
   },
/* added by T. Araki (1998.09.03) end */
#ifdef I2R_LOSSLESS
  { 192000, 36,
     { /* w_low */
      0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,17,19,21,23,25,27,29,
      32,35,38,42,46,51,56,62,69,78,89,103,121
     },
     { /* w_high */
      0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,16,18,20,22,24,26,28,31,
      34,37,41,45,50,55,61,68,77,88,102,120,127
     }, 
     { /* width */
      1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,2,2,2,2,3,3,3,4,4,5,5,
      6,7,9,11,14,18,7
     },
     { /* bval */
      0,3.7,6.93,9.49,11.45,12.96,14.15,15.11,15.9,16.57,17.16,17.67,
      18.13,18.55,18.93,19.6,20.2,20.73,21.21,21.64,22.03,22.39,22.79,
      23.23,23.62,24,24,24,24,24,24,24,24,24,24,24
     },
     { /* qsthr */
      27.28,14.28,12.28,12.28,12.28,12.28,12.28,12.28,12.28,12.28,12.28,
      12.28,12.28,12.28,12.28,15.29,18.29,18.29,20.29,20.29,25.29,25.29,
      27.05,37.05,37.05,48.3,48.3,49.27,49.27,50.06,50.73,51.82,52.69,
      53.74,54.83,50.73
     }
   },
#endif
  { -1 }
};
/* added by T. Araki (1997.07.10) end */

/* added by T. Araki (1997.10.16) */
static double          sample[MAX_TIME_CHANNELS][BLOCK_LEN_LONG*2];
                               /* sample value */

static FFT_TABLE_LONG    fft_tbl_long;  /* table for long fft */
static FFT_TABLE_SHORT    fft_tbl_short;  /* table for short fft */
static PARTITION_TABLE_LONG    *part_tbl_long;  
                               /* partition table for long block */
static PARTITION_TABLE_SHORT    *part_tbl_short;
                               /* partition table for short block */
static PSY_STATVARIABLE_LONG    psy_stvar_long;
                               /* static variables for long block */
static PSY_STATVARIABLE_SHORT    psy_stvar_short;
                               /* static variables for short block */
/* added by T. Araki (1997.10.16) end */

void EncTf_psycho_acoustic_init( void )
{
  int chanNum;
/* added by T. Araki (1997.10.16) */
      psy_fft_table_init(&fft_tbl_long, &fft_tbl_short);
         /* initializing fft table */
      psy_calc_init(sample, &psy_stvar_long, &psy_stvar_short);
	/* initializing static variables */
/* added by T. Araki (1997.10.16) end */

}

/* added by T. Okada (1997.07.10) */
void psy_fft_table_init(FFT_TABLE_LONG *fft_tbl_long, 
			FFT_TABLE_SHORT *fft_tbl_short
			)
{    

    int i,j,k,n,n2,n4,n8;
    double c,s,dc,ds,t;

    /* generating Hann window */
    for(i = 0; i < BLOCK_LEN_LONG*2; ++i)
	fft_tbl_long->hw[i] = (0.5 - 0.5 * cos( M_PI * (double)(i + 0.5)/(double)(BLOCK_LEN_LONG) ) );
    for(i = 0; i < BLOCK_LEN_SHORT*2; ++i)
	fft_tbl_short->hw[i] = (0.5 - 0.5 * cos( M_PI * (double)(i + 0.5)/(double)(BLOCK_LEN_SHORT) ) );

    /* generating sin table (long) */
    n = BLOCK_LEN_LONG * 2;
    n2 = n/2;  
    n4 = n2/2;  
    n8 = n4/2;

    t = sin(M_PI / (double)n);
    dc = 2.0*t*t;
    ds = sqrt(dc * (2.0 - dc));
    t = 2*dc;
    c = fft_tbl_long->st[n4] = 1.0;
    s = fft_tbl_long->st[0] = 0;

    for(i = 1; i < n8; ++i){
	c -= dc;  dc += t * c;
	s += ds;  ds -= t * s;
	fft_tbl_long->st[i] = s;  fft_tbl_long->st[n4 - i] = c;
    }
    if (n8 != 0) fft_tbl_long->st[n8] = sqrt(0.5);
    for (i = 0; i < n4; i++)
	fft_tbl_long->st[n2 - i] = fft_tbl_long->st[i];
    for (i = 0; i < n2 + n4; i++)
	fft_tbl_long->st[i + n2] = -fft_tbl_long->st[i];

    /* generating sin table (short) */
    n = BLOCK_LEN_SHORT * 2;
    n2 = n/2;  
    n4 = n2/2;  
    n8 = n4/2;

    t = sin(M_PI / (double)n);
    dc = 2*t*t;
    ds = sqrt(dc * (2.0 - dc));
    t = 2*dc;
    c = fft_tbl_short->st[n4] = 1.0;
    s = fft_tbl_short->st[0] = 0;

    for(i = 1; i < n8; ++i){
	c -= dc;  dc += t * c;
	s += ds;  ds -= t * s;
	fft_tbl_short->st[i] = s;  fft_tbl_short->st[n4 - i] = c;
    }
    if (n8 != 0) fft_tbl_short->st[n8] = sqrt(0.5);
    for (i = 0; i < n4; i++)
	fft_tbl_short->st[n2 - i] = fft_tbl_short->st[i];
    for (i = 0; i < n2 + n4; i++)
	fft_tbl_short->st[i + n2] = - fft_tbl_short->st[i];

    /* generating bit inverse table (long) */
    n = BLOCK_LEN_LONG * 2;
    n2 = n/2; i = j = 0;

    for(;;){
	fft_tbl_long->brt[i] = j;
	if( ++i >= n ) break;
	k = n2;
	while(k <= j){
	    j -= k;
	    k /= 2;
	}
	j += k;
    }

    /* generating bit inverse table (short) */
    n = BLOCK_LEN_SHORT * 2;
    n2 = n/2; i = j = 0;

    for(;;){
	fft_tbl_short->brt[i] = j;
	if( ++i >= n ) break;
	k = n2;
	while(k <= j){
	    j -= k;
	    k /= 2;
	}
	j += k;
    }
}
/* added by T. Okada (1997.07.10) end */

/* added by T. Araki (1997.07.10) */
void psy_part_table_init( 
			   double sampling_rate,
			   PARTITION_TABLE_LONG **part_tbl_long,
			   PARTITION_TABLE_SHORT **part_tbl_short
			   )
{ 
    int b,bb; /* Jul 10 */
    double tmp;

   *part_tbl_long = &part_tbl_long_all[0];

  /* find correct sampling rate depending parameters */
  while( (*part_tbl_long)->sampling_rate != (int)sampling_rate ) {
    if( (*part_tbl_long)->sampling_rate == -1 ) {
      CommonExit( 1, "Encode: unknown sampling rate in psychoacoustic module %f", sampling_rate );
    }
    (*part_tbl_long)++;
  }

   *part_tbl_short = &part_tbl_short_all[0];

  /* find correct sampling rate depending parameters */
  while( (*part_tbl_short)->sampling_rate != (int)sampling_rate ) {
    if( (*part_tbl_short)->sampling_rate == -1 ) {
      CommonExit( 1, "Encode: unknown sampling rate in psychoacoustic module %f", sampling_rate );
    }
    (*part_tbl_short)++;
  }

    /* added by T. Okada (1997.07.10) */
    for( b = 0; b < (*part_tbl_long)->len; ++b){
	tmp = 0.0;
	for( bb = 0; bb < (*part_tbl_long)->len; ++bb)
	    tmp += sprdngf( (*part_tbl_long)->bval[bb],(*part_tbl_long)->bval[b]);
	(*part_tbl_long)->rnorm[b] = 1.0/tmp;
    }
    /* added by T. Okada (1997.07.10) end */

    /* added by T. Araki (1997.10.16) */
    for( b = 0; b < (*part_tbl_short)->len; ++b){
	tmp = 0.0;
	for( bb = 0; bb < (*part_tbl_short)->len; ++bb)
	    tmp += sprdngf( (*part_tbl_short)->bval[bb],(*part_tbl_short)->bval[b]);
	(*part_tbl_short)->rnorm[b] = 1.0/tmp;
    }
    /* added by T. Araki (1997.10.16) end */
}


void psy_calc_init( 
		   double sample[][BLOCK_LEN_LONG*2],
		   PSY_STATVARIABLE_LONG *psy_stvar_long,
		   PSY_STATVARIABLE_SHORT *psy_stvar_short
		   )
{ 
  int ch;
  int chanNum;
  int w;
  int i;

/* modified by T. Araki (1998.09.03) */
  for(ch = 0; ch < MAX_TIME_CHANNELS; ++ch){
    for(i = 0; i < BLOCK_LEN_LONG*2; ++i){
      sample[ch][i] = 0.0;
    }
  }

  for(ch = 0; ch < MAX_TIME_CHANNELS; ++ch){
    for(i = 0; i < BLOCK_LEN_LONG*3; ++i){
      psy_stvar_long->fft_r[ch][i] = 0.0;
      psy_stvar_long->fft_f[ch][i] = 0.0;
    }
  }

  for(ch = 0; ch < MAX_TIME_CHANNELS; ++ch){
    for(i = 0; i < NPART_LONG*2; ++i){
      psy_stvar_long->nb[ch][i] = 90.0;
    }
  }

  for(ch = 0; ch < MAX_TIME_CHANNELS; ++ch){
    psy_stvar_long->p_fft[ch] = 0;
    psy_stvar_long->p_nb[ch] = 0;
  }

  for(ch = 0; ch < MAX_TIME_CHANNELS; ++ch){
    for(i = 0; i < BLOCK_LEN_SHORT; ++i){
      psy_stvar_short->last6_fft_r[ch][i] = 0.0;
      psy_stvar_short->last6_fft_f[ch][i] = 0.0;
      psy_stvar_short->last7_fft_r[ch][i] = 0.0;
      psy_stvar_short->last7_fft_f[ch][i] = 0.0;
    }
  }

  for(ch = 0; ch < MAX_TIME_CHANNELS; ++ch){
    for(i = 0; i < NPART_SHORT; ++i){
      psy_stvar_short->last7_nb[ch][i] = 90.0;
    }
  }
/* modified by T. Araki (1998.09.03) end */
}

/* main */
void EncTf_psycho_acoustic( 
			   /* input */
			   double sampling_rate,
			   int    ch,         /* no of audio channels */
			   double *p_time_signal[],
			   enum WINDOW_TYPE block_type[],
			   int qcSelect,
			   int frameLength,
			   /* output */
			   CH_PSYCH_OUTPUT p_chpo_long[],
			   CH_PSYCH_OUTPUT p_chpo_short[][MAX_SHORT_WINDOWS]
			   )
{ 

    SR_INFO         *p_sri;

/* added by T. Araki (1997.07.10) */

    static int   flag = 0;
    int i;

    PSY_VARIABLE_LONG    psy_var_long;  /* variables for long block */
    PSY_VARIABLE_SHORT    psy_var_short;  /* variables for short block */
/* added by T. Araki (1997.07.10) end */

    if ((qcSelect==AAC_QC)||(qcSelect==AAC_PRED)) { /* AAC */
	switch (frameLength) {
	case 960:
	    p_sri = &sr_info_960[0];
	    break;
	case 1024:
	    p_sri = &sr_info_aac[0];
	    break;
	default:
	    CommonExit(-1,"\n illegal frameLength in EncTf_psycho_acoustic %d,frameLength");
	}
    }   
   
    else{
	p_sri = &sr_info_NOT_aac[0];
    }

  /* find correct sampling rate depending parameters */
  while( p_sri->sampling_rate != (long)sampling_rate ) {
    if( p_sri->sampling_rate == -1 ) {
      CommonExit( 1, "Encode: unknown sampling rate in psychoacoustic module %f", sampling_rate );
    }
    p_sri++;
  }

/* modified by T. Araki (1998.09.03) */
  if( flag==0 ) {
      psy_part_table_init(sampling_rate, &part_tbl_long, &part_tbl_short);
         /* initializing Table B 2.1.*.a, B 2.1.*.b in ISO/IEC 13818-7 */
      flag = 1;
  }

  psy_step1(p_time_signal, sample, ch);
         /* input sample value */
  psy_step2(sample, &psy_stvar_long, &psy_stvar_short, &fft_tbl_long, &fft_tbl_short, ch);
         /* FFT */
  psy_step3(&psy_stvar_long, &psy_stvar_short, &psy_var_long, &psy_var_short, ch);
         /* caluculation of predicted value */
  psy_step4(&psy_stvar_long, &psy_stvar_short, &psy_var_long, &psy_var_short, ch);
         /* caluculation of unpredictability mesure */
  psy_step5(part_tbl_long, part_tbl_short, &psy_stvar_long, &psy_stvar_short, &psy_var_long, &psy_var_short, ch);
         /* caluculation of energy and unpredictability in each partition */
  psy_step6(part_tbl_long, part_tbl_short, &psy_var_long, &psy_var_short);
         /* convolution of energy and unpredictability */ 
  psy_step7(part_tbl_long, part_tbl_short, &psy_var_long, &psy_var_short);
         /* caluculation of tonality in each partition */
  psy_step8(part_tbl_long, part_tbl_short, &psy_var_long, &psy_var_short);
         /* caluculation of SNR in each partition */
  psy_step9(part_tbl_long, part_tbl_short, &psy_var_long, &psy_var_short);
         /* caluculation of power ratio in each partition */
  psy_step10(part_tbl_long, part_tbl_short, &psy_stvar_long, &psy_stvar_short, &psy_var_long, &psy_var_short, ch);
         /* caluculation of actual energy threshold in each partition */
  psy_step11(part_tbl_long, part_tbl_short, &psy_stvar_long, &psy_stvar_short, ch);
         /* pre-echo control and threshold in quiet */
  psy_step12(part_tbl_long, part_tbl_short, &psy_stvar_long, &psy_stvar_short, &psy_var_long, &psy_var_short, ch);
         /* caluculation of PE */
  psy_step13(&psy_var_long, block_type, ch);
         /* decision of block type */
  psy_step14(p_sri, part_tbl_long, part_tbl_short, &psy_stvar_long, &psy_stvar_short, &psy_var_long, &psy_var_short, ch);
         /* caluculation of 1/SMR in each swb */

  p_chpo_long->p_ratio   = psy_stvar_long.ismr[ch];
  p_chpo_long->cb_width  = p_sri->cb_width_long;
  p_chpo_long->no_of_cb  = p_sri->num_cb_long;

  for( i=0; i<MAX_SHORT_WINDOWS; i++ ) {
    (*p_chpo_short)[i].p_ratio  = psy_stvar_short.ismr[ch][i];
    (*p_chpo_short)[i].cb_width = p_sri->cb_width_short;
    (*p_chpo_short)[i].no_of_cb = p_sri->num_cb_short;
  }
/* modified by T. Araki (1998.09.03) end */

}

/* added by I. Kaneko (1997.7.10) */
/* Itaru Kaneko Absolute threshold generator module */
double psy_get_absthr(double f)
{
	double db;	/* absolute threshold in [db]	*/
	double f0; 	/* freq for min thr		*/
	double db10k;	/* thr @ 10k Hz			*/
	double db100;	/* thr @ 100 Hz			*/
	double dbmax;	/* max absthr			*/
	double pow_ord;	/* power order			*/
	double dbdep;	/* DB depth			*/


	f0 = 2000.0;	/* freq for min thr		*/
	db10k = 20.0;	/* thr @ 10k Hz			*/
	db100 = 30.0;	/* thr @ 100 Hz			*/
	dbmax = 50.0;	/* max absthr			*/
	pow_ord= 3.0;	/* power order			*/
	dbdep = -5.0;	/* DB depth			*/

	if ( f <= 10.0 ) {	/* DC cutoff at 10Hz	*/
		db = 200.0;	/* virtual infinity	*/
	}
	else {
		db = psy_sqr(log(f0/f+1.0)) / psy_sqr(log(f0/100.0+1.0)) * 50.0
		   + pow((f - f0)/(10000.0-f0),pow_ord) * db10k - dbdep;
	}

	if (db <= dbmax)
		return(db);
	else		/* anything is audible at 50dB	*/
		return(dbmax);

}
/* added by I. Kaneko (1997.7.10) end */

/* added by T. Okada (1997.7.10) */
double sprdngf(double b1, double b2)
{
    double tmpx,tmpy,tmpz;

    tmpx = (b2 >= b1) ? 3.0*(b2-b1) : 1.5*(b2-b1);
    
    tmpz = 8.0 * psy_min( (tmpx-0.5)*(tmpx-0.5) - 2.0*(tmpx-0.5),0.0 );

    tmpy = 15.811389 + 7.5*(tmpx + 0.474)-17.5 *sqrt(1.0 + (tmpx+0.474)*(tmpx+0.474));

    return( tmpy < -100.0 ? 0.0 : pow(10.0, (tmpz + tmpy)/10.0) );
}

void psy_step1(double* p_time_signal[],
	       double sample[][BLOCK_LEN_LONG*2], 
	       int ch) 
{
    int i;
    for(i = 0; i < BLOCK_LEN_LONG; ++i){
	sample[ch][i] = sample[ch][i+BLOCK_LEN_LONG];
	sample[ch][i+BLOCK_LEN_LONG] = (*p_time_signal)[i]/32767.0;
    }

}

void psy_step2(double sample[][BLOCK_LEN_LONG*2], 
               PSY_STATVARIABLE_LONG *psy_stvar_long, 
               PSY_STATVARIABLE_SHORT *psy_stvar_short,
	       FFT_TABLE_LONG *fft_tbl_long,
	       FFT_TABLE_SHORT *fft_tbl_short,
	       int ch
	       )
{
    int w,i,j,k,l,h,n,d,ik,k2,n4;
    double t,s,c,dx,dy;
    double *xl,*yl;

    /* FFT for long */
    xl = (double *)malloc( sizeof(double) * BLOCK_LEN_LONG * 2 );
    yl = (double *)malloc( sizeof(double) * BLOCK_LEN_LONG * 2 );

    psy_stvar_long->p_fft[ch] += BLOCK_LEN_LONG;

    if(psy_stvar_long->p_fft[ch] == BLOCK_LEN_LONG * 3)
	psy_stvar_long->p_fft[ch] = 0;

    /* window *//* static int it = 0; */
    for(i = 0; i < BLOCK_LEN_LONG*2; ++i){
	xl[i] = fft_tbl_long->hw[i] * sample[ch][i];
	yl[i] = 0.0;
    }

    n = BLOCK_LEN_LONG*2;
    n4 = n/4;

    for (i = 0; i < n; ++i) {    /* bit inverse */
	j = fft_tbl_long->brt[i];
	if (i < j) {
	    t = xl[i];  xl[i] = xl[j];  xl[j] = t;
	    t = yl[i];  yl[i] = yl[j];  yl[j] = t;
	}
    }
    for (k = 1; k < n; k = k2) {    /* translation */
	h = 0;  k2 = k + k;  d = n / k2;
	for (j = 0; j < k; j++) {
	    c = fft_tbl_long->st[h + n4];
	    s = fft_tbl_long->st[h];
	    for (i = j; i < n; i += k2) {
		ik = i + k;
		dx = s * yl[ik] + c * xl[ik];
		dy = c * yl[ik] - s * xl[ik];
		xl[ik] = xl[i] - dx;  xl[i] += dx;
		yl[ik] = yl[i] - dy;  yl[i] += dy;
	    }
	    h += d;
	}
    }

    for(w = 0; w < BLOCK_LEN_LONG; ++w){
	psy_stvar_long->fft_r[ch][w+psy_stvar_long->p_fft[ch]] 
	    = sqrt(xl[w]*xl[w] + yl[w]*yl[w]);

	if( xl[w] > 0.0 ){
	    if( yl[w] >= 0.0 )
		psy_stvar_long->fft_f[ch][w+psy_stvar_long->p_fft[ch]] = atan( yl[w] / xl[w] );
	    else
		psy_stvar_long->fft_f[ch][w+psy_stvar_long->p_fft[ch]] = atan( yl[w] / xl[w] )+ M_PI * 2.0;
	} else if( xl[w] < 0.0 ) {
	    psy_stvar_long->fft_f[ch][w+psy_stvar_long->p_fft[ch]] = atan( yl[w] / xl[w] ) + M_PI;
	} else {
	    if( yl[w] > 0.0 )
		psy_stvar_long->fft_f[ch][w+psy_stvar_long->p_fft[ch]] = M_PI * 0.5;
	    else if( yl[w] < 0.0 )
		psy_stvar_long->fft_f[ch][w+psy_stvar_long->p_fft[ch]] = M_PI * 1.5;
	    else
		psy_stvar_long->fft_f[ch][w+psy_stvar_long->p_fft[ch]] = 0.0; /* tmp */
	}
    }

    free(xl);
    free(yl);


/* added by T. Araki (1997.10.16) */
    /* FFT for short */
    xl = (double *)malloc( sizeof(double) * BLOCK_LEN_SHORT * 2 );
    yl = (double *)malloc( sizeof(double) * BLOCK_LEN_SHORT * 2 );

    for(l = 0; l < MAX_SHORT_WINDOWS; ++l){

        /* window */        
        for(i = 0; i < BLOCK_LEN_SHORT*2; ++i){
	    xl[i] = fft_tbl_short->hw[i] * sample[ch][OFFSET_FOR_SHORT + BLOCK_LEN_SHORT * l + i];
	    yl[i] = 0.0;
	}

	n = BLOCK_LEN_SHORT*2;
	n4 = n/4;

	for (i = 0; i < n; ++i) {    /* bit inverse */
	    j = fft_tbl_short->brt[i];
	    if (i < j) {
	        t = xl[i];  xl[i] = xl[j];  xl[j] = t;
		t = yl[i];  yl[i] = yl[j];  yl[j] = t;
	    }
	}
	for (k = 1; k < n; k = k2) {    /* translation */
	    h = 0;  k2 = k + k;  d = n / k2;
	    for (j = 0; j < k; j++) {
	        c = fft_tbl_short->st[h + n4];
		s = fft_tbl_short->st[h];
		for (i = j; i < n; i += k2) {
		    ik = i + k;
		    dx = s * yl[ik] + c * xl[ik];
		    dy = c * yl[ik] - s * xl[ik];
		    xl[ik] = xl[i] - dx;  xl[i] += dx;
		    yl[ik] = yl[i] - dy;  yl[i] += dy;
		}
		h += d;
	    }
	}

	for(w = 0; w < BLOCK_LEN_SHORT; w++){
	    psy_stvar_short->fft_r[l][w] 
	        = sqrt(xl[w]*xl[w] + yl[w]*yl[w]);

	    if( xl[w] > 0.0 ){
	        if( yl[w] >= 0.0 )
		    psy_stvar_short->fft_f[l][w] = atan( yl[w] / xl[w] );
		else
		    psy_stvar_short->fft_f[l][w] = atan( yl[w] / xl[w] )+ M_PI * 2.0;
	    } else if( xl[w] < 0.0 ) {
	        psy_stvar_short->fft_f[l][w] = atan( yl[w] / xl[w] ) + M_PI;
	    } else {
	        if( yl[w] > 0.0 )
		    psy_stvar_short->fft_f[l][w] = M_PI * 0.5;
		else if( yl[w] < 0.0 )
		    psy_stvar_short->fft_f[l][w] = M_PI * 1.5;
		else
		    psy_stvar_short->fft_f[l][w] = 0.0; /* tmp */
	    }
	}
    }

    free(xl);
    free(yl);
/* added by T. Araki (1997.10.16) end */
}

void psy_step3(PSY_STATVARIABLE_LONG *psy_stvar_long, 
               PSY_STATVARIABLE_SHORT *psy_stvar_short, 
               PSY_VARIABLE_LONG *psy_var_long, 
               PSY_VARIABLE_SHORT *psy_var_short, 
               int ch
	       )
{
    int w,i;
    int p1_l,p2_l;

    p1_l = psy_stvar_long->p_fft[ch] - BLOCK_LEN_LONG;
    if( p1_l < 0 ) p1_l = BLOCK_LEN_LONG * 2;
    p2_l = p1_l - BLOCK_LEN_LONG;
    if( p2_l < 0 ) p2_l = BLOCK_LEN_LONG * 2;

    for(w = 0; w < BLOCK_LEN_LONG; ++w){
	psy_var_long->r_pred[w] = 2.0 * psy_stvar_long->fft_r[ch][p1_l + w] - psy_stvar_long->fft_r[ch][p2_l + w];
	psy_var_long->f_pred[w] = 2.0 * psy_stvar_long->fft_f[ch][p1_l + w] - psy_stvar_long->fft_f[ch][p2_l + w];
    }

/* added by T. Araki (1997.10.16) */
    for(w = 0; w < BLOCK_LEN_SHORT; ++w){
        psy_var_short->r_pred[0][w] = 2.0 * psy_stvar_short->last7_fft_r[ch][w] - psy_stvar_short->last6_fft_r[ch][w];
        psy_var_short->f_pred[0][w] = 2.0 * psy_stvar_short->last7_fft_f[ch][w] - psy_stvar_short->last6_fft_f[ch][w];
        psy_var_short->r_pred[1][w] = 2.0 * psy_stvar_short->fft_r[0][w] - psy_stvar_short->last7_fft_r[ch][w];
        psy_var_short->f_pred[1][w] = 2.0 * psy_stvar_short->fft_f[0][w] - psy_stvar_short->last7_fft_f[ch][w];
    }

    for(i = 2; i < MAX_SHORT_WINDOWS; ++i){
        for(w = 0; w < BLOCK_LEN_SHORT; ++w){
	    psy_var_short->r_pred[i][w] = 2.0 * psy_stvar_short->fft_r[i - 1][w] - psy_stvar_short->fft_r[i - 2][w];
	    psy_var_short->f_pred[i][w] = 2.0 * psy_stvar_short->fft_f[i - 1][w] - psy_stvar_short->fft_f[i - 2][w];
	}
    }

    for(w = 0; w < BLOCK_LEN_SHORT; ++w){
        psy_stvar_short->last6_fft_r[ch][w] = psy_stvar_short->fft_r[6][w];
	psy_stvar_short->last6_fft_f[ch][w] = psy_stvar_short->fft_f[6][w];
        psy_stvar_short->last7_fft_r[ch][w] = psy_stvar_short->fft_r[7][w];
	psy_stvar_short->last7_fft_f[ch][w] = psy_stvar_short->fft_f[7][w];
    }
/* added by T. Araki (1997.10.16) end */
}

void psy_step4(PSY_STATVARIABLE_LONG *psy_stvar_long,
               PSY_STATVARIABLE_SHORT *psy_stvar_short,
	       PSY_VARIABLE_LONG *psy_var_long, 
	       PSY_VARIABLE_SHORT *psy_var_short,
	       int ch
	       )
{
    int w,i;
    double r,f,rp,fp;

    for(w = 0; w < BLOCK_LEN_LONG; ++w){
	r = psy_stvar_long->fft_r[ch][psy_stvar_long->p_fft[ch]+w];
	f = psy_stvar_long->fft_f[ch][psy_stvar_long->p_fft[ch]+w];
	rp = psy_var_long->r_pred[w];
	fp = psy_var_long->f_pred[w];
	
	if( r + fabs(rp) != 0.0 )
	    psy_var_long->c[w] = sqrt( psy_sqr(r*cos(f) - rp*cos(fp))
				      +psy_sqr(r*sin(f) - rp*sin(fp)) )/ ( r + fabs(rp) ) ;
	else
	    psy_var_long->c[w] = 0.0; /* tmp */
    }

/* added by T. Araki (1997.10.16) */
    for(i = 0; i < MAX_SHORT_WINDOWS; ++i){
        for(w = 0; w < BLOCK_LEN_SHORT; ++w){
	    r = psy_stvar_short->fft_r[i][w];
	    f = psy_stvar_short->fft_f[i][w];
	    rp = psy_var_short->r_pred[i][w];
	    fp = psy_var_short->f_pred[i][w];
	
	    if( r + fabs(rp) != 0.0 )
	        psy_var_short->c[i][w] = sqrt( psy_sqr(r*cos(f) - rp*cos(fp))
				      +psy_sqr(r*sin(f) - rp*sin(fp)) )/ ( r + fabs(rp) ) ;
	    else
	        psy_var_short->c[i][w] = 0.0; /* tmp */
	}
    }
/* added by T. Araki (1997.10.16) end */
}

void psy_step5(PARTITION_TABLE_LONG *part_tbl_long, 
	       PARTITION_TABLE_SHORT *part_tbl_short, 
	       PSY_STATVARIABLE_LONG *psy_stvar_long,
               PSY_STATVARIABLE_SHORT *psy_stvar_short,
	       PSY_VARIABLE_LONG *psy_var_long, 
	       PSY_VARIABLE_SHORT *psy_var_short,
	       int ch
	       )
{
    int b,w,i;
    double tmp_cb;

    for(b = 0; b < part_tbl_long->len; ++b){
	psy_var_long->e[b] = 0.0;
	tmp_cb = 0.0;

/* added by T. Araki (1997.10.16) */
	for(w = part_tbl_long->w_low[b]; w <= part_tbl_long->w_high[b]; ++w){
	    psy_var_long->e[b] += psy_sqr(psy_stvar_long->fft_r[ch][psy_stvar_long->p_fft[ch]+w]);
	    tmp_cb += psy_sqr(psy_stvar_long->fft_r[ch][psy_stvar_long->p_fft[ch]+w]) * psy_var_long->c[w]; 
	}
/* added by T. Araki (1997.10.16) end */

	psy_var_long->c[b] = tmp_cb;
    }

/* added by T. Araki (1997.10.16) */
    for(i = 0; i < MAX_SHORT_WINDOWS; ++i){
        for(b = 0; b < part_tbl_short->len; ++b){
	    psy_var_short->e[i][b] = 0.0;
	    tmp_cb = 0.0;

	    for(w = part_tbl_short->w_low[b]; w <= part_tbl_short->w_high[b]; ++w){
	        psy_var_short->e[i][b] += psy_sqr(psy_stvar_short->fft_r[i][w]);
		tmp_cb += psy_sqr(psy_stvar_short->fft_r[i][w]) * psy_var_short->c[i][w]; 
	    }

	    psy_var_short->c[i][b] = tmp_cb;
	}
    }
/* added by T. Araki (1997.10.16) end */

}

void psy_step6(PARTITION_TABLE_LONG *part_tbl_long, 
	       PARTITION_TABLE_SHORT *part_tbl_short, 
	       PSY_VARIABLE_LONG *psy_var_long, 
	       PSY_VARIABLE_SHORT *psy_var_short
	       )
{
    int b,bb,i;
    double ecb,ct;
    double sprd;

    for(b = 0; b < part_tbl_long->len; ++b){
	ecb = 0.0;
	ct = 0.0;
	for(bb = 0; bb < part_tbl_long->len; ++bb){
	    sprd = sprdngf(part_tbl_long->bval[bb],part_tbl_long->bval[b]);
	    ecb += psy_var_long->e[bb] * sprd;
	    ct += psy_var_long->c[bb] * sprd;
	}
	if (ecb!=0.0) {
	  psy_var_long->cb[b] = ct / ecb;
	} else {
	  psy_var_long->cb[b] = 0.0;
	}
	psy_var_long->en[b] = ecb * part_tbl_long->rnorm[b];
    }

/* added by T. Araki (1997.10.16) */
    for(i = 0; i < MAX_SHORT_WINDOWS; ++i){ 
        for(b = 0; b < part_tbl_short->len; ++b){
	    ecb = 0.0;
	    ct = 0.0;
	    for(bb = 0; bb < part_tbl_short->len; ++bb){
	        sprd = sprdngf(part_tbl_short->bval[bb],part_tbl_short->bval[b]);
		ecb += psy_var_short->e[i][bb] * sprd;
		ct += psy_var_short->c[i][bb] * sprd;
	    }
	    if (ecb!=0.0) {	
	      psy_var_short->cb[i][b] = ct / ecb;
	    } else {
	      psy_var_short->cb[i][b] = 0.0;
	    }
	    psy_var_short->en[i][b] = ecb * part_tbl_short->rnorm[b];
	}
    }
/* added by T. Araki (1997.10.16) end */
}

void psy_step7(PARTITION_TABLE_LONG *part_tbl_long,
	       PARTITION_TABLE_SHORT *part_tbl_short,
	       PSY_VARIABLE_LONG *psy_var_long, 
	       PSY_VARIABLE_SHORT *psy_var_short)
{
    int b,i;

    for(b = 0; b < part_tbl_long->len; ++b){
      if (psy_var_long->cb[b]>0.0) {
	psy_var_long->tb[b] = -0.299 - 0.43 * log( psy_var_long->cb[b] );
      } else {
	psy_var_long->tb[b] = 1.0;
      }
      if( psy_var_long->tb[b] > 1.0 ) psy_var_long->tb[b] = 1.0;
      else if( psy_var_long->tb[b] < 0.0 ) psy_var_long->tb[b] = 0.0;
    }


/* added by T. Araki (1997.10.16) */
    for(i = 0;  i < MAX_SHORT_WINDOWS; ++i){
        for(b = 0; b < part_tbl_short->len; ++b){
	  if (psy_var_short->cb[i][b]>0.0) {
	    psy_var_short->tb[i][b] = -0.299 - 0.43 * log( psy_var_short->cb[i][b] );
	  } else {
	    psy_var_short->tb[i][b] = 1.0;
	  }
	  if( psy_var_short->tb[i][b] > 1.0 ) psy_var_short->tb[i][b] = 1.0;
	  else if( psy_var_short->tb[i][b] < 0.0 ) psy_var_short->tb[i][b] = 0.0;
	}
    }
/* added by T. Araki (1997.10.16) end */
}

void psy_step8(PARTITION_TABLE_LONG *part_tbl_long,
	       PARTITION_TABLE_SHORT *part_tbl_short,
	       PSY_VARIABLE_LONG *psy_var_long, 
	       PSY_VARIABLE_SHORT *psy_var_short)
{
    int b,i;
    double tmn = 18.0,nmt = 6.0;
    
    for(b = 0; b < part_tbl_long->len; ++b)
	psy_var_long->snr[b] = psy_var_long->tb[b] * tmn + (1.0 - psy_var_long->tb[b] ) * nmt ;

/* added by T. Araki (1997.10.16) */
    for(i = 0;  i < MAX_SHORT_WINDOWS; ++i){
        for(b = 0; b < part_tbl_short->len; ++b)
	    psy_var_short->snr[i][b] = psy_var_short->tb[i][b] * tmn + (1.0 - psy_var_short->tb[i][b] ) * nmt ;
    }
/* added by T. Araki (1997.10.16) end */

}    

void psy_step9(PARTITION_TABLE_LONG *part_tbl_long,
	       PARTITION_TABLE_SHORT *part_tbl_short,
	       PSY_VARIABLE_LONG *psy_var_long, 
	       PSY_VARIABLE_SHORT *psy_var_short
	       )
{
    int b,i;
    
    for(b = 0; b < part_tbl_long->len; ++b)
	psy_var_long->bc[b] = pow(10.0, -psy_var_long->snr[b]/10.0);

/* added by T. Araki (1997.10.16) */
    for(i = 0;  i < MAX_SHORT_WINDOWS; ++i){
        for(b = 0; b < part_tbl_short->len; ++b)
	    psy_var_short->bc[i][b] = pow(10.0, -psy_var_short->snr[i][b]/10.0);
    }
/* added by T. Araki (1997.10.16) end */
}

void psy_step10(PARTITION_TABLE_LONG *part_tbl_long,
		PARTITION_TABLE_SHORT *part_tbl_short,
		PSY_STATVARIABLE_LONG *psy_stvar_long, 
		PSY_STATVARIABLE_SHORT *psy_stvar_short, 
		PSY_VARIABLE_LONG *psy_var_long, 
		PSY_VARIABLE_SHORT *psy_var_short,
		int ch
		)
{
    int b,i;
    
    psy_stvar_long->p_nb[ch] += NPART_LONG;

    if( psy_stvar_long->p_nb[ch] == NPART_LONG*2 ) psy_stvar_long->p_nb[ch] = 0;

    for(b = 0; b < part_tbl_long->len; ++b){
	psy_stvar_long->nb[ch][psy_stvar_long->p_nb[ch] + b]
	    = psy_var_long->en[b] * psy_var_long->bc[b];
    }

/* added by T. Araki (1997.10.16) */
    for(i = 0;  i < MAX_SHORT_WINDOWS; ++i){
        for(b = 0; b < part_tbl_short->len; ++b){
	    psy_stvar_short->nb[i][b]
	        = psy_var_short->en[i][b] * psy_var_short->bc[i][b];
	}
    }
/* added by T. Araki (1997.10.16) end */
}

void psy_step11(PARTITION_TABLE_LONG *part_tbl_long, 
		PARTITION_TABLE_SHORT *part_tbl_short, 
		PSY_STATVARIABLE_LONG *psy_stvar_long, 
		PSY_STATVARIABLE_SHORT *psy_stvar_short, 
		int ch
		)
{
    int b,i;
    int p1,p2;
    double f;

    p1 = psy_stvar_long->p_nb[ch];
    if( p1 == 0 ) p2 = NPART_LONG;
    else if( p1 == NPART_LONG ) p2 = 0;

    for(b = 0; b < part_tbl_long->len; ++b){
	f = part_tbl_long->sampling_rate 
	    * (double)(part_tbl_long->w_low[b] + part_tbl_long->w_high[b])
		/ 4.0 / (double)(part_tbl_long->w_high[part_tbl_long->len - 1] + 1);

	psy_stvar_long->nb[ch][p1+b]
#if 0 /*  to use Itaru Kaneko's absolute threshold function */
	    = psy_max( pow(10.0,psy_get_absthr(f)/10.0)/32767.0 * part_tbl_long->width[b],
		       psy_min( psy_stvar_long->nb[ch][p1+b],2.0*psy_stvar_long->nb[ch][p2+b])
		      ); 
#endif
#if 1 /* to use ISO/IEC 13818-7 model*/
	    = psy_max( pow(10.0,part_tbl_long->qsthr[b]/10.0)/32767.0 ,
		       psy_min( psy_stvar_long->nb[ch][p1+b],2.0*psy_stvar_long->nb[ch][p2+b])
		      );
#endif
    }

/* added by T. Araki (1997.10.16) */
    for(b = 0; b < part_tbl_short->len; ++b){
	f = part_tbl_short->sampling_rate 
	    * (double)(part_tbl_short->w_low[b] + part_tbl_short->w_high[b])
		/ 4.0 / (double)(part_tbl_short->w_high[part_tbl_short->len - 1] + 1);

	psy_stvar_short->nb[0][b]
#if 0 /*  to use Itaru Kaneko's absolute threshold function */
	    = psy_max( pow(10.0,psy_get_absthr(f)/10.0)/32767.0 * part_tbl_short->width[b],
		       psy_min( psy_stvar_short->nb[0][b],psy_stvar_short->last7_nb[ch][b])
		      ); 
#endif
#if 1 /* to use ISO/IEC 13818-7 model*/
	    = psy_max( pow(10.0,part_tbl_short->qsthr[b]/10.0)/32767.0 ,
		       psy_min( psy_stvar_short->nb[0][b],psy_stvar_short->last7_nb[ch][b])
		      );
#endif
    }

    for(i = 1;  i < MAX_SHORT_WINDOWS; ++i){
        for(b = 0; b < part_tbl_short->len; ++b){
	    f = part_tbl_short->sampling_rate 
	        * (double)(part_tbl_short->w_low[b] + part_tbl_short->w_high[b])
	            / 4.0 / (double)(part_tbl_short->w_high[part_tbl_short->len - 1] + 1);

	    psy_stvar_short->nb[i][b]
#if 0 /*  to use Itaru Kaneko's absolute threshold function */
	        = psy_max( pow(10.0,psy_get_absthr(f)/10.0)/32767.0 * part_tbl_short->width[b],
			   psy_min( psy_stvar_short->nb[i][b],psy_stvar_short->nb[i - 1][b])
			   ); 
#endif
#if 1 /* to use ISO/IEC 13818-7 model*/
	        = psy_max( pow(10.0,part_tbl_short->qsthr[b]/10.0)/32767.0 ,
			   psy_min( psy_stvar_short->nb[i][b],psy_stvar_short->nb[i - 1][b])
			   );
#endif
        }
    }

    for(b = 0; b < part_tbl_short->len; ++b){
        psy_stvar_short->last7_nb[ch][b] = psy_stvar_short->nb[7][b];
    }
/* added by T. Araki (1997.10.16) end */

}
/* added by T. Okada (1997.7.10) end */

/* added by T. Araki (1997.7.10) */
void psy_step12(PARTITION_TABLE_LONG *part_tbl_long, 
	        PARTITION_TABLE_SHORT *part_tbl_short,
		PSY_STATVARIABLE_LONG *psy_stvar_long,
		PSY_STATVARIABLE_SHORT *psy_stvar_short, 
	        PSY_VARIABLE_LONG *psy_var_long, 
	        PSY_VARIABLE_SHORT *psy_var_short,
	        int ch
		)
{
    int b;

    psy_var_long->pe = 0.0;
    for(b = 0; b < part_tbl_long->len; ++b){
	psy_var_long->pe -= part_tbl_long->width[b]
	    * log10(psy_stvar_long->nb[ch][psy_stvar_long->p_nb[ch] + b]
	            / (psy_var_long->e[b] + 0.0000000001)); 

    }

}

void psy_step13(PSY_VARIABLE_LONG *psy_var_long, 
	        enum WINDOW_TYPE *block_type,
		int ch
		)
{

#define SWITCH_PE -1000 
          /* SWITCH_PE should be implementation dependent */

  if(psy_var_long->pe > SWITCH_PE) {
        *block_type = ONLY_SHORT_WINDOW;
  } else {
        *block_type = ONLY_LONG_WINDOW;
  }
}

void psy_step14(SR_INFO *p_sri,
		PARTITION_TABLE_LONG *part_tbl_long, 
	        PARTITION_TABLE_SHORT *part_tbl_short,
		PSY_STATVARIABLE_LONG *psy_stvar_long,
		PSY_STATVARIABLE_SHORT *psy_stvar_short, 
	        PSY_VARIABLE_LONG *psy_var_long, 
	        PSY_VARIABLE_SHORT *psy_var_short,
	        int ch
		)
{
    int b, n, w, i;
    int w_low, w_high;
    double thr, minthr;
    
    w_high = 0;
    for(n = 0; n < p_sri->num_cb_long; ++n){
	w_low = w_high;
	w_high += p_sri->cb_width_long[n];

        psy_var_long->epart[n] = 0.0;
	for(w = w_low; w < w_high; ++w){
	    psy_var_long->epart[n]
	      += psy_stvar_long->fft_r[ch][psy_stvar_long->p_fft[ch] + w]
	      * psy_stvar_long->fft_r[ch][psy_stvar_long->p_fft[ch] + w];
	}
    }

    for(b = 0; b < part_tbl_long->len; ++b){
        thr = psy_stvar_long->nb[ch][psy_stvar_long->p_nb[ch] + b]
	        / part_tbl_long->width[b];
        for(w = part_tbl_long->w_low[b]; w <= part_tbl_long->w_high[b]; ++w){
	    psy_var_long->thr[w] = thr;
	}
    }
    w_high = 0;
    for(n = 0; n < p_sri->num_cb_long; ++n){
        w_low = w_high;
	w_high += p_sri->cb_width_long[n];

        minthr = psy_var_long->thr[w_low];
	for(w = w_low; w < w_high; ++w){
	    if(psy_var_long->thr[w] < minthr){
	        minthr = psy_var_long->thr[w];
	    }
	}

	psy_var_long->npart[n] = minthr * (w_high - w_low);
    }

    for(n = 0; n < p_sri->num_cb_long; ++n){
      if (psy_var_long->epart[n]!=0.0) {
	psy_stvar_long->ismr[ch][n] = psy_var_long->npart[n] / psy_var_long->epart[n];
      } else {
	psy_stvar_long->ismr[ch][n] = 0.0;
      }
    }

/* added by T. Araki (1997.10.16) */
    for(i = 0; i < MAX_SHORT_WINDOWS; ++i){
        w_high = 0;
	for(n = 0; n < p_sri->num_cb_short; ++n){
	    w_low = w_high;
	    w_high += p_sri->cb_width_short[n];

	    psy_var_short->epart[i][n] = 0.0;
	    for(w = w_low; w < w_high; ++w){
	        psy_var_short->epart[i][n]
		    += psy_stvar_short->fft_r[i][w]
		    * psy_stvar_short->fft_r[i][w];
	    }
	}

	for(b = 0; b < part_tbl_short->len; ++b){
            thr = psy_stvar_short->nb[i][b] / part_tbl_short->width[b];
	    for(w = part_tbl_short->w_low[b]; w <= part_tbl_short->w_high[b]; ++w){
	        psy_var_short->thr[i][w] = thr;
	    }
	}

	w_high = 0;
	for(n = 0; n < p_sri->num_cb_short; ++n){
            w_low = w_high;
	    w_high += p_sri->cb_width_short[n];

	    minthr = psy_var_short->thr[i][w_low];
	    for(w = w_low + 1; w < w_high; ++w){
	        if(psy_var_short->thr[i][w] < minthr){
	            minthr = psy_var_short->thr[i][w];
	        }
	    }

	    psy_var_short->npart[i][n] = minthr * (w_high - w_low);
        }

	for(n = 0; n < p_sri->num_cb_short; ++n){
	  if (psy_var_short->epart[i][n]!=0.0) {
	    psy_stvar_short->ismr[ch][i][n] = psy_var_short->npart[i][n] / psy_var_short->epart[i][n];
	  } else {
	    psy_stvar_short->ismr[ch][i][n] = 0.0;
	  }
	}
    }
/* added by T. Araki (1997.10.16) end */

}
/* added by T. Araki (1997.7.10) end */
#ifdef I2R_LOSSLESS
void EncTf_settable(int sampling_rate,CH_PSYCH_OUTPUT p_chpo_long[])
{

    SR_INFO         *p_sri = &sr_info_aac[0];
  while( p_sri->sampling_rate != (long)sampling_rate ) {
    if( p_sri->sampling_rate == -1 ) {
      CommonExit( 1, "Encode: unknown sampling rate in psychoacoustic module %f", sampling_rate );
    }
    p_sri++;
  }
  p_chpo_long->cb_width = p_sri->cb_width_long;
  p_chpo_long->no_of_cb = p_sri->num_cb_long;
}


void EncTf_settable_short(int sampling_rate,CH_PSYCH_OUTPUT p_chpo_short[][MAX_SHORT_WINDOWS])
{


    SR_INFO         *p_sri = &sr_info_aac[0];
    int i;
  while( p_sri->sampling_rate != (long)sampling_rate ) {
    if( p_sri->sampling_rate == -1 ) {
      CommonExit( 1, "Encode: unknown sampling rate in psychoacoustic module %f", sampling_rate );
    }
    p_sri++;
  }
  /*p_chpo_short->cb_width = p_sri->cb_width_short;
  p_chpo_short->no_of_cb = p_sri->num_cb_short;*/
  for( i=0; i<MAX_SHORT_WINDOWS; i++ ) {
    p_chpo_short[i]->cb_width = p_sri->cb_width_short;
    p_chpo_short[i]->no_of_cb = p_sri->num_cb_short;
  }

}
#endif


